package edu.northwestern.cs.aqualab.Cattle;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.net.URL;
import java.net.URLConnection;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import javax.net.SocketFactory;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocketFactory;

import edu.northwestern.cs.aqualab.Cattle.json.JSONArray;
import edu.northwestern.cs.aqualab.Cattle.json.JSONException;
import edu.northwestern.cs.aqualab.Cattle.json.JSONObject;

/**
 * A Report (informally, a Cow) represents a single JSON report. Populate it by
 * calling getJSONList() and adding your data. To send it on its way, call
 * send(). (This method is synchronous to simplify error handling. Deal with
 * threading yourself.)<br>
 * <br>
 * setDestinationHost() and setResource() must be called before send(), but
 * setDestinationPort() is optional (the default destination port is 80).<br>
 * <br>
 * If you have one Report, you're likely to want more with similar
 * configurations. The instance method clear() helps you reuse a Report by
 * wiping all key/value pairs stored in its backing JSONObject. Alternatively,
 * the instance method clone() creates new Reports copied from a prototype.
 */
public class HttpsChannel implements Serializable {

	private final static Logger LOGGER = Logger.getLogger("edu.northwestern.dasu");
	private static final long serialVersionUID = 1L;

    private static ArrayList<KeyStore> TRUSTED_KEYSTORES = new ArrayList<KeyStore>();
    private static SocketFactory SSL_FACTORY = SSLSocketFactory.getDefault();

    private long last_attempt = -1;
    private int num_attempts = 0;
    private String dest_host;
    private int dest_port = 443;
    private String resource;
    private List<JSONObject> beefs;


    /**
     * Attempt to add a KeyStore to the trusted list and reinitialize the
     * SSLSocketFactory. Restores the old factory on failure.
     *
     * @param ks
     *            The new KeyStore.
     * @return True if the new KeyStore was added successfully, false otherwise.
     */
    public static synchronized boolean addKeyStore(KeyStore ks) {
        TRUSTED_KEYSTORES.add(ks);
        SocketFactory old_factory = SSL_FACTORY;

        try {
            SSL_FACTORY = new SelfSignedSSLSocketFactory(TRUSTED_KEYSTORES);
        } catch (KeyManagementException kme) {
            SSL_FACTORY = old_factory;
            TRUSTED_KEYSTORES.remove(ks);
            return false;
        } catch (NoSuchAlgorithmException nsae) {
            SSL_FACTORY = old_factory;
            TRUSTED_KEYSTORES.remove(ks);
            return false;
        }

        return true;
    }

    public HttpsChannel(String hostname, Integer port, String resource) {

    	this.setDestinationHost(hostname);
    	this.setDestinationPort(port);
    	this.setResource(resource);
    	
//    	HostnameVerifier allHostsValid = new HostnameVerifier() {	    		
//    		public boolean verify(String arg0, SSLSession arg1) {
//    			return true;
//    		}				
//    	};
//    	HttpsURLConnection.setDefaultHostnameVerifier(allHostsValid);
    	
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append(this.dest_host);
        sb.append(":");
        sb.append(String.valueOf(this.dest_port));
        sb.append(this.resource);
        return sb.toString();
    }


    /**
     * Must be called before send().
     */
    public void setDestinationHost(String dest) {
        this.dest_host = dest;
    }


    public void setDestinationPort(int port) {
        this.dest_port = port;
    }


    /**
     * Must be called before send().
     */
    public void setResource(String res) {
        this.resource = res;
    }


    /**
     * @return The timestamp, in millis, when the last call to GET() completed.
     */
    public long getLastAttempt() {
        return this.last_attempt;
    }


    /**
     * @return The number of times GET() has been called.
     */
    public int getNumAttempts() {
        return this.num_attempts;
    }

    public JSONObject GETData(String resource) {
    	
        JSONObject ret = null; 
        
    	try {
    		
    		LOGGER.info("CONFIG: Contacting " + this.dest_host + ":" + this.dest_port + "/" + this.resource + "/" + resource);
    		
    		URL url = new URL(this.dest_host + ":" + this.dest_port + "/" + this.resource + "/" + resource);
	    	HttpsURLConnection conn = (HttpsURLConnection)url.openConnection();
	    	conn.setSSLSocketFactory((SSLSocketFactory) SSL_FACTORY);

	    	InputStream inputstream = conn.getInputStream();
	    	InputStreamReader inputstreamreader = new InputStreamReader(inputstream);
	    	BufferedReader bufferedreader = new BufferedReader(inputstreamreader);
	
	    	String string = null;
	    	String res = new String();
	    	while ((string = bufferedreader.readLine()) != null) {	    	    
	    	    res = res.concat(string);
	    	}
	    	
	    	LOGGER.info("CONFIG: Received (" + res + ")");
	    	ret = new JSONObject( res );
	    	
    	}
    	catch (Exception e) {
    		//e.printStackTrace();
    		LOGGER.info("CONFIG: ERROR: " + e.getMessage());
    	}
    	
        this.last_attempt = System.currentTimeMillis();
        ++this.num_attempts;
    	return ret;
    }
    
    public boolean GETFile(String resource, String file) {
    	
    	boolean ret = true;
		int chunkSize=1024;
        InputStream inputstream = null;
        OutputStream outStream = null;
                
    	try {

    		URL url = new URL(this.dest_host + ":" + this.dest_port + "/" + this.resource + "/" + resource);
	    	HttpsURLConnection conn = (HttpsURLConnection)url.openConnection();
	    	conn.setSSLSocketFactory((SSLSocketFactory) SSL_FACTORY);
	    	inputstream = conn.getInputStream();	    	
			outStream = new BufferedOutputStream(new FileOutputStream(file));

			byte[] buf;
			int ByteRead;

			buf = new byte[chunkSize];
			while ((ByteRead = inputstream.read(buf)) != -1) {
				outStream.write(buf, 0, ByteRead);
			}

			LOGGER.info("CONFIG: Downloaded Successfully (" + url + ")");
	    	
    	}
    	catch (Exception e) {
			LOGGER.info("CONFIG: downloadFileFromURL: Timeout: " + 
					this.dest_host + ":" + this.dest_port + "/" + this.resource + "/" + resource);
			ret = false;
			
		} finally {
			try {

				if (inputstream != null)
					inputstream.close();
				if(outStream != null) {
					outStream.close();
				}
				
			} catch (IOException e) {
				LOGGER.warning("CONFIG: downloadFileFromURL: File not found!: " + file);
				ret = false;
			}
		}
    	
		this.last_attempt = System.currentTimeMillis();
        ++this.num_attempts;
		return ret;
    	
    }

    /**
     * Clear the JSONObjects associated with this Report.
     */
    public void clear() {
        this.beefs = new ArrayList<JSONObject>();
    }

    private void writeObject(ObjectOutputStream out) throws IOException {
        out.writeLong(serialVersionUID);
        out.writeLong(this.last_attempt);
        out.writeInt(this.num_attempts);
        out.writeObject(this.dest_host);
        out.writeInt(this.dest_port);
        out.writeObject(this.resource);
        out.writeInt(this.beefs.size());
        for (int i = 0; i < this.beefs.size(); ++i) {
            out.writeObject(this.beefs.get(i).toString());
        }
    }


    private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
        long serialVersionID = in.readLong();
        // I will make the assumption that we won't need more than INT_MAX
        // versions.
        switch ((int) serialVersionID) {
        case 1:
            this.last_attempt = in.readLong();
            this.num_attempts = in.readInt();
            this.dest_host = (String) in.readObject();
            this.dest_port = in.readInt();
            this.resource = (String) in.readObject();
            this.beefs = new ArrayList<JSONObject>();
            int num_beefs = in.readInt();
            try {
                // An exception here would indicate a bug in the JSON
                // implementation or in this serialization code.
                for (int i = 0; i < num_beefs; ++i) {
                    this.beefs.add(new JSONObject((String) in.readObject()));
                }
            } catch (JSONException jsone) {}
        }
    }
    
    public static void main(String[] args) {
    	
    	try {
	    	KeyStore ks = KeyStore.getInstance("JKS");
	    	FileInputStream is = new FileInputStream(new File("/Users/mas939/Mario's Stuff/NORTHWESTERN/Dropbox/workspace/ISP Interference/Dasu-Client/dist/dasu_keyStore.jks"));
	        ks.load(is, "fadazani".toCharArray());
	        HttpsChannel.addKeyStore(ks);
	        
	        HttpsChannel https = new HttpsChannel("https://mas939-imac.cs.northwestern.edu", 443, "config/data/dasu/dynamic/");
	    	https.GETData("UpdateConfig.php");
//	        HttpsChannel https = new HttpsChannel("https://google.com", 443, "");
//	    	https.GETData("");

    	}
    	catch (Exception e) {
    		e.printStackTrace();
    	}
        

    }
}
