/**
 * Dasu Project
 *
 * File:         Main.java
 * RCS:          $Id$
 * Description:  Main class (see below)
 * Author:       Mario Sanchez, Dave Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      May 24, 2010 at 2:25:01 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package edu.northwestern.dasu;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.NetworkInterface;
import java.net.Socket;
import java.net.SocketException;
import java.net.URL;
import java.net.URLConnection;
import java.net.UnknownHostException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Properties;
import java.util.Random;
import java.util.TimeZone;

import org.gudy.azureus2.core3.global.GlobalManager;
import org.gudy.azureus2.core3.logging.LogIDs;
import org.gudy.azureus2.plugins.PluginException;
import org.gudy.azureus2.plugins.PluginListener;
import org.gudy.azureus2.plugins.UnloadablePlugin;
import org.gudy.azureus2.plugins.config.ConfigParameter;
import org.gudy.azureus2.plugins.config.ConfigParameterListener;
import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.logging.LoggerChannel;
import org.gudy.azureus2.plugins.logging.LoggerChannelListener;
import org.gudy.azureus2.plugins.ui.UIInstance;
import org.gudy.azureus2.plugins.ui.UIManagerListener;
import org.gudy.azureus2.plugins.ui.config.BooleanParameter;
import org.gudy.azureus2.plugins.ui.model.BasicPluginConfigModel;
import org.gudy.azureus2.plugins.ui.model.BasicPluginViewModel;
import org.gudy.azureus2.plugins.update.UpdateCheckInstance;
import org.gudy.azureus2.plugins.update.UpdateManagerListener;
import org.gudy.azureus2.plugins.utils.Utilities;
import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloader;
import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloaderException;
import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloaderFactory;
import org.gudy.azureus2.pluginsimpl.local.utils.resourcedownloader.ResourceDownloaderFactoryImpl;
import org.gudy.azureus2.ui.swt.plugins.UISWTInstance;
import org.gudy.azureus2.ui.swt.plugins.UISWTViewEventListener;
import org.xbill.DNS.Record;

import edu.northwestern.cs.aqualab.Cattle.json.JSONObject;
import edu.northwestern.dasu.asn.NetworkAdminASNLookupImpl;
import edu.northwestern.dasu.database.CallbackReporter;
import edu.northwestern.dasu.database.DBReporterManager;
import edu.northwestern.dasu.database.DatabaseReporter;
import edu.northwestern.dasu.database.NullReporter;
import edu.northwestern.dasu.drools.ReportableFact;
import edu.northwestern.dasu.simulator.SimulatedDownload;
import edu.northwestern.dasu.stats.StatManager;
import edu.northwestern.dasu.stats.Statistics;
import edu.northwestern.dasu.time.SntpClient;
import edu.northwestern.dasu.timer.AzureusTimer;
import edu.northwestern.dasu.timer.ITimer;
import edu.northwestern.dasu.timer.ITimerEvent;
import edu.northwestern.dasu.timer.ITimerEventPerformer;
import edu.northwestern.dasu.ui.DasuView;
import edu.northwestern.dasu.util.DasuPluginInterface;
import edu.northwestern.dasu.util.LoggerFilter;
import edu.northwestern.dasu.util.PingManager;
import edu.northwestern.dasu.util.PluginInterface;
import edu.northwestern.dasu.util.Util;

import com.aelitis.azureus.core.impl.AzureusCoreImpl;

import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.SimpleFormatter;

/**
 * @author Mario Sanchez, John Otto, David Choffnes &lt;msanchez@eecs.northwestern.edu&gt;
 * 
 *         The Main class ...
 */
public class Main implements UnloadablePlugin {

	public static boolean DEBUG = false;
	public static boolean AME_DEBUG_SKIP_VERIFICATION = false;
	public static boolean AME_DEBUG_SKIP_DOWNLOAD = false;
	public static boolean RE_DEBUG_SKIP_VERIFICATION = false;
	public static boolean SKIP_UPDATE_UI_RULES = false;
	
	private final static java.util.logging.Logger LOGGER = java.util.logging.Logger.getLogger("edu.northwestern.dasu");
	private final static java.util.logging.Level LOGLEVEL = Level.INFO; //Level.WARNING; //Level.INFO //
    private final static java.util.logging.Filter LOGFILTER = new LoggerFilter(new String[]{"LOGFILTER"});
	//private final static java.util.logging.Filter LOGFILTER = new LoggerFilter(new String[]{"CONFIG", "AME", "RMI","UPDATER", "REPORTER", "RULEENGINE", "AMC", "DEBUG", "DYNAMIC", "STOP", "BROWSER_MODULE});
	//private final static java.util.logging.Filter LOGFILTER = new LoggerFilter(new String[]{"DYNAMIC", "BROWSER_MODULE", "UI", "CONFIG", "RULEENGINE", "CLASSLOADER", "UPDATER", "STOP"});
	
	private static Main self = null;
	
	/** time in seconds before azureus stops reporting stats */
	
	private static String publicIp;
	private static String localIp;
	private static String dasuDomain;
	private static boolean isShuttingDown = false;
	protected static ITimer shutdownTimer;
	protected static ITimer updateConfigTimer;
	protected static ITimer initializeConfigTimer;
	private static Random rand;	
	private static long uninstallRequest = 0;
	
	private static UISWTInstance swtInstance = null;
	private static UISWTViewEventListener myView = null;

	public static PluginInterface pi_az = null;
	
	private Utilities ut;
	private static BasicPluginViewModel model;
	private static LoggerChannel pluginLog;
	private ArrayList<URL> urls = new ArrayList<URL>();
	private static UIManagerListener uiListener;
	private static BasicPluginConfigModel config_model;
	private static LoggerChannelListener logListener;
	private static PluginListener pluginListener;
	private static PluginListener initializationCompleteListener;
	private BooleanParameter enableStatsParam;
	private BooleanParameter enableDasuParam;
	private BooleanParameter enablePopupsParam;
	private BooleanParameter enableRunExperimentsParam;
	public static boolean initialized = false;
	private static UpdateManagerListener updateListener;
	
	private static Thread dasuManager = null;

	/** downloader factory */
	private static ResourceDownloaderFactory rd_factory = ResourceDownloaderFactoryImpl.getSingleton();	
	
	public static boolean isSupportedPlatform() {
		Utilities ut = pi_az.getUtilities();
		return ut.isWindows() || ut.isOSX() || ut.isLinux();
	}
	
	public static void setLogFilter(String[] filterArgs) {	
				
		LoggerFilter lf = new LoggerFilter(filterArgs);
		LOGGER.setFilter(lf);
		DEBUG=true;
	}
		
	public Main() {
		
		self = this;
		LOGGER.setLevel(LOGLEVEL);	
		LOGGER.setFilter(LOGFILTER);		
		
		/*
		try {
			
			class myConsoleHandler extends ConsoleHandler {
				public myConsoleHandler() {
					super();
					this.setLevel(Level.FINE);
					this.setOutputStream(System.out);										
				}
			}
						
			Handler ch = new myConsoleHandler();
			LOGGER.setUseParentHandlers(false);
			LOGGER.addHandler(ch);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		*/
			
	}
	
	public static Main getInstance() {
		return self;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.gudy.azureus2.plugins.Plugin#initialize(org.gudy.azureus2.plugins
	 * .PluginInterface)
	 */
	public void initialize(org.gudy.azureus2.plugins.PluginInterface arg0) throws PluginException {
		
		pi_az = new DasuPluginInterface();
		((DasuPluginInterface) pi_az).setPluginInterface(arg0);
		((DasuPluginInterface) pi_az).dasuClassLoader = this.getClass().getClassLoader();
		
		pluginLog = pi_az.getLogger().getChannel("Dasu");

		/* register listeners to handle application shutdown */
		pluginListener = new PluginListener() {

			public void closedownInitiated() {
				try {
					unload();
				} catch (PluginException e) {
					e.printStackTrace();
				}
			}

			public void closedownComplete() {}
			public void initializationComplete() {}
		};
		pi_az.addListener(pluginListener);
		
		ut = pi_az.getUtilities();

		isShuttingDown = false;
		PingManager.activate();
		
		/* LOG */
		
		try {
			if ( Main.DEBUG) {								
				LOGGER.setUseParentHandlers(false);
				FileHandler fh = new FileHandler( Main.getDir() + File.separator + "debug.log");
				fh.setFormatter(new SimpleFormatter());
				LOGGER.addHandler(fh);				
			} 
		} catch (Exception e) {
			e.printStackTrace();
		} 
		
		
		/* GUI STUFF */

		ut.getLocaleUtilities().integrateLocalisedMessageBundle("edu.northwestern.dasu.messages");

		// TODO add more GUI options
		config_model = pi_az.getUIManager().createBasicPluginConfigModel("plugins", "Dasu.name");

		
		/*
		//enable detection
		enableDasuParam = config_model.addBooleanParameter2(
				"dasu.config.enable.detection", "dasu.config.enable.detection",
				DasuConfiguration.getInstance().isRecordStats());
		
		DasuData.getInstance().update("dasu.config.enable.detection", DasuConfiguration.getInstance().isRecordStats());
		
		enableDasuParam.addConfigParameterListener(new ConfigParameterListener() {
					
			public void configParameterChanged(ConfigParameter param) {
						
						DasuData.getInstance().update("dasu.config.enable.detection", enableDasuParam.getValue());

						if (!enableDasuParam.getValue()) {
							DasuManager.getInstance().stop();
						} else {
							startDasuManager();
						}
						
						if(!enableDasuParam.getValue()) {
							ArrayList<String> ll = new ArrayList<String>();
							ll.add("The Dasu engine is disabled.");
							ll.add("You can re-enable Dasu via the plugin configuration window.");
							DasuData.getInstance().update("dasu.disabled.reasons", ll);
							DasuData.getInstance().update("ui.state", "error");
						}

						DasuConfiguration.getInstance().updatePropertiesJson();
						
						//getReporter().insertConfigEventDasu(-1, "enable.detection=" + enableDasuParam.getValue(), Util.currentDasuTime());
						//Statistics.commitConfigEventDasu("enable.detection=" + enableDasuParam.getValue(), Util.currentDasuTime());
						ReportableFact rf = new ReportableFact("configEvent");
						rf.getContentsMap().put("timestamp", Util.currentDasuTime());
						rf.getContentsMap().put("event", "enable.detection="+enableDasuParam.getValue());
						Statistics.getInstance().addReportableFact(rf);


					}
				});
		 */
		
		/* enable reporting */
		enableStatsParam = config_model.addBooleanParameter2(
				"dasu.config.enable.report", "dasu.config.enable.report",
				DasuConfiguration.getInstance().isSendToDatabase());
		DasuData.getInstance().update("dasu.config.enable.report", DasuConfiguration.getInstance().isSendToDatabase());
		enableStatsParam
				.addConfigParameterListener(new ConfigParameterListener() {
					public void configParameterChanged(ConfigParameter param) {
						DasuData.getInstance().update("dasu.config.enable.report", enableStatsParam.getValue());
						if (enableStatsParam.getValue()) {	
							DasuConfiguration.getInstance().setSendToDatabase(enableStatsParam.getValue());
							
							//getReporter().insertConfigEventDasu(Statistics.getInstance().updateIdMap(), "enable.report=" + enableStatsParam.getValue(), Util.currentDasuTime());
							//Statistics.commitConfigEventDasu("enable.report=" + enableStatsParam.getValue(), Util.currentDasuTime());							
							ReportableFact rf = new ReportableFact("configEvent");
							rf.getContentsMap().put("timestamp", Util.currentDasuTime());
							rf.getContentsMap().put("event", "enable.report=" + enableStatsParam.getValue());
							Statistics.getInstance().addReportableFact(rf);

							// Re-register in the usage table
							StatManager.registerUsage();
						}
						else {
							
							//getReporter().insertConfigEventDasu(Statistics.getInstance().updateIdMap(), "enable.report=" + enableStatsParam.getValue(), Util.currentDasuTime());
							//Statistics.commitConfigEventDasu("enable.report=" + enableStatsParam.getValue(), Util.currentDasuTime());
							ReportableFact rf = new ReportableFact("configEvent");
							rf.getContentsMap().put("timestamp", Util.currentDasuTime());
							rf.getContentsMap().put("event", "enable.report=" + enableStatsParam.getValue());
							Statistics.getInstance().addReportableFact(rf);

							DasuConfiguration.getInstance().setSendToDatabase(enableStatsParam.getValue());							
						}
						DasuConfiguration.getInstance().updatePropertiesJson();
					}
				});
		
		/* enable popups */
		enablePopupsParam = config_model.addBooleanParameter2(
				"dasu.config.enable.popups", "dasu.config.enable.popups",
				DasuConfiguration.getInstance().isEnablePopups());
		DasuData.getInstance().update("dasu.config.enable.popups", DasuConfiguration.getInstance().isEnablePopups());
		enablePopupsParam
				.addConfigParameterListener(new ConfigParameterListener() {
					public void configParameterChanged(ConfigParameter param) {
						DasuConfiguration.getInstance().setEnablePopups(enablePopupsParam.getValue());
						DasuConfiguration.getInstance().updatePropertiesJson();
						DasuData.getInstance().update("dasu.config.enable.popups", enablePopupsParam.getValue());
						
						ReportableFact rf = new ReportableFact("configEvent");
						rf.getContentsMap().put("timestamp", Util.currentDasuTime());
						rf.getContentsMap().put("event", "enable.popups=" + enablePopupsParam.getValue());
						Statistics.getInstance().addReportableFact(rf);

						if (enablePopupsParam.getValue())
							DasuManager.getInstance().startMessageManager();
						else
							DasuManager.getInstance().stopMessageManager();
					}
				});
 
		/* enable AME */
		enableRunExperimentsParam = config_model.addBooleanParameter2(
				"dasu.config.enable.experiments", "dasu.config.enable.experiments", DasuConfiguration.getInstance().isRunExperiments());		
		DasuData.getInstance().update("dasu.config.enable.experiments", DasuConfiguration.getInstance().isRunExperiments());		
		enableRunExperimentsParam
			.addConfigParameterListener(new ConfigParameterListener() {
				public void configParameterChanged(ConfigParameter param) {
					
					DasuConfiguration.getInstance().setRunExperiments(enableRunExperimentsParam.getValue());
					DasuConfiguration.getInstance().updatePropertiesJson();
					DasuData.getInstance().update("dasu.runExperiments", enableRunExperimentsParam.getValue());
					
					ReportableFact rf = new ReportableFact("configEvent");
					rf.getContentsMap().put("timestamp", Util.currentDasuTime());
					rf.getContentsMap().put("event", "enable.runExperiments=" + enableRunExperimentsParam.getValue());
					Statistics.getInstance().addReportableFact(rf);
					
					//if (enableRunExperimentsParam.getValue())
					//	DasuManager.getInstance().startAME();
					//else
					if (!enableRunExperimentsParam.getValue())
						DasuManager.getInstance().stopAME();
					
				}
			});
		
		config_model.addLabelParameter2("dasu.config.privacynote");
		if (pi_az.getUtilities().compareVersions("2.5.0.2",
				pi_az.getAzureusVersion()) <= 0) {
			config_model.addHyperlinkParameter2("aqualab.teamSig",
					"http://aqualab.cs.northwestern.edu");
		} else {
			config_model.addLabelParameter2("aqualab.teamSig");
		}

		model = pi_az.getUIManager().createBasicPluginViewModel("Dasu Logs");

		model.getActivity().setVisible(false);
		model.getProgress().setVisible(false);

		uiListener = new UIManagerListener() {
			public void UIAttached(UIInstance instance) {
				if (instance instanceof UISWTInstance) {
					swtInstance = (UISWTInstance) instance;
					myView = new DasuView();
					swtInstance.addView(UISWTInstance.VIEW_MAIN, DasuView.VIEWID, myView);
					LOGGER.info("UI: UIAttached");
				}
			}

			public void UIDetached(UIInstance instance) {
				if (instance instanceof UISWTInstance) {
					swtInstance.removeViews(UISWTInstance.VIEW_MAIN,DasuView.VIEWID);
					swtInstance = null;
					LOGGER.info("UI: UIDetached");
				}
			}
		};

		pi_az.getUIManager().addUIListener(uiListener);

		
		updateListener = new UpdateManagerListener(){
			public void checkInstanceCreated(UpdateCheckInstance uci) {

				if (uci.getType() == UpdateCheckInstance.UCI_UNINSTALL)
					uninstallRequest = Util.currentTimeMillis();
			}
		};
		
		pi_az.getUpdateManager().addListener(updateListener);
		
		
		// print output to log window
		logListener = new LoggerChannelListener() {
			public void messageLogged(int type, String message) {
				model.getLogArea().appendText(message + "\n");
			}

			public void messageLogged(String str, Throwable error) {
				model.getLogArea().appendText(str + ": " + error.toString() + "\n");
			}
		};

		pluginLog.addListener(logListener);
		
		/* register listeners to handle application initialization */
		initializationCompleteListener = new PluginListener() {

			public void closedownComplete() {
			}

			public void closedownInitiated() {
			}
			
			public void initializationComplete() {
				
				startInitializeConfigTimer();

			}
		};
		
		pi_az.addListener(initializationCompleteListener);


	} // end initialize
	
	
	public static void startInitializeConfigTimer() {
		
		initializeConfigTimer = createTimer("DasuInitializeConfigTimer");
		initializeConfigTimer.addEvent(System.currentTimeMillis() + 1*1000, 
				new ITimerEventPerformer(){ 

				public void perform(ITimerEvent event) {
					
					getDasuTime();
					
					String tz = TimeZone.getDefault().getDisplayName(((TimeZone.getDefault().getDSTSavings() == 0)?false:true), TimeZone.LONG);
					long initUserUtc = Util.currentTimeMillis();
					long initNtpLocal = Util.currentDasuTime();
					long initNtpUtc = Util.currentGMTTime();
					
					LOGGER.info("CONFIG: currentMillis(): " + new Date(initUserUtc) + " " + initUserUtc);
					LOGGER.info("CONFIG: currentDasu(): " + new Date(initNtpLocal) + " " + initNtpLocal);
					LOGGER.info("CONFIG: currentGMT(): " + new Date(initNtpUtc) + " " + initNtpUtc);
					LOGGER.info("CONFIG: TimeZone(): " + tz);
					
					LOGGER.info("Initializing dasu version " + pi_az.getPluginVersion());

					String domain = getDomainName();
					DasuConfiguration.getInstance().initializeProperties(domain);
					
					/*
					if(enableDasuParam.getValue()) {
						startDasuManager();			
					} else {
						ArrayList<String> ll = new ArrayList<String>();
						ll.add("The Dasu engine is disabled.");
						ll.add("You can re-enable Dasu via the plugin configuration window.");
						DasuData.getInstance().update("dasu.disabled.reasons", ll);
						DasuData.getInstance().update("ui.state", "error");
					}
					*/

					logPlugin("Initializing Dasu version " + pi_az.getPluginVersion());

					createThread("DasuDBManager", DBReporterManager.getInstance());
					dasuManager = createThread("DasuManager", new DasuManager());
					
					initialized = true;
					
					startUpdateConfigTimer();
					
				}}
			);
	}
	
	public static void startUpdateConfigTimer() {
		
		updateConfigTimer = createTimer("DasuUpdateConfigTimer");
		updateConfigTimer.addPeriodicEvent(DasuConfiguration.getInstance().getUpdateCheckFreqSec()*1000, 
				new ITimerEventPerformer(){ 

				public void perform(ITimerEvent event) {
					
					JSONObject json = DasuManager.getInstance().getConfigService().GETData(DasuConfiguration.DASU_SCIPT_UPDATE_CONFIG + 
							DasuConfiguration.getInstance().getResourceScriptRequest());
					
					if (json != null) {
						
						if ( json.length() > 1) {
							LOGGER.info("CONFIG: Remote configuration update");
							DasuConfiguration.getInstance().updatePropertiesJson(json);
						}

					}
				}}
			);
	}

	/**
	 * @param downloader
	 * @param data
	 */
	protected void writeAndLoadFile(ResourceDownloader downloader,
			InputStream data) {
		File outputFile = new File(Main.getDir()
				+ File.separatorChar
				+ downloader.getName().substring(
						downloader.getName().lastIndexOf('/')));

		if (outputFile.exists()) {
			outputFile.delete();
		}

		FileWriter out;

		try {
			out = new FileWriter(outputFile);

			int c;

			while ((c = data.read()) != -1)
				out.write(c);

			out.flush();
			out.close();

			urls.add(outputFile.toURL());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#finalize()
	 */
	@Override
	protected void finalize() throws Throwable {
		unload();
	}

	
	public static void startUnloadConfigTimer() {
		
		initializeConfigTimer = createTimer("DasuInitializeConfigTimer");
		initializeConfigTimer.addEvent(System.currentTimeMillis() + 1*1000, 
				new ITimerEventPerformer(){ 

				public void perform(ITimerEvent event) {
					
					if (initialized) {
						
						if (Util.currentTimeMillis() - uninstallRequest < 10000) {
							//getReporter().insertConfigEventDasu(Statistics.updateIdMap(), "remove.dasu=true", Util.currentDasuTime());
							//Statistics.commitConfigEventDasu("remove.dasu=true", Util.currentDasuTime());
							ReportableFact rf = new ReportableFact("configEvent");
							rf.getContentsMap().put("timestamp", Util.currentDasuTime());
							rf.getContentsMap().put("event", "remove.dasu=true");
							Statistics.getInstance().addReportableFact(rf);
						}

						//Statistics.getInstance().flush();
						Main.startShutdown();
				
						try {

							try {
				
								dasuUnload();
								pluginLog.removeListener(logListener);
								pi_az.getUIManager().removeUIListener(uiListener);
								pi_az.getUpdateManager().removeListener(updateListener);
								pi_az.removeListener(pluginListener);
								pi_az.removeListener(initializationCompleteListener);
								swtInstance.removeViews(UISWTInstance.VIEW_MAIN,DasuView.VIEWID);
								//shutdownTimer.destroy();
								config_model.destroy();
								model.destroy();					
								myView = null;					
								updateConfigTimer.destroy();
								initializeConfigTimer.destroy();
				
							} catch (Exception e) {
								e.printStackTrace();
							}
				
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
					
				}}
			);
	}

	
	public void unload() throws PluginException{		
		
		startUnloadConfigTimer();
		/*
		if (initialized) {
			
			if (Util.currentTimeMillis() - uninstallRequest < 10000) {
				//getReporter().insertConfigEventDasu(Statistics.updateIdMap(), "remove.dasu=true", Util.currentDasuTime());
				//Statistics.commitConfigEventDasu("remove.dasu=true", Util.currentDasuTime());
				ReportableFact rf = new ReportableFact("configEvent");
				rf.getContentsMap().put("timestamp", Util.currentDasuTime());
				rf.getContentsMap().put("event", "remove.dasu=true");
				Statistics.getInstance().addReportableFact(rf);
			}

			//Statistics.getInstance().flush();
			Main.startShutdown();
	
			try {

				// catch all exceptions so it is sure to unload
				//Main.getReporter().setExtendedConnection(true);
	
				try {
	
					dasuUnload();
					pluginLog.removeListener(logListener);
					pi_az.getUIManager().removeUIListener(uiListener);
					pi_az.getUpdateManager().removeListener(updateListener);
					pi_az.removeListener(pluginListener);
					pi_az.removeListener(initializationCompleteListener);
					swtInstance.removeViews(UISWTInstance.VIEW_MAIN,DasuView.VIEWID);
					//shutdownTimer.destroy();
					config_model.destroy();
					model.destroy();					
					myView = null;					
					updateConfigTimer.destroy();
					unloadConfigTimer.destroy();
					initializeConfigTimer.destroy();
	
				} catch (Exception e) {
					e.printStackTrace();
				}
	
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		*/
	}
	
	
	private void startDasuManager() {
		
		try {
			dasuManager = createThread("DasuManager", new DasuManager());
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	
	public void restartDasuManager() {
		
		if(DasuManager.getInstance() != null && DasuManager.getInstance().isRunning()) {
			DasuManager.getInstance().stop();
			startDasuManager();
		}
	}

	/**
	 * 
	 */
	private static void getDasuTime() {
		long offset = -1;
		try {
			offset = -1 * SntpClient.getOffset("pool.ntp.org");
		} catch (SocketException e) {
			//e.printStackTrace();
		} catch (UnknownHostException e) {
			//e.printStackTrace();
		} catch (IOException e) {
			//e.printStackTrace();
		}

		Util.setDasuTimeOffset(offset);
	}

	public static String getPlatform() {

		String platform = null;
		if (Main.getPluginInterface().getUtilities().isWindows())
			platform = "windows";
		else if (Main.getPluginInterface().getUtilities().isOSX())
			platform = "OSX";
		else
			platform = "linux";
		
		return platform;

	}
	
	public static PluginInterface getPluginInterface() {
		return pi_az;
	}
	
	public static String getDir() {
		if ( pi_az != null )
			return pi_az.getPluginconfig().getPluginUserFile("").getAbsolutePath();
		return
			new File(".").getAbsolutePath();
	}
	
	public static GlobalManager getGlobalManager() {
		return AzureusCoreImpl.getSingleton().getGlobalManager();
	}
	
	public static void logPlugin(String text) {
		pluginLog.log("[" + new Date() + "]: " + text);
	}
	
	public static void logVuzeInfo(String text) {
		org.gudy.azureus2.core3.logging.Logger.log(new org.gudy.azureus2.core3.logging.LogEvent(LogIDs.PLUGIN, org.gudy.azureus2.core3.logging.LogEvent.LT_INFORMATION, "[Dasu] " + text));
	}

	public static void logVuzeWarning(String text) {
		org.gudy.azureus2.core3.logging.Logger.log(new org.gudy.azureus2.core3.logging.LogEvent(LogIDs.PLUGIN, org.gudy.azureus2.core3.logging.LogEvent.LT_WARNING, "[Dasu] " + text));
	}

	public static void logVuzeError(String text) {
		org.gudy.azureus2.core3.logging.Logger.log(new org.gudy.azureus2.core3.logging.LogEvent(LogIDs.PLUGIN, org.gudy.azureus2.core3.logging.LogEvent.LT_ERROR, "[Dasu] " + text));
	}

	public static String getPublicIpAddressFirstTime() {
		checkForLocalIpAddress();
		try {
			if (Main.getPluginInterface().getUtilities().getPublicAddress() != null) {
				NetworkAdminASNLookupImpl.getInstance().setAddress(
						Main.getPluginInterface().getUtilities().getPublicAddress());
				String ip = Main.getPluginInterface().getUtilities().getPublicAddress().toString().substring(1);

				publicIp = ip;
				return ip;
			} else {
				String s = getLocalPublicAddress();
				if (s.startsWith("192.168")) return null;
				publicIp = s;
				return s;
			}
		} catch (Exception e){
			LOGGER.warning(e.getMessage());
			return null;
		}
	}

	public static String getPublicIpAddress() {
				
		try {
			checkForLocalIpAddress();
			
			if (Main.getPluginInterface().getUtilities().getPublicAddress() != null) {
				NetworkAdminASNLookupImpl.getInstance().setAddress(
						Main.getPluginInterface().getUtilities().getPublicAddress());
				String ip = Main.getPluginInterface().getUtilities().getPublicAddress().toString().substring(1);
				
				if (DasuConfiguration.getInstance().getUUID() == null) {
					DasuConfiguration.getInstance().loadPropertiesJson();
				}

				if (publicIp!=null && !ip.equals(publicIp)){
					
					//Statistics.getInstance().reportIpChange(publicIp, ip, getLocalIPAddress().getHostAddress(), Util.currentGMTTime());
					ReportableFact rf = new ReportableFact("ipChange");
					rf.getContentsMap().put("oldIp", publicIp);
					rf.getContentsMap().put("newIp", ip);
					rf.getContentsMap().put("localIp", getLocalIPAddress().getHostAddress());
					rf.getContentsMap().put("timestamp", Util.currentGMTTime());
					Statistics.getInstance().addReportableFact(rf);
					
				} else if (publicIp==null && ip!=null && ip.length()>0){
					
					//Statistics.getInstance().reportIpChange(ip, ip, getLocalIPAddress().getHostAddress(), Util.currentGMTTime());					
					ReportableFact rf = new ReportableFact("ipChange");
					rf.getContentsMap().put("oldIp", ip);
					rf.getContentsMap().put("newIp", ip);
					rf.getContentsMap().put("localIp", getLocalIPAddress().getHostAddress());
					rf.getContentsMap().put("timestamp", Util.currentGMTTime());
					Statistics.getInstance().addReportableFact(rf);
					
				}
				publicIp = ip;
				return ip;
				
			} else {
				String s = getLocalPublicAddress();
				if (s.startsWith("192.168")) return null;
				publicIp = s;
				return s;
			}
		} catch (Exception e){
			return null;
		}
	}

	public static String getCurrentLocalIpAddress() {
		return localIp;
	}
	
	public static String getCurrentPublicIpAddress() {
		return publicIp;
	}

	private static void checkForLocalIpAddress(){
		InetAddress addresses[];
		try {
			addresses = InetAddress.getAllByName(InetAddress.getLocalHost().getHostName());
			for (InetAddress addr : addresses){
				if (addr.isSiteLocalAddress()) {
					NetworkInterface eth = NetworkInterface.getByInetAddress(addr);
					String interfaceName = getInterfaceName();
					if(eth != null && eth.getDisplayName().equals(interfaceName)){
						localIp = addr.getHostAddress();
					}
				}
			}
		} catch (UnknownHostException e) {
			e.printStackTrace();
		} catch (SocketException e) {
			e.printStackTrace();
		}
	}

	public static InetAddress getLocalIPAddress() {
				
		URL origin = null;
		String hostName= null;
		String strProtocol = null;
		Socket local = null;
		InetAddress objLocalHost = null;
		try {
		  origin = new URL("http://www.google.com"); //this is the Applet-class
		  hostName = origin.getHost();
		  strProtocol = origin.getProtocol();

		  if (origin.getPort() != -1)
		     local = new Socket(hostName, origin.getPort());
		  else
		  {
		     if (strProtocol.equalsIgnoreCase("http"))
		        local = new Socket(hostName, 80);

		     else if (strProtocol.equalsIgnoreCase("https"))
		        local = new Socket(hostName, 443);
		  }

		  objLocalHost = local.getLocalAddress();
		  return objLocalHost;
		  
		} catch (Exception ex) {
			//ex.printStackTrace();
			//pass
		} finally {
		  try {
			  if (local != null)
				  local.close();
		  } catch (IOException ioe) {
		     ioe.printStackTrace();
		  }
		}

		return objLocalHost;
	}
	
	private static String interfaceName = null;
	
	/* return eth0 by default if the interface cant' be found */
	public static String getInterfaceName() {

		if(interfaceName != null) return interfaceName;
		
		String ret = "eth0";
		try {
			NetworkInterface eth = NetworkInterface.getByInetAddress(getLocalIPAddress());
			if (eth != null)
				ret = eth.getDisplayName();

		} catch (Exception e) {
			interfaceName = "eth0";
			return "eth0";
		}

		interfaceName = ret;
		return ret;

	}

	/* return eth0 by default if the interface cant' be found */
	public static String getInterfaceMAC() {

		String ret = "eth0";
		try {
			NetworkInterface eth = NetworkInterface.getByInetAddress(InetAddress.getLocalHost());
			if (eth != null) {
				byte[] mac =eth.getHardwareAddress();  
				StringBuilder macaddress= new StringBuilder();  
				if (mac != null){
				for (int i = 0; i < mac.length; i++) {  
					macaddress.append(String.format("%02X%s", mac[i], (i < mac.length - 1) ? "-" : "")) ;  
				}  

				ret = macaddress.toString();  
				}
			}

		} catch (UnknownHostException e) {
			return "unk";
		} catch (SocketException e) {
			return "unk";
		}

		return ret;

	}

	private static String getLocalPublicAddress() throws UnknownHostException {
		InetAddress addresses[] = InetAddress.getAllByName(InetAddress.getLocalHost().getHostName());
		// TODO verify that this always picks the "right" IP address
		//s = addresses[addresses.length-1].getHostAddress();
		for (InetAddress addr : addresses){
			if (!addr.isSiteLocalAddress()) return addr.getHostAddress();
		}
		return "";
	}

	public static boolean isShuttingDown(){
		return isShuttingDown;
	}

	public static Random getRandom() {
		if (rand==null){
			rand = new Random();
		}
		return rand;
	}

	public static Thread createThread(String name, Runnable target) {

		//getPluginInterface().getUtilities().createThread(name, target);
		LOGGER.info("THREAD: " + "createThread: " + target.toString() + " " + target.getClass().getClassLoader());
		Thread t = new Thread(target);
		t.setDaemon(true);
		t.setName(name);
		if ( !isShuttingDown() ) {
			t.start();
		}
		return t;
	
	}

	public static DatabaseReporter getReporter() {
		
		if (DasuManager.USE_SIMULATION_V2) 
			return NullReporter.getInstance();

		return CallbackReporter.getInstance();
	}

	public static ITimer createTimer(String name) {
		return new AzureusTimer(Main.getPluginInterface().getUtilities().createTimer(name), name);
	}

	public static void dasuUnload(){

		//DasuDistributedCoordinator_AME
		//
		LOGGER.info("STOP: Stopping DasuView");
		DasuData.getInstance().getUi().stopDasuView();
		
		LOGGER.info("STOP: Stopping ping manager");
		PingManager.stop();

		try {
			LOGGER.info("STOP: Stopping stat manager");
			StatManager.stop();
		} catch (Exception e){
			e.printStackTrace();
		}

		LOGGER.info("STOP: Stopping statistics");
		Statistics.getInstance().stop();
		
		LOGGER.info("STOP: Stopping db manager");
		DBReporterManager.getInstance().stop();

		LOGGER.info("STOP: Stopping dasu manager");
		if(DasuManager.getInstance() != null)
			DasuManager.getInstance().stop();	
		
		DasuConfiguration.getInstance().updatePropertiesJson();
		
		LOGGER.info("STOP: done...");
				
	}

	protected static void startShutdown(){
		
		isShuttingDown = true;
		//why a shutdown timer?
		//shutdownTimer = createTimer("DasuShutdownTimer");
		//shutdownTimer.addEvent(Util.currentTimeMillis()+DasuConfiguration.getInstance().getReportingTimeout()*1000, 
		//		new ITimerEventPerformer(){
		//
		//	public void perform(ITimerEvent event) {
		//		isShuttingDown = true;
		//	}}
		//);
	}

	public static String getDomainName() {

		if ((dasuDomain != null) && (!dasuDomain.equals("default")))
			return dasuDomain;

		String myDomain = "default";
		String myIp = null;

		myIp = Main.getPublicIpAddressFirstTime();

		LOGGER.info("getPublicIpAddressFirstTime() => ip: " + myIp);


		if (myIp != null) {
			Record[] records = Util.doLookup(new String[]{"-x", myIp});
			if (records == null)
				return myDomain;

			String domain = null;
			if ( records.length > 0)
				domain = records[0].toString();

			if (domain != null) {

				String[] parts = domain.split("\\.");
				int length = parts.length;

				/* 
				 * If last part of domain name is 2 character's long, 
				 * then it contains CC top level domain
				 */
				if ((length > 3) && ( parts[length-1].length() < 3 ))
					myDomain = parts[length-3] + "." +
					parts[length-2] + "." + parts[length-1];

				else 
					myDomain = parts[length-2] + "." + parts[length-1];

			}
		}


		LOGGER.fine("IP " + myIp);
		LOGGER.fine("DOMAIN " + myDomain);
		
		dasuDomain = myDomain;
		
		return myDomain;

	}
	
	@Deprecated
	public static Properties downloadFromURL(String url, int timeout) {
		InputStream is = null;
		ResourceDownloader dl = null;

		Properties details = new Properties();
		try {
			dl = rd_factory.create(new URL(url));

			dl = rd_factory.getTimeoutDownloader(dl, timeout);

			// download the experiment list
			is = dl.download();
			details.load(is);
			dl.cancel();
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ResourceDownloaderException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {
			if (is != null) {
				try {
					is.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		return details;
	}

	@Deprecated
	public static boolean downloadFileFromURL(String url, String localFileName, String destinationDir) {
		return downloadFileFromURL(url, destinationDir + File.separator + localFileName);
	}
	
	@Deprecated
	public static boolean downloadFileFromURL(String url, String file) {

		int chunkSize=1024;
		URLConnection  uCon = null;
		OutputStream outStream = null;
		InputStream is = null;

		try {
			URL Url;
			byte[] buf;
			int ByteRead,ByteWritten=0;
			Url= new URL(url);
			outStream = new BufferedOutputStream(new
					FileOutputStream(file));

			uCon = Url.openConnection();

			is = uCon.getInputStream();
			buf = new byte[chunkSize];
			while ((ByteRead = is.read(buf)) != -1) {
				outStream.write(buf, 0, ByteRead);
				ByteWritten += ByteRead;
			}

			LOGGER.info("Downloaded Successfully (" + url + ")");


		} catch (Exception e) {
			LOGGER.info("downloadFileFromURL: Timeout: " + url);
			return false;

		} finally {
			try {

				if (is != null)
					is.close();
				if(outStream != null) {
					outStream.close();
				}
				
			} catch (IOException e) {
				LOGGER.warning("downloadFileFromURL: File not found!: " + file);
				return false;
			}
		}
		return true;

	}

//	public static void updateProperties() {
//		DasuConfiguration hc = DasuConfiguration.getInstance();
//		hc.writeProperties(props, getPropertiesLocations());
//	}
	
	public static HashMap<String, String> getPropertiesFromServer(String url) {

		Properties listProperties = null;
		HashMap<String, String> retList = new HashMap<String, String>();

		/* Obtain list of 'objects' to perform specified probe */
		while (listProperties == null || listProperties.isEmpty()) {

			LOGGER.info("DasuManager: Connecting to " + url);

			listProperties = Main.downloadFromURL(url, 30 * 1000);
			if (listProperties.isEmpty()) {
				LOGGER.warning("DasuManager: ERROR loading " + url);
				LOGGER.info("DasuManager: Retrying url in 60 seconds... ");
				try {
					Thread.sleep(60 * 1000);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

		if (listProperties != null) {

			for (Integer i = 1; listProperties.containsKey("key."
					+ i.toString()); i++) {
				String key = "key." + i.toString();
				String value = (String) listProperties.get(key);
				retList.put(key, value);
			}
		}

		return retList;
	}



	/*SERIALIZABLE OPERATIONS API*/

	public static void saveObject(Serializable object, String filename) {
		ObjectOutputStream objstream;
		try {
			objstream = new ObjectOutputStream(new FileOutputStream(filename));
			objstream.writeObject(object);
			objstream.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	} 

	/**
	 * Loads an object.
	 */
	public static Object loadObject(String filename) {
		
		ObjectInputStream objstream;
		Object object = null; 

		try {
			File f = new File(filename);
			if (f.isFile()) {
				objstream = new ObjectInputStream(new FileInputStream(f));
				object = objstream.readObject();
				objstream.close();				
			}
		} catch (FileNotFoundException e) {
			//e.printStackTrace();
		} catch (IOException e) {
			//e.printStackTrace();
			//when versionID for persistenData() class changed, then don't load,
			//ignore error and simply override file
		} catch (ClassNotFoundException e) {
			//e.printStackTrace();
		}

		return object;
	}

	public static int getId(Download d) {
		
		if ( d instanceof SimulatedDownload) {
			return ((SimulatedDownload)d).getDownloadId();
		}
		
		MessageDigest md;
		try {
			md = MessageDigest.getInstance("SHA");

			md.update(d.getName().getBytes());
			md.update(Util.convertLong(d.getCreationTime()));
			byte hash[] = md.digest();
			String s = new String(hash);
			return s.hashCode();
		} catch (NoSuchAlgorithmException e) {
			return (int)(d.getName().hashCode() + d.getCreationTime());
		}
	}
	
}
