/**
 * Dasu Project
 *
 * File:         PhpReporter.java
 * RCS:          $Id$
 * Description:  PhpReporter class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Oct 30, 2007 at 12:53:21 PM
 * Language:     Java
 * Package:      edu.northwestern.news.database
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2007, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.database;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.nio.charset.Charset;
import java.sql.SQLException;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Map.Entry;
import java.util.logging.Logger;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import com.azureus.plugins.aznetmon.main.RSTPacketStats;

import edu.northwestern.dasu.DasuConfiguration;
import edu.northwestern.dasu.DasuManager;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.drools.FactTemplate;
import edu.northwestern.dasu.measurement.TraceEntry;
import edu.northwestern.dasu.stats.Statistics;
import edu.northwestern.dasu.util.Pair;
import edu.northwestern.dasu.util.Util;

/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 * 
 * The PhpReporter class reports data to a web service based on PHP.
 */
public class PhpReporter implements DatabaseReporter {

	private final static Logger LOGGER = Logger.getLogger("edu.northwestern.dasu");
	
	//pointers to the active tables to use for reporter, can switch between persistent data to stats manager
	public static Hashtable<Integer, Integer> peerIds = Statistics.getInstance().peerIds;
	public static Hashtable<String, Integer> peerMap = Statistics.getInstance().peerMap;
	public static Hashtable<Integer, Integer> routerIds = Statistics.getInstance().routerIds;
	public static Hashtable<String, Integer> routerMap = Statistics.getInstance().routerMap;
	public static Hashtable<Integer, Integer> peerClientIds = Statistics.getInstance().peerClientIds;
	public static Hashtable<String, Integer> peerClientMap = Statistics.getInstance().peerClientMap;
	public static Hashtable<Integer, Integer> dnsIds = Statistics.getInstance().dnsIds;
	public static Hashtable<String, Integer> dnsMap = Statistics.getInstance().dnsMap;
	public static Hashtable<Integer, Integer> urlIds = Statistics.getInstance().urlIds;
	public static Hashtable<String, Integer> urlMap = Statistics.getInstance().urlMap;

	static final boolean TESTING = true;
	private static final boolean TESTING_GZIP = false;

	private static final String DEFAULT_ENCODING = "US-ASCII";
	private static final String BYTE_ENCODING = "ISO-8859-1";
	static Charset DEFAULT_CHARSET;
	/** the largest post that will be sent */
	
	public static int POST_THRESHOLD = DBReporterManager.POST_THRESHOLD;

	//private static final String DEFAULT_SERVER = DasuConfiguration.DASU_DEFAULT_SERVER 
	//	+ DasuConfiguration.DASU_DEFAULT_REPORTER_SCRIPT;

	private static final String DEFAULT_SERVER = DasuManager.getInstance().getNewRandomDasuUrl() 
		+ DasuConfiguration.DASU_DEFAULT_REPORTER_SCRIPT;

	private NumberFormat nf = NumberFormat.getInstance();

	private static PhpReporter currentInstance = null;
	
	static {
		try {
			Charset.forName(BYTE_ENCODING);
			DEFAULT_CHARSET = Charset.forName(DEFAULT_ENCODING);

		} catch (Throwable e) {

			e.printStackTrace();
		}
	}
	
	/*
	private ByteArrayOutputStream dsdBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream dssBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream eventBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream lpsdBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream lpssBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream pingDasuBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream httpDasuBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream dnsDasuBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream speedTestBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream upnpTestBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream rstStatBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream rpcsBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream ndtDasuBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream ispInfoBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream staticUPnPBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream interferenceBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream eventDasuBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream interferenceEventBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream taskBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream userCommentsBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream reBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream rpsdBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream rpssBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream teBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream tedBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream seBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream sedBaos = new ByteArrayOutputStream();
	private ByteArrayOutputStream timeoutDnsBaos = new ByteArrayOutputStream();
	*/
	
	private ByteArrayOutputStream dsdBaos = Statistics.getInstance().dsdBaos;
	private ByteArrayOutputStream dssBaos = Statistics.getInstance().dssBaos;
	private ByteArrayOutputStream eventBaos = Statistics.getInstance().eventBaos;
	private ByteArrayOutputStream lpsdBaos = Statistics.getInstance().lpsdBaos;
	private ByteArrayOutputStream lpssBaos = Statistics.getInstance().lpssBaos;
	private ByteArrayOutputStream pingDasuBaos = Statistics.getInstance().pingDasuBaos;
	private ByteArrayOutputStream httpDasuBaos = Statistics.getInstance().httpDasuBaos;
	private ByteArrayOutputStream dnsDasuBaos = Statistics.getInstance().dnsDasuBaos;
	private ByteArrayOutputStream speedTestBaos = Statistics.getInstance().speedTestBaos;
	private ByteArrayOutputStream upnpTestBaos = Statistics.getInstance().upnpTestBaos;
	private ByteArrayOutputStream rstStatBaos = Statistics.getInstance().rstStatBaos;
	private ByteArrayOutputStream rpcsBaos = Statistics.getInstance().rpcsBaos;
	private ByteArrayOutputStream ndtDasuBaos = Statistics.getInstance().ndtDasuBaos;
	private ByteArrayOutputStream ispInfoBaos = Statistics.getInstance().ispInfoBaos;
	private ByteArrayOutputStream staticUPnPBaos = Statistics.getInstance().staticUPnPBaos;
	private ByteArrayOutputStream interferenceBaos = Statistics.getInstance().interferenceBaos;
	private ByteArrayOutputStream eventDasuBaos = Statistics.getInstance().eventDasuBaos;
	private ByteArrayOutputStream interferenceEventBaos = Statistics.getInstance().interferenceEventBaos;
	private ByteArrayOutputStream taskBaos = Statistics.getInstance().taskBaos;
	private ByteArrayOutputStream userCommentsBaos = Statistics.getInstance().userCommentsBaos;
	private ByteArrayOutputStream reBaos = Statistics.getInstance().reBaos;
	private ByteArrayOutputStream rpsdBaos = Statistics.getInstance().rpsdBaos;
	private ByteArrayOutputStream rpssBaos = Statistics.getInstance().rpssBaos;
	private ByteArrayOutputStream teBaos = Statistics.getInstance().teBaos;
	private ByteArrayOutputStream tedBaos = Statistics.getInstance().tedBaos;
	private ByteArrayOutputStream seBaos = Statistics.getInstance().seBaos;
	private ByteArrayOutputStream sedBaos = Statistics.getInstance().sedBaos;
	private ByteArrayOutputStream timeoutDnsBaos = Statistics.getInstance().timeoutDnsBaos;	
	
	private ArrayList<Pair<String, ByteArrayOutputStream>> buffers;

	private String currentUrl;

	/** if true, data can be sent; otherwise, no data sent to server */
	private static boolean canSendData;

	private Long lastFailTime = -1L;

	private boolean hasConfig = false;
	
	private boolean useGzip = false; 

	private static final boolean REPORT_ERROR = false;

	private static final String DSD_HEADER = "type=dsd&data=";
	private static final String DSS_HEADER = "type=dss&data=";
	private static final String EVENT_HEADER = "type=event&data=";
	private static final String LPSD_HEADER = "type=lpsd&data=";
	private static final String LPSS_HEADER = "type=lpss&data=";
	private static final String PING_HEADER_DASU = "type=pingdasu&data=";
	private static final String HTTP_HEADER_DASU = "type=httpdasu&data=";
	private static final String TIMEOUT_HEADER_DNS = "type=dnstimeout&data=";
	private static final String DNS_HEADER_DASU = "type=dnsdasu&data=";
	private static final String SPEED_TEST_HEADER = "type=speedtest&data=";
	private static final String UPNP_TEST_HEADER = "type=upnptest&data=";
	private static final String RST_STAT_HEADER = "type=rststat&data=";
	private static final String RPCS_STAT_HEADER = "type=rpcstat&data=";
	private static final String NDT_HEADER_DASU = "type=ndtdasu&data=";
	private static final String INTERFERENCE_HEADER = "type=interference&data=";
	private static final String EVENT_DASU_HEADER = "type=eventdasu&data=";
	private static final String FEEDBACK_HEADER = "type=feedback&data=";
	private static final String INTERFERENCE_EVENT_HEADER = "type=interferenceevent&data=";
	private static final String TASK_HEADER = "type=task&data=";
	private static final String USER_COMMENTS_HEADER = "type=usercomments&data=";
	private static final String ISP_INFO_HEADER = "type=ispinfo&data=";
	private static final String STATIC_UPNP_HEADER = "type=supnp&data=";
	private static final String RE_HEADER = "type=re&data=";
	private static final String RPSD_HEADER = "type=rpsd&data=";
	private static final String RPSS_HEADER = "type=rpss&data=";
	private static final String TE_HEADER = "type=te&data=";
	private static final String TED_HEADER = ""; // "type=ted&data=";
	private static final String SE_HEADER = "type=se&data=";
	private static final String SED_HEADER = "";

	/**
	 * @return
	 */
	public synchronized static DatabaseReporter getInstance() {
		if (currentInstance == null
				|| (!(currentInstance instanceof PhpReporter) && !Main
						.isShuttingDown())) {
			currentInstance = new PhpReporter();
		}
		
		return currentInstance;
	}

	public synchronized boolean getCanSendData() {
		return canSendData;
	}
	
	public synchronized String getDefaultServer() {
		return DEFAULT_SERVER;
	}
 
	public synchronized String getCurrentUrl() {
		return currentUrl;
	}
	
	public synchronized Charset getDefaultCharSet() {
		return DEFAULT_CHARSET;
	}
	
	public synchronized boolean isUseGzip() {
		return useGzip;
	}
		
	public PhpReporter() {
		canSendData = true;
		nf.setMaximumFractionDigits(4);

		currentUrl = DEFAULT_SERVER;

		buffers = new ArrayList<Pair<String, ByteArrayOutputStream>>();
		buffers
				.add(new Pair<String, ByteArrayOutputStream>(DSD_HEADER,
						dsdBaos));
		buffers
				.add(new Pair<String, ByteArrayOutputStream>(DSS_HEADER,
						dssBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(EVENT_HEADER,
				eventBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(LPSD_HEADER,
				lpsdBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(LPSS_HEADER,
				lpssBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(PING_HEADER_DASU,
				pingDasuBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(TIMEOUT_HEADER_DNS,
				timeoutDnsBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(HTTP_HEADER_DASU,
				httpDasuBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(DNS_HEADER_DASU,
				dnsDasuBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(SPEED_TEST_HEADER,
				speedTestBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(UPNP_TEST_HEADER,
				upnpTestBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(RST_STAT_HEADER,
				rstStatBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(RPCS_STAT_HEADER,
				rpcsBaos));		
		buffers.add(new Pair<String, ByteArrayOutputStream>(NDT_HEADER_DASU,
				ndtDasuBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(INTERFERENCE_HEADER,
				interferenceBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(EVENT_DASU_HEADER,
				eventDasuBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(INTERFERENCE_EVENT_HEADER,
				interferenceEventBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(TASK_HEADER,
				taskBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(ISP_INFO_HEADER,
				ispInfoBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(STATIC_UPNP_HEADER,
				staticUPnPBaos));		
		buffers.add(new Pair<String, ByteArrayOutputStream>(USER_COMMENTS_HEADER,
				userCommentsBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(RE_HEADER, reBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(RPSD_HEADER,
				rpsdBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(RPSS_HEADER,
				rpssBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(TE_HEADER, teBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(TED_HEADER,	tedBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(SE_HEADER, seBaos));
		buffers.add(new Pair<String, ByteArrayOutputStream>(SED_HEADER,	sedBaos));
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#canConnect()
	 */
	public boolean canConnect() {
		
		return lastFailTime < 0
				|| (Util.currentTimeMillis() - lastFailTime) > DasuConfiguration
						.getInstance().getDbReconnectInterval() * 1000;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#connect()
	 */
	public boolean connect() {
		if (!hasConfig) {
			hasConfig = true;
			sendData("type=config", null);
		}

		return true;
	}
	
	public synchronized void updateConfig(){
		
		if ( Main.isShuttingDown() )
			return;
		
		DasuConfiguration oc = DasuConfiguration.getInstance();
		Properties props = Main.downloadFromURL(
				DasuManager.getInstance().getNewRandomDasuUrl() + DasuConfiguration.DASU_CONFIG,
				60*1000);
		
		for (Entry<Object, Object> ent : props.entrySet()){
			try {
			String key = (String)ent.getKey();
			String value = (String)ent.getValue();
			if (key.equals("dasu.stats.dbUpdate")){
				oc.setDbUpdateFreqSec(Integer.valueOf(value));
			} else if (key.equals("dasu.update.updateCheck")){
				oc.setUpdateCheckFreqSec(Integer.valueOf(value));
			}else if (key.equals("dasu.doTraceroute")){
				oc.setDoTraceroute(value.equals("true") ? true : false);
			}else if (key.equals("dasu.enablePhPReporter")){
				oc.setEnablePhPReporter(value.equals("true") ? true : false);
			}else if (key.equals("dasu.doAnaLookup")){
				oc.setDoAnaLookup(value.equals("true") ? true : false);
			}else if (key.equals("dasu.stats.peerUpdate")){
				oc.setPeerUpdateIntervalSec(Integer.valueOf(value));
			}else if (key.equals("dasu.reportingTimeout")){
				oc.setReportingTimeout(Integer.valueOf(value));
			} else if (key.equals("dasu.maxPings")){
				oc.setMaxPings(Integer.valueOf(value));
			} else if (key.equals("dasu.maxAMExperiments")){
				oc.setMaxAMExperiments(Integer.valueOf(value));
			} else if (key.equals("dasu.ignoreStateChangeRST")){
				oc.setIgnoreStateChangeRST(value.equals("true") ? true : false);
			} else if (key.equals("dasu.sampleInterval")){
				oc.setSampleInterval(Integer.valueOf(value));
				RSTPacketStats.INTERVAL = oc.getSampleInterval()*1000;
			} else if (key.equals("dasu.enableAzureusSpeedTest")){
				oc.setEnableAzureusSpeedTest(value.equals("true") ? true : false);
			}else if (key.equals("dasu.inference.version")){
				oc.setInferenceVersion(Float.valueOf(value));
			}else if (key.equals("dasu.schedulerInterval")){
				oc.setSchedulerInterval(Integer.valueOf(value));
			}else if (key.equals("dasu.messageManagerInterval")){
				oc.setMessageManagerInterval(Integer.valueOf(value));
			}else if (key.equals("dasu.AMECheckupInterval")){
				oc.setAMECheckupInterval(Integer.valueOf(value));
			}else if (key.equals("dasu.stateChangeRSTInterval")){
				oc.setStateChangeRSTInterval(Integer.valueOf(value));
			}else if (key.equals("dasu.AMCSleepInterval")){
				oc.setAMCSleepInterval(Integer.valueOf(value));
			}else if (key.equals("dasu.db.managerSleepInterval")){
				oc.setDBManagerSleepInterval(Integer.valueOf(value));
			}else if (key.equals("dasu.db.staticFlushInterval")){
				oc.setDbStaticFlushInterval(Integer.valueOf(value));
			}else if (key.equals("dasu.managerSleepInterval")){
				oc.setManagerSleepInterval(Integer.valueOf(value));
			}else if (key.equals("dasu.maxLogEntries")){
				oc.setMaxLogEntries(Integer.valueOf(value));
			}else if (key.equals("dasu.maxHistoricSignals")){
				oc.setMaxHistoricSignals(Integer.valueOf(value));
			}else if (key.equals("dasu.maxNumEntriesPerHistoricSignal")){
				oc.setMaxNumEntriesPerHistoricSignal(Integer.valueOf(value));
			}else if (key.equals("dasu.runExperiments")){
				oc.setRunExperiments(value.equals("true") ? true : false);
			}else if (key.equals("dasu.RSTimeWindowSize")){
				oc.setRSTimeWindowSize(Integer.valueOf(value));
			}else if (key.equals("dasu.timeSpan")){
				oc.setTimeSpan(Integer.valueOf(value));
			} else if (key.equals("dasu.stats.dnsTimeout")){
				oc.setDnsTimeout(Integer.valueOf(value));
			} else {
				if (TESTING) System.err.println("Unknown key: "+key);
			}
			} catch (Exception e){
				System.err.println("Error processing property...");
				e.printStackTrace();
			} catch (Error e){
				System.err.println("Error processing property...");
				e.printStackTrace();
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#disconnect()
	 */
	public void disconnect() {
		// for (Pair<String, ByteArrayOutputStream> p : buffers){
		// if (p.getValue().size()>0){
		// sendData(p.getKey(), p.getValue());
		// }
		// }

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#doCreateTable(java.lang.String)
	 */
	public void doCreateTable(String s) throws SQLException, IOException,
			ClassNotFoundException {
		throw new RuntimeException("Not supported!");

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#doInsert(java.lang.String,
	 *      java.lang.String)
	 */
	public int doInsert(String insert, String select) throws IOException,
			ClassNotFoundException, SQLException {
		throw new RuntimeException("Not supported!");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#getNextPosition()
	 */
	public int getNextPosition() {
		throw new RuntimeException("Not supported!");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#getRecordingInterval(java.lang.String)
	 */
	public int getRecordingInterval(String columnName) {
		throw new RuntimeException("Not supported!");
		// return 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#insertDataTransferResult(long,
	 *      int, byte, int, int, int, int, double, double)
	 */
	public synchronized boolean insertDataTransferResult(long time,
			int experimentId, byte type, int size, int sourceId, int destId,
			int currentId, double raceResultPrimary, double raceResultSecondary) {
		throw new RuntimeException("Not supported!");
	}

	private String sendData(String header) {
		
		if ( !DasuConfiguration.getInstance().isRecordStats() ||
				!canConnect()) 
			return null;
		
		if (TESTING)
			LOGGER.info("******REPORTER: sendData: " + header);

		try {
			if (canSendData) {
				ByteArrayOutputStream data = new ByteArrayOutputStream();
				return getResponse(header, data);
			}

		} catch (MalformedURLException e1) {
			if (TESTING)
				e1.printStackTrace();
			
		} catch (IOException e) {
			
			if (TESTING)
				e.printStackTrace();
		}
		
		return null;

	}
	
	
	private long sendData(String header, ByteArrayOutputStream data) {
		
		try {
			
			if ( !DasuConfiguration.getInstance().isRecordStats() ) 
				return -1;

			if (data == null)
				data = new ByteArrayOutputStream();
			String response = getResponse(header, data);
			
			if (TESTING)
				LOGGER.info("******REPORTER: SenData Thread: header: " + header + " reponse: " + response);
			
			if (response == null || response.length()==0)
				return -1;
			if (response.contains("Error")) {
				//data.reset();
				return -1;
			} else if (response.contains("Fail")) {
				//data.reset();
				updateLastFailTime();
				return -1;
			} else if (response.contains("Success")) {
				data.reset();
				return 0;
			} else if (response.length() > 0) {
				if (header.equals("type=config")) {
					updateConfig();
					return 0;
				} else {
					try {
						//if (data != null && data.size() > 0)
						//	data.reset();
						// must be encoded as <length>:<number>
						String pair[] = response.split("\\:");
						if (pair.length != 2)
							return -1;
						int size = Integer.parseInt(pair[0]);
						if (pair[1].length() == size)
							return Integer.parseInt(pair[1]);
						else
							return -1;
					} catch (NumberFormatException e) {
						return -1;
					}
				}
			} else {
				return -1;
			}

		} catch (MalformedURLException e1) {
			
			if (TESTING)
				e1.printStackTrace();
			
		} catch (IOException e) {
			
			if (TESTING)
				e.printStackTrace();
			
			updateLastFailTime();
			
			//clear to avoid filling the memory
			if (data.size() > DBReporterManager.DROP_THRESHOLD)
				data.reset();
			
		}
		//} finally {
		//	if (data!=null) data.reset();
		//}
		return -1;
	}

	/**
	 * Update lastFailTime in a way that prevents a thundering herd by using randomness
	 */
	public synchronized void updateLastFailTime() {
		lastFailTime = Util.currentTimeMillis() + 
			(long)(DasuConfiguration.getInstance().getDbReconnectInterval()*
					1000*(0.5-Main.getRandom().nextDouble()));

		hasConfig = false;
	}

	/**
	 * Update lastFailTime in a way that prevents a thundering herd by using randomness
	 */
	public synchronized void setLastFailTime(long val) {
		lastFailTime = val;
	}
	
	private String getResponse(String header, ByteArrayOutputStream data)
			throws IOException {
		
		StringBuilder sbout = new StringBuilder();
		sbout.append("\n#######\n");
		if (TESTING)
			sbout.append("REPORTER: getResponse: lastFailTime: " + new Date(lastFailTime) + "\n");
		
		if (!canConnect()) 
			return null;
		
		URL url = null;
		URLConnection urlConn = null;
		InputStream is = null;

		while (true) {
			try {
				url = new URL(currentUrl);
				if (TESTING) {
					String h = header;
					if (header.contains("\n"))
						h = header.split("\\n")[0];
					if (h == null || h.length() == 0) {
						h = data.toString();
						if (h.contains("\n"))
							h = h.split("\\n")[0];
					}
					sbout.append("REPORTER: ["
							+ DateFormat.getDateTimeInstance().format(
									new Date()) + "]" + " Connecting to: "
							+ url + "(" + h + ")\n");
				}
			} catch (MalformedURLException e2) {
				currentUrl = DasuManager.getInstance().getNewRandomDasuUrl();
				updateLastFailTime();
				return null;
			}

			// URL connection channel.
			try {
				urlConn = url.openConnection();
				break;
			} catch (IOException e2) {
				
				currentUrl = DasuManager.getInstance().getNewRandomDasuUrl();
				updateLastFailTime();
				return null;

			}
		}

		lastFailTime = -1L; // reset failure time since we are successful
		
		try {
			byte headerB[] = DEFAULT_CHARSET.encode(header).array();
			byte dataB[] = data.toByteArray();
			int rawLength = headerB.length+dataB.length;
			// Let the run-time system (RTS) know that we want input.
			urlConn.setDoInput(true);
			// Let the RTS know that we want to do output.
			urlConn.setDoOutput(true);
			// No caching, we want the real thing.
			urlConn.setUseCaches(false);
			// Specify the content type.
			urlConn.setRequestProperty("Content-Type",
					"application/x-www-form-urlencoded");
			urlConn.setRequestProperty("accept-encoding",
                    "gzip");
			// java gzip implementation wastes more space than it saves if size 
			// is less than 150
			if (rawLength>=150 && useGzip)urlConn.setRequestProperty("Content-Encoding",
            "gzip");
			
			// Send POST output.
			OutputStream os = urlConn.getOutputStream();
			
			if (useGzip){
				if (rawLength<150){
//					os = new NewsGZIPOutputStream(os, 512, Deflater.BEST_COMPRESSION);
				}
				else os = new GZIPOutputStream(os);
			}
			
			if (TESTING) 
				sbout.append("REPORTER: [Sending "+(headerB.length+dataB.length)+" bytes]\n");

			os.write(headerB);
			if (data.size() > 0)
				os.write(dataB);

			if (useGzip && os instanceof GZIPOutputStream) 
				((GZIPOutputStream)os).finish();
			
			os.flush();		
			os.close();
			
			// Get response data.
			for (Entry<String, List<String>> ent : urlConn.getHeaderFields().entrySet() ){
				for (String s : ent.getValue()){
					if (s.equals("gzip")) {
						useGzip = true;
						break;
					}
					if (useGzip) break;
				}
			}
			is = urlConn.getInputStream();
			if (useGzip) is = new GZIPInputStream(is);
			StringBuffer sb = new StringBuffer();
			byte b[] = new byte[1];
			boolean eof = false;
			int readCount = 0;
			while (!eof) {
				int c = is.read(b);
				if (c < 0)
					break;
				sb.append((char) b[0]);
				readCount++;
				if (readCount > 10000) {
					LOGGER.info("REPORTER: ERROR: Something wrong!?\n");
					break;
				}
			}
			is.close();			
			
			if (TESTING) 
				sbout.append("REPORTER: " + sb.toString());
			
			LOGGER.info(sbout.toString() + "\n#######\n");			
			return sb.toString();
			
		} finally {
			if (TESTING) {
				sbout.append("REPORTER: [Done]\n#######\n");
				LOGGER.info(sbout.toString());
			}
			if (is != null) {
				try {
					is.close();
				} catch (IOException e) {
					LOGGER.info("REPORTER: ERROR: " + e.getCause());
				}
			}
		}
				
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#insertDownloadStatDynamic(int,
	 *      int, long, int, float, int, int, int, int, int, int, int, int, int)
	 */
	public synchronized boolean insertDownloadStatDynamic(int selfId,
			int downloadId, long timeRecorded, int numPeers,
			float availability, int completed, int discarded, int downloadRate,
			int uploadRate, int shareRatio, int numKnownNonSeeds,
			int numKnownSeeds, int state, int priority,
			long downloaded, long uploaded, int position,
			int totalNumKnownNonSeeds, int totalNumKnownSeeds, long hashFails,
			int connectedSeeds, int connectedLeechers, long swarmPeerSpeed) {

		if (!peerIds.containsKey(selfId))
			return false;
		
		selfId = peerIds.get(selfId); 
		
		ByteArrayOutputStream baos = dsdBaos;
		String out = selfId + ";" + downloadId + ";" + timeRecorded + ";"
				+ numPeers + ";" + formatDouble(availability) + ";" + completed
				+ ";" + discarded + ";" + downloadRate + ";" + uploadRate + ";"
				+ shareRatio + ";" + numKnownNonSeeds + ";" + numKnownSeeds
				+ ";" + state + ";" + priority + ";" + position 
				+ ";" + downloaded + ";" + uploaded 
				+ ";" + totalNumKnownNonSeeds + ";" + totalNumKnownSeeds 
				+ ";" + hashFails + ";" + DasuConfiguration.getInstance().getUUIDid() 
				+ ";" + connectedSeeds + ";" + connectedLeechers + ";" + swarmPeerSpeed + "\n";
		
		if ( TESTING ) {
			LOGGER.info("REPORTER: insertDownloadStatDynamic: " + out);
		}
		
		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		if (baos.size() > POST_THRESHOLD) {
			long response = sendData(DSD_HEADER, baos);
			return response >= 0;
		} else
			return true;
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#insertDownloadStatSummary(int,
	 *      int, long, long, long, int, int, long, long, long, long)
	 */
	public synchronized boolean insertDownloadStatSummary(int selfId,
			int downloadId, long currentTime, long downloadingTime,
			long seedingTime, int maxDownloadRateLimit, int maxUploadLimit,
			long totalUploaded, long totalDownloaded, long size, long pieceSize) {
		
		if (TESTING)
			LOGGER.info("REPORTER: insertDownloadStatSummary()");
				
		if (!peerIds.containsKey(selfId))
			return false;
		
		selfId = peerIds.get(selfId);

		ByteArrayOutputStream baos = dssBaos;
		String out = selfId + ";" + downloadId + ";" + currentTime + ";"
				+ downloadingTime + ";" + seedingTime + ";"
				+ maxDownloadRateLimit + ";" + maxUploadLimit + ";"
				+ totalUploaded + ";" + totalDownloaded + ";" + size + ";"
				+ pieceSize + ";" + DasuConfiguration.getInstance().getUUIDid() + "\n";

		
		if ( TESTING )
			LOGGER.info("REPORTER: insertDownloadStatSummary: " + out);
		
		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		if (baos.size() > POST_THRESHOLD)
			return sendData(DSS_HEADER, baos) >= 0;

		return true;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#insertEvent(int, int,
	 *      int, int, long, int)
	 */
	public synchronized boolean insertEvent(int customerId, int edgeId,
			int peerId, int eventType, long time, int timeZoneId) {
		
		ByteArrayOutputStream baos = eventBaos;
		String out = customerId + ";" + edgeId + ";" + peerId + ";" + eventType
				+ ";" + time + ";" + timeZoneId + ";" + DasuConfiguration.getInstance().getUUIDid() + "\n";

		if ( TESTING )
			LOGGER.info("REPORTER: insertEvent: " + out);
		
		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}

		if (baos.size() > POST_THRESHOLD) {
			long response = sendData(EVENT_HEADER, baos);
			return response >= 0;	
			//Main.getPluginInterface().getUtilities().createThread(EVENT_HEADER, new SendData("EVENT_HEADER", baos));
		} else
			return true;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#insertLocalPeerStatDynamic(int,
	 *      long, int, int)
	 */
	public synchronized boolean insertLocalPeerStatDynamic(int selfId,
			long currentTime, int uploadRate, int downloadRate, 
			int maxDownloadRate, int maxUploadRate, boolean isAutoSpeed, int rst, 
				int numConnections, int passiveOpens, int activeOpens, int failedConns,
				int numTorrents, int numSeedingTorrents, int numLeechingTorents, 
				int numConnectedSeeds, int numConnectedLeechers,
				long bytesSentLpsd, long bytesReceivedLpsd, long bytesSentNetstat, long bytesReceivedNetstat, long swarmPeerSpeed,
				long numberOfActiveConnections, long totalBytesSent, long totalBytesReceived, long totalPacketsSent, 
				long totalPacketsReceived, long uptime) {
		
		if (TESTING)
			LOGGER.info("REPORTER: insertLocalPeerStatDynamic");
		
		if (!peerIds.containsKey(selfId))
			return false;

		selfId = peerIds.get(selfId);

		ByteArrayOutputStream baos = lpsdBaos;
		String out = selfId + ";" + currentTime + ";" + uploadRate + ";"
			+ downloadRate + ";"+ 
			//maxDownloadRate + ";" + maxUploadRate + ";" + 
			(isAutoSpeed?1:0) + ";" + rst + ";" + numConnections 
			+ ";" + passiveOpens+ ";" + activeOpens+ ";" + failedConns+ 
			";" + DasuConfiguration.getInstance().getUUIDid() +
			";" + numTorrents + ";" + numSeedingTorrents + ";" + numLeechingTorents + 
			";" + numConnectedSeeds + ";" + numConnectedLeechers + 
			";" + bytesSentLpsd + ";" + bytesReceivedLpsd + ";" + bytesSentNetstat + 
			";" + bytesReceivedNetstat + ";" + swarmPeerSpeed + 
			";" + numberOfActiveConnections + ";" + totalBytesSent + ";" + totalBytesReceived +
			";" + totalPacketsSent + ";" + totalPacketsReceived + ";" + uptime + "\n";
		
		if (TESTING)
			LOGGER.info("REPORTER: insertLocalPeerStatDynamic: " + out);

		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		if (baos.size() > POST_THRESHOLD) {
			long response = sendData(LPSD_HEADER, baos);
			return response >= 0;
			//Main.getPluginInterface().getUtilities().createThread(LPSD_HEADER, new SendData("LPSD_HEADER", baos));
		} else
			return true;
				
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#insertLocalPeerStatStatic(int,
	 *      long, int, int)
	 */
	public synchronized boolean insertLocalPeerStatStatic(int selfId, long uptime,
			int maxUploadRate, int maxDownloadRate, int maxPeerConnectionsGlobal, 
			int maxPeerConnectionsPerTorrent, int maxPeerConnectionsPerTorrentSeeding, 
			int maxUploadRateSeeding, int maxSeedsPerTorrent, int lazyBitfield, int requireEncryption,
			String encryptionMinLevel, int allowNonEncryptedOUT, int allowNonEncryptedIn,
			int useCryptoPortExtension, int tcpPortIn, int udpPortIn) {
		
		if (TESTING)
			LOGGER.info("REPORTER: insertLocalPeerStatStatic()");
		
		if (!peerIds.containsKey(selfId))
			return false;

		selfId = peerIds.get(selfId);
		
		ByteArrayOutputStream baos = lpssBaos;
		String out = selfId + ";"
				+ uptime + ";"
				+ maxUploadRate + ";"
				+ maxDownloadRate + ";"
				+ Util.currentGMTTime() + ";"
				+ DasuConfiguration.getInstance().getUUIDid() + ";"
				+ maxPeerConnectionsGlobal + ";"
				+ maxPeerConnectionsPerTorrent + ";"
				+ maxPeerConnectionsPerTorrentSeeding + ";" 
				+ maxUploadRateSeeding + ";"
				+ maxSeedsPerTorrent + ";"
				+ lazyBitfield  + ";"
				+ requireEncryption + ";"
				+ encryptionMinLevel + ";"
				+ allowNonEncryptedOUT + ";"
				+ allowNonEncryptedIn + ";"
				+ useCryptoPortExtension + ";"
				+ tcpPortIn + ";"
				+ udpPortIn				
				+"\n";
		
		if (TESTING)
			LOGGER.info("REPORTER: insertLocalPeerStatStatic: " + out);
		
		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
			
		} catch (IOException e) {
			e.printStackTrace();
		}

		if (baos.size() > POST_THRESHOLD)
			return sendData(LPSS_HEADER, baos) >= 0;
			
		return true;

	}

	public synchronized boolean insertPingResultDasu(int sourceId, int destId,float rtt,  
			long time, int destType, int taskId) {

		if (TESTING)
			LOGGER.info("REPORTER: insertPingResultDasu()");
		
		if (!peerIds.containsKey(destId))
			return false;

		destId = peerIds.get(destId);
		sourceId = peerIds.get(sourceId);
		
		ByteArrayOutputStream baos = pingDasuBaos;
		String out = sourceId + ";" + destId + ";" 
				+ formatDouble(rtt) + ";" + time + ";" + destType + ";" + taskId +
				";" + DasuConfiguration.getInstance().getUUIDid() + "\n";
		
		if (TESTING)
			LOGGER.info("REPORTER: insertPingResultDasu: " + out);

		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
	
		if (baos.size() > POST_THRESHOLD) {
			long response = sendData(PING_HEADER_DASU, baos);
			return response >= 0;
			//Main.getPluginInterface().getUtilities().createThread(PING_HEADER_DASU, new SendData("PING_HEADER_DASU", baos));
		} else
			return true;
		
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#insertRemotePeerStatDynamic(int,
	 *      int, long, int, int, int, int, boolean, double, boolean, boolean)
	 */
	public synchronized boolean insertRemotePeerStatDynamic(int selfId,
			int peerId, long currentTime, int bytesReceivedFromPeer,
			int bytesSentToPeer, int currentReceiveRate, int currentSendRate,
			boolean isSeed, double ping, boolean isInterested,
			boolean isInteresting) {
		
		if (TESTING)
			LOGGER.info("REPORTER: insertRemotePeerStatDynamic()");
		
		if (!peerIds.containsKey(selfId) || !peerIds.containsKey(peerId))
			return false;
		
		selfId = peerIds.get(selfId);
		int pid = peerIds.get(peerId);
		
		ByteArrayOutputStream baos = rpsdBaos;
		String out = selfId + ";" + pid + ";" + currentTime + ";"
				+ bytesReceivedFromPeer + ";" + bytesSentToPeer + ";"
				+ currentReceiveRate + ";" + currentSendRate + ";" + isSeed
				+ ";" + formatDouble(ping) + ";" + isInterested + ";"
				+ isInteresting + 
				";" + DasuConfiguration.getInstance().getUUIDid() +"\n";
		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		if (TESTING)
			LOGGER.info("REPORTER: insertRemotePeerStatDynamic: " + out);

		if (baos.size() > POST_THRESHOLD) {
			long response = sendData(RPSD_HEADER, baos);
			return response >= 0;
			//Main.getPluginInterface().getUtilities().createThread(RPSD_HEADER, new SendData("RPSD_HEADER", baos));
		} else
			return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#insertRemotePeerStatStatic(int,
	 *      int, int, long, int, int, boolean)
	 */
	public synchronized boolean insertRemotePeerStatStatic(int selfId, int peerId,
			int peerClient, long uptime, int tcpPort, int udpPort,
			boolean sameCluster, int type, String encryption, long pieceSize) {
		
		if (TESTING)
			LOGGER.info("REPORTER: insertRemotePeerStatStatic()");
		
		if (!peerIds.containsKey(selfId) || !peerIds.containsKey(peerId) || !peerClientIds.containsKey(peerClient))
			return false;

		selfId = peerIds.get(selfId);
		peerId = peerIds.get(peerId);
		peerClient = peerClientIds.get(peerClient);
		
		ByteArrayOutputStream baos = rpssBaos;
		String out = selfId
				+ ";"
				+ peerId
				+ ";"
				+ peerClient
				+ ";"
				+ uptime
				+ ";"
				+ tcpPort
				+ ";"
				+ udpPort
				+ ";"
				+ (sameCluster ? "1" : "0")
				+ ";"
				+ Util.currentGMTTime() 
				+ ";" 
				+ DasuConfiguration.getInstance().getUUIDid() 
				+ ";"
				+ type
				+ ";"
				+ encryption 
				+ ";"
				+ pieceSize
				+"\n";
		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		if (TESTING)
			LOGGER.info("REPORTER: insertRemotePeerStatStatic: " + out);
		

		if (baos.size() > POST_THRESHOLD)
			return sendData(RPSS_HEADER, baos) >= 0;

		return true;
	}

	public boolean insertHttpResultDasu(int sourceId, int urlId,  
    		double downloadDuration, double downloadSpeed, long time, 
    		int noBytes, int taskId) {
		
		if (TESTING)
			LOGGER.info("REPORTER: insertHttpResultDasu()");

		if (!peerIds.containsKey(sourceId))
			return false;

		sourceId = peerIds.get(sourceId);
		urlId = urlIds.get(urlId);  
		
		ByteArrayOutputStream baos = httpDasuBaos;
		String out = sourceId + ";" + urlId + ";" 
				+ formatDouble(downloadDuration) + ";" + formatDouble(downloadSpeed) + ";"
				+ noBytes + ";" + time + ";" + taskId + 
				";" + DasuConfiguration.getInstance().getUUIDid() +"\n";
		
		if (TESTING)
			LOGGER.info("REPORTER: insertHttpResultDasu: " + out);
		
		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
	
		if (baos.size() > POST_THRESHOLD) {
			long response = sendData(HTTP_HEADER_DASU, baos);
			return response >= 0;
			//Main.getPluginInterface().getUtilities().createThread(HTTP_HEADER_DASU, new SendData("HTTP_HEADER_DASU", baos));
		} else
			return true;
	}
	
	public boolean insertDnsMapping(int myId, int publicDns, int privateDns, String type, int order, long timeRecorded) {

		if (TESTING)
			LOGGER.info("REPORTER: insertDnsMapping()");

		if (!peerIds.containsKey(myId) || !dnsIds.containsKey(publicDns) || !dnsIds.containsKey(privateDns))
			return false;

		myId = peerIds.get(myId);
		publicDns = dnsIds.get(publicDns);
		privateDns = dnsIds.get(privateDns);
		
		String data = "type=dnsMapping&data=" + myId + ";" + publicDns + ";"
				+ privateDns + ";" + type + ";" + order + ";" + timeRecorded + ";" 
				+ DasuConfiguration.getInstance().getUUIDid() + "\n";
		
		if (TESTING)
			LOGGER.info("REPORTER: insertDnsMapping: " + data);

		
		long response = sendData(data, null);
		return response >= 0;
	}
	
	public boolean insertDnsResult(int myId, int dnsId, 
			long requestTime, long responseTime, String query,
			String type, String options, String response, boolean failed,
			long taskId) {
		
		if (TESTING)
			LOGGER.info("REPORTER: insertDnsResult()");

		if (!peerIds.containsKey(myId) || !dnsIds.containsKey(dnsId))
			return false;

		myId =  peerIds.get(myId);
		dnsId = dnsIds.get(dnsId);

		ByteArrayOutputStream baos = dnsDasuBaos;
		String out = myId + ";" + dnsId + ";" 
				+ requestTime + ";" + responseTime + ";"
				+ query + ";" + type + ";" + options + ";" + response + ";" 
				+ (failed ? "1" : "0") + ";" + taskId + 
				";" + DasuConfiguration.getInstance().getUUIDid() +"\n";
		
		if (TESTING)
			LOGGER.info("REPORTER: insertDnsResult: " + out);
		
		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
	
		if (baos.size() > POST_THRESHOLD) {
			long resp = sendData(DNS_HEADER_DASU, baos);
			return resp >= 0;			
		} else
			return true;
	
	}

	
	
	public boolean insertSpeedTestResult(int sourceId, long testTime, 
			int downloadSpeed, int uploadSpeed, int encryption, char type, 
			int downloadSpeedNetstat, int uploadSpeedNetstat) {

		if (TESTING)
			LOGGER.info("REPORTER: insertSpeedTestResult()");
				
		if (!peerIds.containsKey(sourceId))
			return false;

		sourceId = peerIds.get(sourceId);
		
		ByteArrayOutputStream baos = speedTestBaos;
		String out = sourceId + ";" + testTime + ";" 
				+ downloadSpeed + ";" + uploadSpeed + ";"
				+ encryption + ";" + type + ";"
				+ DasuConfiguration.getInstance().getUUIDid() + ";"
				+ downloadSpeedNetstat + ";" + uploadSpeedNetstat + "\n";
		
		if (TESTING)
			LOGGER.info("REPORTER: insertSpeedTestResult: " + out);
		
		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		long response = sendData(SPEED_TEST_HEADER, baos);
		return response >= 0;
	}

	
	public boolean insertUpnpTestResult(int sourceId, long testTime, 
        	int port, String protocol, int success) {

		if (TESTING)
			LOGGER.info("REPORTER: insertUpnpTestResult()");

		if (!peerIds.containsKey(sourceId))
			return false;

		sourceId = peerIds.get(sourceId);
		
		ByteArrayOutputStream baos = upnpTestBaos;
		String out = sourceId + ";" + testTime + ";" 
				+ port + ";" + protocol + ";"
				+ success 
				+ ";" + DasuConfiguration.getInstance().getUUIDid() +"\n";
		
		if (TESTING)
			LOGGER.info("REPORTER: insertUpnpTestResult: " + out);
		
		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
	
		long response = sendData(UPNP_TEST_HEADER, baos);
		return response >= 0;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#insertTraceEntry(int,
	 *      int, long)
	 */
	
	public synchronized long insertTraceEntry(int sourceId, int destId,
			long time, int entries, int taskId) {

		if (TESTING)
			LOGGER.info("REPORTER: insertTraceEntry()");

		if (!peerIds.containsKey(sourceId) || !peerIds.containsKey(destId))
			return -1;

		sourceId = peerIds.get(sourceId);
		destId = peerIds.get(destId);		
		
		ByteArrayOutputStream baos = tedBaos;

		if (baos.size() > POST_THRESHOLD) {
			sendData("", baos);
		}

		String s;
		entries += 1;
		if (tedBaos.size() == 0)
			s = "type=te&data=" + sourceId + ";" + destId + ";" + time + ";"
					+ entries + ";" + taskId + ";" + DasuConfiguration.getInstance().getUUIDid() +"\n";
		else
			s = sourceId + ";" + destId + ";" + time + ";" + entries + ";" + taskId + ";" + DasuConfiguration.getInstance().getUUIDid() + "\n";
		try {
			baos.write(DEFAULT_CHARSET.encode(s).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		if (TESTING)
			LOGGER.info("REPORTER: insertTraceEntry: " + s);

		return 0;
	}
	
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#insertScrapeEntry
	 */
	
	public long insertScrapeEntry(String type, int myId, String url, String hash,
			int entries, int numSeeds, int numLeechers, int taskId, long time) {

		if (TESTING)
			LOGGER.info("REPORTER: insertScrapeEntry()");
		
		if (!peerIds.containsKey(myId))
			return -1;

		myId = peerIds.get(myId);
		
		ByteArrayOutputStream baos = sedBaos;
		
		if (baos.size() > POST_THRESHOLD) {
			sendData("", baos);
		}

		int typeId = -1;
		if (type.equals("DHT"))
				typeId = 3;
		else if (type.equals("TRACKER_ANNOUNCE"))
			typeId = 2;
		else if (type.equals("TRACKER_SCRAPE"))
			typeId = 1;
		
		String s;
		if (sedBaos.size() == 0)
			s = "type=se&data=" + myId + ";" + typeId + ";" + url + ";" + hash + ";"
					+ entries + ";" + numSeeds + ";" + numLeechers + ";"
					+ time + ";" + taskId + ";" 
					+ DasuConfiguration.getInstance().getUUIDid() +"\n";
		else
			s = myId + ";" + typeId + ";" + url + ";" + hash + ";"
				+ entries + ";" + numSeeds + ";" + numLeechers + ";"
				+ time + ";" + taskId + ";" 
				+ DasuConfiguration.getInstance().getUUIDid() +"\n";
			
		try {
			baos.write(DEFAULT_CHARSET.encode(s).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		if (TESTING)
			LOGGER.info("REPORTER: insertScrapeEntry: " + s);

		return 0;
	}

	public void flushScrapes() {
		sendData("", sedBaos);
	}
	

	public synchronized boolean insertTraceEntryData(long entryId,
			int[] myRouterIds, TraceEntry te) {
		
		if (TESTING)
			LOGGER.info("REPORTER: insertTraceEntryData()");
		
		int[] dbRouterIds = new int[myRouterIds.length];
		for (int i=0; i<myRouterIds.length; i++) {

			if (!routerIds.containsKey(myRouterIds[i]))
				return false;

			dbRouterIds[i] = routerIds.get(myRouterIds[i]);
		}

		ByteArrayOutputStream baos = tedBaos;
		StringBuffer sb = new StringBuffer();

		for (int i = 0; i < te.rtt.length; i++) {
			sb.append(dbRouterIds[i]);
			sb.append(";");
			sb.append(formatDouble(te.rtt[i]));
			if (i == te.rtt.length - 1)
				sb.append("\n");
			else
				sb.append(";");
		}

		try {
			baos.write(DEFAULT_CHARSET.encode(sb.toString()).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		if (TESTING)
			LOGGER.info("REPORTER: insertTraceEntryData: " + sb);


		return true;

	}
	
	public synchronized boolean insertScrapeEntryData(long entryId,
			int peerId, int tcp, int udp) {

		if (TESTING)
			LOGGER.info("REPORTER: insertScrapeEntryData()");
		
		if (!peerIds.containsKey(peerId)) {
			LOGGER.info("REPORTER: " + peerIds);
			LOGGER.info("REPORTER: " + peerMap);
			LOGGER.info("REPORTER: not in peerid!! why? |"  + peerId + " " + peerIds.containsKey(peerId) +
					" " + peerIds.get(10l) +  "|\n\n\n\n");
			return false;
		}

		peerId = peerIds.get(peerId);
		
		ByteArrayOutputStream baos = sedBaos;

		StringBuffer sb = new StringBuffer();
		sb.append(entryId + ";" + peerId + ";" + tcp + ";" + udp + "\n");

		try {
			baos.write(DEFAULT_CHARSET.encode(sb.toString()).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		if (TESTING)
			LOGGER.info("REPORTER: insertScrapeEntryData: " + sb);

//		 if (baos.size()>POST_THRESHOLD){
//			 long response = sendData("", baos);
//		 }
		
		return true;

	}
	
	public synchronized boolean insertDnsTimeout(int myDnsId, int myId, long startTime, long endTime, int giveUp) {

		if (TESTING)
			LOGGER.info("REPORTER: inserDnsTimeout()");

		if ((!peerIds.containsKey(myId)) || (!dnsIds.containsKey(myDnsId)))
			return false;

		myDnsId = dnsIds.get(myDnsId);
		myId = peerIds.get(myId);
		
		
		ByteArrayOutputStream baos = timeoutDnsBaos;
		String out = myDnsId + ";" + myId + ";" + startTime + ";" + endTime + ";" +  giveUp + ";" +
			DasuConfiguration.getInstance().getUUIDid() + "\n";
		
		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		if (TESTING)
			LOGGER.info("REPORTER: inserDnsTimeout: " + out);
		
		if (baos.size() > POST_THRESHOLD) {
			long response = sendData(TIMEOUT_HEADER_DNS, baos);
			return response >= 0;
		}
		
		return true; 
	}
	
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#makeId(java.lang.String,
	 *      java.lang.String, java.lang.String)
	 */
	public int makeId(String tableName, String colName, String value) {
				
		String data = "type=makeId&data=" + tableName + ";" + colName + ";"
				+ value + "\n";
		
		if (TESTING)
			LOGGER.info("REPORTER: makeId: " + data);

		return (int) sendData(data, null);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#registerUsage(int,
	 *      long, java.lang.String)
	 */
	public boolean registerUsage(int myId, long GMTTime, String uuid, long currentTime,
			String timeZone, boolean failed) {

		if (TESTING)
			LOGGER.info("REPORTER: registerUsage()");
		
		if (!peerIds.containsKey(myId))
			return false;

		myId = peerIds.get(myId);
		
		String data = "type=registerUsage&data=" + myId + ";" + GMTTime +
			";" + uuid + "&os="+getOsString() + "&clientVer=" + 
			Main.getPluginInterface().getPluginVersion() + "&ruleVer=" + 
			DasuConfiguration.getInstance().getInferenceVersion() + 
			"&currTime=" + currentTime + "&timeZone=" + timeZone + 
			"&failed=" + (failed?1:0) + "\n";
		
		if (TESTING)
			LOGGER.info("REPORTER: registerUsage: " + data);
		
		return sendData(data, null) >= 0;

	}
	
	public String getIspInfo(String myIp) {
		
		if (TESTING)
			LOGGER.info("REPORTER: getIspInfo()");

		
		String data = "type=getispinfo&data=" + myIp;
		return sendData(data);
	}

	
	public String getOsString(){
		String name = System.getProperty("os.name");
		String version = System.getProperty("os.version");
		String arch = System.getProperty("os.arch");
		if (name != null){
			return name + ";" + version + ";"+arch;
		}
		else return "Unknown";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#reportError(java.lang.Exception)
	 */
	public void reportError(Exception e) {
		
		if (!REPORT_ERROR ) 
			return;
		
		String error = e.toString() + "\n";
		error += e.getCause() != null ? e.getCause().toString() + "\n" : e
				.getMessage()
				+ "\n";
		for (StackTraceElement ste : e.getStackTrace()) {
			error += ste.toString() + "\n";
		}

		String data = "type=error&data=" + error + "\n";
		sendData(data, null);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#setExtendedConnection(boolean)
	 */
	public void setExtendedConnection(boolean extend) {
		// this has no meaning in a web service

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.northwestern.dasu.database.DatabaseReporter#stopReporting()
	 */
	public void stopReporting() {
		canSendData = false;

	}

	public synchronized void flush() {
		
		if (TESTING)
			LOGGER.info("REPORTER: PhpReporter flushing");
		
		for (Pair<String, ByteArrayOutputStream> p : buffers) {
						
			if (p.getValue().size() > 0) {
				
				Thread s = new SendDataPhpReporter(p.getKey(), p.getValue());
				s.start();

			}
		}
	}

	public Map<String, Integer> makeIdBatch(String tableName,
			String columnName, List<String> values) {

		if (TESTING)
			LOGGER.info("REPORTER: makeIdBatch: "  + tableName + ": " + values.size());

		if (!canSendData)
			return null;
		
		long start = Util.currentTimeMillis();
		HashMap<String, Integer> toReturn = new HashMap<String, Integer>();
		try {
			String data = "type=makeIds&data=" + tableName + ";" + columnName
					+ "\n";

			for (String s : values) {
				data += s + "\n";
			}
			LinkedList<Integer> keys = sendDataBatch(data);
			Iterator<String> sIt = values.iterator();
			Iterator<Integer> iIt = keys.iterator();
			
			while (sIt.hasNext() && iIt.hasNext()) {
				int id = iIt.next();
				if (id == -1)
					sIt.next();
				else
					toReturn.put(sIt.next(), id);
			}
			return toReturn;
		} finally {
			long time = Util.currentTimeMillis() - start;
			if (TESTING)
				LOGGER.info("REPORTER: Took " + time / 1000.0 + " seconds, found " 
						+ toReturn.size() + " entries...");
		}
	}

	private LinkedList<Integer> sendDataBatch(String data) {
		
		LinkedList<Integer> toReturn = new LinkedList<Integer>();
		
		if (TESTING)
			LOGGER.info("REPORTER: sendDataBatch()");

		try {
			
			String s = getResponse(data, new ByteArrayOutputStream());
			if (s == null) 
				return toReturn;
			
			String lines[] = s.split("\\n");
			for (String response : lines) {
				try {

					// must be encoded as <length>:<number>
					String pair[] = response.split("\\:");
					if (pair.length != 2) {
						toReturn.add(-1);
					} else {
						int size = Integer.parseInt(pair[0]);
						if (pair[1].length() == size)
							toReturn.add(Integer.parseInt(pair[1]));
						else
							toReturn.add(-1);
					}
				} catch (NumberFormatException e) {
					toReturn.add(-1);
				}
			}

		} catch (IOException e) {
			
			if (TESTING)
				e.printStackTrace();
			
			updateLastFailTime();
		}
		return toReturn;
	}

	private String formatDouble(double d) {
		String s = Double.toString(d);
		if (s.length() > 10) {
			if (s.contains("E")) {
				return s.substring(0, 7) + s.substring(s.indexOf('E'));
			} else
				return s.substring(0, 9);
		}
		return s;
	}

	public void reportIpChange(int oldIp, int newIp, int localIp,
			long currentGMTTime) {

		if ( !peerIds.contains(oldIp) || peerIds.contains(newIp) || !peerIds.contains(localIp))
			return;
		
		oldIp = peerIds.get(oldIp);
		newIp = peerIds.get(newIp);
		localIp = peerIds.get(localIp);

		String data = "type=ipchange&data=" + oldIp +";" + newIp +";"
			+ localIp +";" + currentGMTTime+ ";" + DasuConfiguration.getInstance().getUUIDid() + "\n";
		sendData(data, null);
		
	}

	public boolean insertNdtResultDasu(int myId, String osName, String osArch,
			String osVer, String interfaceName, String javaVer, String server,
			double rate, double rate2, double mylink, int mismatch,
			double avgrtt, int packetSize, 
			int connectionStalledTimes,
			double connectionIdlePercentage, int packetQueuing,
			int packetQueuing2,
			double throughputPacketQueueingDetectedPercentage,
			double connectionReceiverLimitedPercentage,
			double connectionSenderLimitedPercentage,
			double connectionNetworkLimitedPercentage, int RFC2018,
			int RFC896, int RFC3168, int RFC1323TimeStamping,
			int RFC1323WindowScaling, int serverScalingFactor,
			int clientScalingFactor, int serverBehindFirewall,
			int clientBehindFirewall, double theoreticalNetworkLimit,
			double serverBuffer, double serverThroughputLimit,
			double clientBuffer, double clientThroughputLimit, long time,
			int failed, String errorMsg, double c2sRateNetstat, double s2cRateNetstat,
			double networkFlowControlThroughputLimit, int congestion, int congestionSignals,
			int fastRetran, int timeouts, double outOfOrderPercentage, 
			int pktsRetrans, int dupAcksIn, int sACKsRcvd, double packetLoss) {
		

		if (TESTING)
			LOGGER.info("REPORTER: insertNdtResultDasu()");
		

		if (!peerIds.containsKey(myId))
			return false;
		
		myId = peerIds.get(myId);

		ByteArrayOutputStream baos = ndtDasuBaos;

		String out = myId + ";" + osName + ";" + osArch + ";" +
			osVer + ";" + interfaceName + ";" + javaVer + ";" + server + ";" +
			rate + ";" + rate2 + ";" + mylink + ";" + mismatch + ";" +
			avgrtt + ";" + packetSize + ";" + connectionStalledTimes + ";" +
			connectionIdlePercentage + ";" + packetQueuing + ";" +
			packetQueuing2 + ";" + throughputPacketQueueingDetectedPercentage + ";" +
			connectionReceiverLimitedPercentage + ";" + 
			connectionSenderLimitedPercentage + ";" +
			connectionNetworkLimitedPercentage + ";" + RFC2018 + ";" +
			RFC896 + ";" + RFC3168 + ";" + RFC1323TimeStamping + ";" +
			RFC1323WindowScaling + ";" + serverScalingFactor + ";" +
			clientScalingFactor + ";" + serverBehindFirewall + ";" +
			clientBehindFirewall + ";" + theoreticalNetworkLimit + ";" +
			serverBuffer + ";" + serverThroughputLimit + ";" +
			clientBuffer + ";" + clientThroughputLimit + ";" + time + ";" +
			failed + ";" + DasuConfiguration.getInstance().getUUIDid() + ";" +
			c2sRateNetstat + ";" + s2cRateNetstat + ";" + networkFlowControlThroughputLimit + ";" +
			congestion + ";" + congestionSignals + ";" + fastRetran + ";" +  timeouts + ";" +
			outOfOrderPercentage + ";" + pktsRetrans + ";" + dupAcksIn + ";" + sACKsRcvd + ";" +
			packetLoss + "\n";
		
		if (TESTING)
			LOGGER.info("REPORTER: insertNdtResultDasu: " + out);
		
		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
	
		long response = sendData(NDT_HEADER_DASU, baos);
		return response >= 0;

		
	}

	public boolean insertUserReports(int sourceId, String downloadIds, String comment) {
		
		if (TESTING)
			LOGGER.info("REPORTER: insertUserReports()");
		
		if (!peerIds.containsKey(sourceId))
			return false;

		sourceId = peerIds.get(sourceId);
		
		comment = comment.replace(";", ",");
		comment = comment.replace(" ", "_");
		comment = comment.replace("&", "n");
		ByteArrayOutputStream baos = userCommentsBaos;
		String out = sourceId + ";" + Util.currentGMTTime() + ";" +
			downloadIds + ";" + comment 
			+ ";" + DasuConfiguration.getInstance().getUUIDid() + "\n";

		if (TESTING)
			LOGGER.info("REPORTER: insertUserReports: " + out);

		
		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
	
		long response = sendData(USER_COMMENTS_HEADER, baos);
		return response >= 0;

	}

	public boolean insertIspInfo(int sourceId, String prefix, String ispname, long timeRecorded) {

		if (!peerIds.containsKey(sourceId))
			return false;

		sourceId = peerIds.get(sourceId);
		
		ispname = ispname.replace("&", "n");
		ispname = ispname.replace(";", ",");
		ispname = ispname.replace(" ", "_");

		ByteArrayOutputStream baos = ispInfoBaos;
		String out = sourceId + ";" + timeRecorded + ";" +
			prefix + ";" + ispname + ";"  + DasuConfiguration.getInstance().getUUIDid() +"\n";

		if (TESTING)
			LOGGER.info("REPORTER: insertIspInfo: " + out);
		
		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
	
		long response = sendData(ISP_INFO_HEADER, baos);
		return response >= 0;

	}

	public boolean insertConfigEventDasu(int sourceId, String event, long timestamp) {
		
		if (peerIds.containsKey(sourceId))
			sourceId = peerIds.get(sourceId);
		else 
			sourceId = -1;			

		ByteArrayOutputStream baos = eventDasuBaos;
		String out = sourceId + ";" + timestamp + ";" + event 
			+ ";" + DasuConfiguration.getInstance().getUUIDid() +"\n";

		if (TESTING)
			LOGGER.info("REPORTER: insertConfigEventDasu: " + out);
		
		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
	
		long response = sendData(EVENT_DASU_HEADER, baos);
		return response >= 0;
		
	}
	
	public boolean insertFeedback(String type, String name, String email, String message, String dump,
			long systime, long dasuGmtTime) {

		if (TESTING)
			LOGGER.info("REPORTER: insertFeedback()");
		
		int myId  = Statistics.getInstance().updateIdMap();

		if (!peerIds.containsKey(myId))
			return false;

		int sourceId = peerIds.get(myId);
		
		dump = dump.replace(";", "|");
		dump = dump.replace("&", "AND");
		if (message != null) {
			message = message.replace(";", ",");
			message = message.replace("&", "and");
		}
		if (type == null ) type = "bug";
		
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		String out = sourceId + ";" + systime + ";" + dasuGmtTime 
			+ ";" + type + ";" + name + ";" + email + ";" + message 
			+ ";" + DasuConfiguration.getInstance().getUUIDid()
			+ ";" + dump + "\n";

		if (TESTING)
			LOGGER.info("REPORTER: insertFeedback: " + out);
		
		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
	
		long response = sendData(FEEDBACK_HEADER, baos);
		return response >= 0;
		
	}
	
	public boolean insertInterference(int sourceId, String type, long timestamp) {
		
		if (TESTING)
			LOGGER.info("REPORTER: insertInterference()");
				
		if (!peerIds.containsKey(sourceId))
			return false;

		sourceId = peerIds.get(sourceId);
		
		ByteArrayOutputStream baos = interferenceBaos;
		String out = sourceId + ";" + timestamp + ";" + type 
		+ ";" + DasuConfiguration.getInstance().getUUIDid() +"\n";

		if (TESTING)
			LOGGER.info("REPORTER: insertInterference: " + out);

		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
	
		long response = sendData(INTERFERENCE_HEADER, baos);
		return response >= 0;
		
	}
	
	public boolean insertInterferenceEvent(int sourceId, String type, long timestamp, long downloadId, long droppedTo, float mean) {
		
		if (!peerIds.containsKey(sourceId))
			return false;

		sourceId = peerIds.get(sourceId);
		
		ByteArrayOutputStream baos = interferenceEventBaos;
		String out = sourceId + ";" + timestamp + ";" + type 
		+ ";" + downloadId + ";" + droppedTo + ";" + mean
		+ ";" + DasuConfiguration.getInstance().getUUIDid() +"\n";

		if (TESTING) {
			LOGGER.info("REPORTER: insertInterferenceEvent()");
			LOGGER.info("REPORTER: " + out);
		}

		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
	
		if (baos.size() > POST_THRESHOLD) {
			long response = sendData(INTERFERENCE_EVENT_HEADER, baos);
			return response >= 0;
		} else
			return true;
	
	}


	public boolean insertTask(int sourceId, long timeStamp, int taskId) {
		
		if (TESTING) 
			LOGGER.info("REPORTER: insertTask()");
		

		if (!peerIds.containsKey(sourceId))
			return false;

		sourceId = peerIds.get(sourceId);

		ByteArrayOutputStream baos = taskBaos;
		String out = sourceId + ";" + timeStamp + ";" + taskId 
		+ ";" + DasuConfiguration.getInstance().getUUIDid() +"\n";

		if (TESTING) 
			LOGGER.info("REPORTER: insertTask: " + out);
		
		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
	
		long response = sendData(TASK_HEADER, baos);
		return response >= 0;
		
	}

	/* (non-Javadoc)
	 * @see edu.northwestern.dasu.database.DatabaseReporter#insertRSTStat(int, int, long, int, long, int, int, java.lang.String, java.lang.String, boolean, boolean, int, int)
	 */
	public boolean insertRSTStat(int sourceId, int destId, long downloadId,
			int numRst, long timeRecorded, int type, int port,
			String bitTorrentClient, String source, int isSeed,
			int amSeed, int numSeeds, int numPeers, String encryption) {

		if (TESTING) 
			LOGGER.info("REPORTER: insertRSTStat()");		

		if (!peerIds.containsKey(sourceId) || !peerIds.containsKey(destId))
			return false;

		sourceId = peerIds.get(sourceId);
		destId = peerIds.get(destId);

		ByteArrayOutputStream baos = rstStatBaos;
		String out = sourceId + ";" + destId + ";" 
				+ downloadId + ";" + 
				+ timeRecorded + ";" + type + ";"
				+ port + ";" + bitTorrentClient + ";" 
				+ source + ";" + isSeed + ";"
				+ amSeed + ";" + numSeeds + ";" + numPeers + ";"
				+ DasuConfiguration.getInstance().getUUIDid() + ";"
				+ encryption +"\n";
		
		if (TESTING)
			LOGGER.info("REPORTER: insertRSTStat: " + out);
		
		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
	
		if (baos.size() > POST_THRESHOLD) {
			long response = sendData(RST_STAT_HEADER, baos);
			return response >= 0;
			//Main.getPluginInterface().getUtilities().createThread(RST_STAT_HEADER, new SendData("RST_STAT_HEADER", baos));
		} else
			return true;
	}
	
	public boolean insertRemotePeerConnectionStat(int sourceId, int destId,
			long timeRecorded, int type, int port,
			String bitTorrentClient, int isSeed,
			int amSeed, int numSeeds, int numPeers, String encryption) {

		if (TESTING)
			LOGGER.info("REPORTER: insertRemotePeerConnectionStat()");

		if (!peerIds.containsKey(sourceId) || !peerIds.containsKey(destId))
			return false;

		sourceId = peerIds.get(sourceId);
		destId = peerIds.get(destId);
		
		ByteArrayOutputStream baos = rpcsBaos;
		String out = sourceId + ";" + destId + ";"  
				+ timeRecorded + ";" + type + ";"
				+ port + ";" + bitTorrentClient + ";" 
				+ isSeed + ";"
				+ amSeed + ";" + numSeeds + ";" + numPeers
				+ ";" + DasuConfiguration.getInstance().getUUIDid()
				+ ";" + encryption +"\n";
		
		if (TESTING)
			LOGGER.info("REPORTER: insertRemotePeerConnectionStat: " + out);
		
		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
	
		if (baos.size() > POST_THRESHOLD) {
			long response = sendData(RPCS_STAT_HEADER, baos);
			return response >= 0;
			//Main.getPluginInterface().getUtilities().createThread(RPCS_STAT_HEADER, new SendData("RPCS_STAT_HEADER", baos));
		} else
			return true;
	}

	/* (non-Javadoc)
	 * @see edu.northwestern.dasu.database.DatabaseReporter#insertStaticUPnPInfo(int, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
	 */
	public boolean insertStaticUPnPInfo(int myId, String UpstreamMaxBitRate,
			String DownstreamMaxBitRate, String NATEnabled,
			String Layer1UpstreamMaxBitRate, String Layer1DownstreamMaxBitRate,
			String PortMappingEnabled, String WANAccessType,
			String MaximumActiveConnections, String ExternalIPAddress) {
		
		if (TESTING)
			LOGGER.info("REPORTER: insertStaticUPnPInfo()");

		if (!peerIds.containsKey(myId))
			return false;

		myId = peerIds.get(myId);
		
		ByteArrayOutputStream baos = staticUPnPBaos;
		String out = myId + ";" + Util.currentGMTTime() + ";" +
			UpstreamMaxBitRate + ";" + DownstreamMaxBitRate + ";" + NATEnabled + ";" +
			Layer1UpstreamMaxBitRate + ";" + Layer1DownstreamMaxBitRate + ";" + 
			PortMappingEnabled + ";" + WANAccessType + ";" + 
			MaximumActiveConnections + ";" + ExternalIPAddress + ";" + DasuConfiguration.getInstance().getUUIDid() +"\n";

		if (TESTING)
			LOGGER.info("REPORTER: insertStaticUPnPInfo: " + out);
		
		try {
			baos.write(DEFAULT_CHARSET.encode(out).array());
		} catch (IOException e) {
			e.printStackTrace();
		}
	
		long response = sendData(STATIC_UPNP_HEADER, baos);
		return response >= 0;
		
	}

	public boolean pushGenericReport(FactTemplate obj) {
		return false;
	}

	
}
