/**
 * Dasu Project
 *
 * File:         SessionRestartMonitor.java
 * RCS:          $Id: SessionRestartMonitor.java,v 1.0 Oct 19, 2012 6:08:01 PM mas939 Exp $
 * Description:  SessionRestartMonitor class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Oct 19, 2012 at 6:08:01 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.database
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2012, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.database;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import edu.northwestern.dasu.DasuManager;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The SessionRestartMonitor class ...
 */
public class SessionRestartMonitor2 {
	
	public final static java.util.logging.Logger LOGGER = java.util.logging.Logger.getLogger("edu.northwestern.dasu");
	private Long loadedSessionStartTime = 0l;
	private Long currentSessionStartTime = 0l;
	
	Hashtable<String, Integer> peerMap;
	Hashtable<Integer, Integer> peerIds;
	Hashtable<String, Integer> peerClientMap;
	Hashtable<Integer, Integer> peerClientIds;
	Hashtable<String, Integer> routerMap;
	Hashtable<Integer, Integer> routerIds;
	Hashtable<String, Integer> urlMap;
	Hashtable<Integer, Integer> urlIds;
	Hashtable<String, Integer> dnsMap;
	Hashtable<Integer, Integer> dnsIds;
	
	public SessionRestartMonitor2(long currentTime, Hashtable<String, Integer> peerMap, Hashtable<Integer, Integer> peerIds,
			Hashtable<String, Integer> peerClientMap, Hashtable<Integer, Integer> peerClientIds,
			Hashtable<String, Integer> routerMap, Hashtable<Integer, Integer> routerIds,
			Hashtable<String, Integer> urlMap, Hashtable<Integer, Integer> urlIds,
			Hashtable<String, Integer> dnsMap, Hashtable<Integer, Integer> dnsIds) {
		
		LOGGER.info("REPORTER: starting session: " + new Date(currentTime));
		currentSessionStartTime = currentTime;
		DasuManager.getPersistentDataManager().getDbIdMaps().put(currentTime, new Hashtable<String, Hashtable<String, Integer>>());
		DasuManager.getPersistentDataManager().getDbIdMaps().get(currentTime).put("peerMap", new Hashtable<String, Integer>());
		DasuManager.getPersistentDataManager().getDbIdMaps().get(currentTime).put("routerMap", new Hashtable<String, Integer>());
		DasuManager.getPersistentDataManager().getDbIdMaps().get(currentTime).put("dnsMap", new Hashtable<String, Integer>());
		DasuManager.getPersistentDataManager().getDbIdMaps().get(currentTime).put("peerClientMap", new Hashtable<String, Integer>());
		DasuManager.getPersistentDataManager().getDbIdMaps().get(currentTime).put("urlMap", new Hashtable<String, Integer>());

		this.peerMap = peerMap;
		this.peerIds = peerIds;
		this.peerClientMap = peerClientMap;
		this.peerClientIds = peerClientIds;
		this.routerMap = routerMap;
		this.routerIds = routerIds;
		this.urlMap = urlMap;
		this.urlIds = urlIds;
		this.dnsMap = dnsMap;
		this.dnsIds = dnsIds;
		
		swapDbIdsToSession(getSessionTimes().get(0));

	}
	
	public Long getLoadedSessionStartTime() {
		return loadedSessionStartTime;
	}
	
	public Long getCurrentSessionStartTime() {
		return currentSessionStartTime;
	}
	
	public List<Long> getSessionTimes() {
		
		List<Long> ret = new ArrayList<Long>();
		ret.addAll(DasuManager.getPersistentDataManager().getDbIdMaps().keySet());
		Collections.sort(ret);
		return ret;
		
	}
	
	private void removeSession(long sessionTime) {
		
		if (DasuManager.getPersistentDataManager().getDbIdMaps().containsKey(sessionTime) &&
				sessionTime != loadedSessionStartTime) {			
			DasuManager.getPersistentDataManager().getDbIdMaps().remove(sessionTime);
		}
		else {
			LOGGER.warning("REPORTER: failed! sessionTime does not exist or is the current active session: " + new Date(sessionTime));
		}
		
	}
	
	public synchronized Long swapDbIdsToSessionStartTimeBefore(long thisTime) {
		
		Iterator<Long> iter = getSessionTimes().iterator();
		while ( iter.hasNext() ) {
			
			long session = iter.next();
			if ( session <= thisTime ) {
				swapDbIdsToSession(session);
				return session;
			}
		}
		
		LOGGER.warning("REPORTER: could not find session with start time previous to: " + new Date(thisTime));
		return -1l;
		
	}
	
	public synchronized Long swapToNextSessionAndRemoveCurrent() {
		
		List<Long> sessionTimes = getSessionTimes();
		if ( loadedSessionStartTime < sessionTimes.get(0) ) {
			LOGGER.warning("REPORTER: current session must be the firs session to allow this");
			return -1l;
		}
		
		if ( loadedSessionStartTime.equals(currentSessionStartTime) ) {
			LOGGER.warning("REPORTER: current session is the currentSession");
			return -1l;
		}
		
		//this should never happen
		if ( sessionTimes.size() == 1) {
			LOGGER.warning("REPORTER: current session is the only one!");
			return -1l;
		}

		Long loadedSession = this.loadedSessionStartTime;
		Long newSession = sessionTimes.get(1);
		swapDbIdsToSession(newSession);
		removeSession(loadedSession);
		
		return newSession;
		
	}
	
	private void swapDbIdsToSession(long sessionTime) {
		
		LOGGER.info("REPORTER: swapping session: " + new Date(sessionTime));
		
		if (DasuManager.getPersistentDataManager().getDbIdMaps().containsKey(sessionTime) ) {
		
			this.peerMap = DasuManager.getPersistentDataManager().getDbIdMaps().get(sessionTime).get("peerMap");
			this.routerMap = DasuManager.getPersistentDataManager().getDbIdMaps().get(sessionTime).get("routerMap");
			this.dnsMap = DasuManager.getPersistentDataManager().getDbIdMaps().get(sessionTime).get("dnsMap");
			this.peerClientMap = DasuManager.getPersistentDataManager().getDbIdMaps().get(sessionTime).get("peerClientMap");
			this.urlMap = DasuManager.getPersistentDataManager().getDbIdMaps().get(sessionTime).get("urlMap");

			this.peerIds.clear();
			this.routerIds.clear();
			this.urlIds.clear();
			this.peerClientIds.clear();
			this.dnsIds.clear();

			loadedSessionStartTime = sessionTime;
			
			//this should never happen
			/*
			if ( dnsMap == null ) {
				peerMap = new Hashtable<String, Integer>();
				routerMap = new Hashtable<String, Integer>();
				dnsMap = new Hashtable<String, Integer>();
				peerClientMap = new Hashtable<String, Integer>();
				urlMap = new Hashtable<String, Integer>();
			}
			*/
		}
		else
			LOGGER.info("REPORTER: could not swap session, sessionTime does not exist: " + new Date(sessionTime));
		
	}

}
