/**
 * Dasu Project
 *
 * File:         FactTemplate.java
 * RCS:          $Id$
 * Description:  FactTemplate class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      May 31, 2010 at 5:18:18 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.drools
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package edu.northwestern.dasu.drools;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyDescriptor;
import java.io.Serializable;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;

import edu.northwestern.cs.aqualab.Cattle.json.JSONException;
import edu.northwestern.cs.aqualab.Cattle.json.JSONObject;
import edu.northwestern.dasu.util.Util;


/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The FactTemplate class ...
 */
public abstract class FactTemplate implements Serializable {

	private long id = System.nanoTime();
	private long creationGMTTime = Util.currentGMTTime();
	
	public FactTemplate() {
	}
	
    protected PropertyChangeSupport pcs = new PropertyChangeSupport(this);

	public void addPropertyChangeListener(PropertyChangeListener pcl) {
        pcs.addPropertyChangeListener(pcl);
    }
    public void removePropertyChangeListener(PropertyChangeListener pcl) {
        pcs.removePropertyChangeListener(pcl);
    }	
		
	public long getId() {
		return id;
	}
	public void setId(long id) {
		this.id = id;
	}
	
	public long getCreationGMTTime() {
		return creationGMTTime;
	}
	
	@Override
	public String toString() {

		String s = new String();

		try {
			s = this.getClass().getSimpleName() + ": " + this.getClass().getClassLoader() + ": ";
				
			BeanInfo beanInfoOfMyClass =
				Introspector.getBeanInfo(this.getClass());
			PropertyDescriptor propertiesOfMyClass[] =
			beanInfoOfMyClass.getPropertyDescriptors();

			for(int i=0;i<propertiesOfMyClass.length;i++){
				
				try {
					Method getter = propertiesOfMyClass[i].getReadMethod();
					Object value = getter.invoke(this, new Object[]{});
					s = s.concat(propertiesOfMyClass[i].getName() + ": " + value + " ");
				}
				catch (Exception e) {
					//e.printStackTrace();
				}
			}	
			
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IntrospectionException e) {
			// TODO Auto-generated catch block
		}

		return s;
			
	}
	
	public JSONObject toJSON() {

		JSONObject report = new JSONObject();
		HashMap<Object, Object> map = new HashMap();
		try {
			
			/*
			for (Field f : this.getClass().getDeclaredFields()) {
				if (f.isAccessible())
					map.put(f.getName(), f.get(this));
				System.out.println(f);
			}
			*/
			for (Field f : this.getClass().getFields()) {
				if ( !map.containsKey(f.getName()) )
					map.put(f.getName(), f.get(this));
				//System.out.println(f);
			}
		
			report.append("values", map);
			
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (JSONException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
		
		return report;
	}

	
}
