package edu.northwestern.dasu.dynamic.drools;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;

import org.xbill.DNS.Message;
import org.xbill.DNS.RRset;
import org.xbill.DNS.Record;
import org.xbill.DNS.Section;

import edu.northwestern.dasu.drools.FactProbeResult;
import edu.northwestern.dasu.measurement.types.ProbeType;

public class FactDnsProbeResult extends FactProbeResult{
	
	public String server = null;
	public String toLookup = null;
	public String options = null;
	public Long requestTime = -1L;
	public String type = null;
	public MessageSerializable response = null;
	public Long responseTime = -1L;
	public int taskId = -1;
	public String simpleResponse = null;
	public boolean failed = false;
	public boolean timeout = false;

	public FactDnsProbeResult(String server, String toLookup, String options,
			Long requestTime, Message response, Long responseTime, String type, int taskId) {
		super(ProbeType.DNS);
		
		this.server = server;
		this.toLookup = toLookup;
		this.options = options;
		this.requestTime = requestTime;
		this.response = new MessageSerializable(response);
		this.responseTime = responseTime;
		this.type = type;
		this.taskId = taskId;
		this.simpleResponse = getSimpleResponse();
	}

	public MessageSerializable getResponse() {
		return response;
	}
	
	public String getSimpleResponse() {		
		
		if (response != null)
			try {
				Record[] rec = response.getSectionArray(Section.ANSWER);
				return rec[0].rdataToString();
			}
			catch (Exception e) {
				
			}
		
		return "";
			
	}
	
	public Record[] readSection(int section) {				
		return response.getSectionArray(section);
	}
	
	public String getOptions() {
		return options;
	}
	
	public MessageSerializable readMessage() {
		return response;
	}

	public void setResponse(MessageSerializable response) {
		this.response = response;
	}
	
	public void setResponse(Message response) {
		this.response = new MessageSerializable(response);
	}

	public void setOptions(String options) {
		this.options = options;
	}

	public Long getRequestTime() {
		return requestTime;
	}

	public void setRequestTime(Long requestTime) {
		this.requestTime = requestTime;
	}

	public Long getResponseTime() {
		return responseTime;
	}

	public void setResponseTime(Long responseTime) {
		this.responseTime = responseTime;
	}

	public String getServer() {
		return server;
	}

	public void setServer(String server) {
		this.server = server;
	}
	
	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public String getToLookup() {
		return toLookup;
	}

	public void setToLookup(String toLookup) {
		this.toLookup = toLookup;
	}

	public int getTaskId() {
		return taskId;
	}

	public void setTaskId(int taskId) {
		this.taskId = taskId;
	}

	public void setSimpleResponse(String simpleResponse) {
		this.simpleResponse = simpleResponse;
	}

	public boolean isFailed() {
		return failed;
	}

	public void setFailed(boolean failed) {
		this.failed = failed;
	}

	/**
	 * @return Returns the timeout.
	 */
	public boolean isTimeout() {
		return timeout;
	}

	/**
	 * @param timeout The timeout to set.
	 */
	public void setTimeout(boolean timeout) {
		this.timeout = timeout;
	}

	public class MessageSerializable implements Cloneable, Serializable {
		
		private static final long serialVersionUID = 1L;
		
		private List<Record[]> sectionsRecord = new LinkedList();
		private List<RRset[]> sectionsRRSets = new LinkedList();
		private Record [] emptyRecordArray = new Record[0];
		private RRset [] emptyRRsetArray = new RRset[0];
		
		public MessageSerializable(Message msg) {
			
			/** The question (first) section: QUESTION = 0; **/
			/** The answer (second) section: ANSWER	= 1; **/
			/** The authority (third) section: AUTHORITY = 2; **/
			/** The additional (fourth) section: ADDITIONAL	= 3; **/
			/* Aliases for dynamic update */
			/** The zone (first) section of a dynamic update message: ZONE = 0; **/
			/** The prerequisite (second) section of a dynamic update message: PREREQ = 1; **/
			/** The update (third) section of a dynamic update message: UPDATE	= 2; **/

			for ( int section = 0; section <= 3; section++) {
				sectionsRecord.add(section, msg.getSectionArray(section));
				sectionsRRSets.add(section, msg.getSectionRRsets(section));
			}
			
		}
		
		public Record [] getSectionArray( int section ) {
			
			if (sectionsRecord.get(section) == null)
				return emptyRecordArray;
			
			return sectionsRecord.get(section);
			
		}
		
		public RRset []
				getSectionRRsets(int section) {
			
			if (sectionsRRSets.get(section) == null)
				return emptyRRsetArray;
			return sectionsRRSets.get(section);
			
		}
		
	}
}
