/**
 * Dasu Project
 *
 * File:         DasuHttpModule.java
 * RCS:          $Id$
 * Description:  DasuHttpModule class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Apr 15, 2010 at 07:38:30 PM
 * Language:     Java
 * Package:      edu.northwestern.ono.experiment
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.dynamic.measurement;

import java.io.IOException;
import java.net.InetAddress;
import java.net.SocketTimeoutException;
import java.util.ArrayList;
import java.util.Hashtable;

import org.apache.zookeeper.client.FourLetterWordMain;
import org.xbill.DNS.*; 

import edu.northwestern.dasu.DasuConfiguration;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.drools.FactDnsTimeout;
import edu.northwestern.dasu.drools.FactTemplate;
import edu.northwestern.dasu.drools.ReportableFact;
import edu.northwestern.dasu.dynamic.drools.FactDnsProbeResult;
import edu.northwestern.dasu.measurement.AMCoordinatorV2;
import edu.northwestern.dasu.measurement.DasuProbeModule;
import edu.northwestern.dasu.measurement.tasks.ProbeTask;
import edu.northwestern.dasu.measurement.tasks.ProbeTask.ProbeSubTask;
import edu.northwestern.dasu.measurement.types.SourceType;
import edu.northwestern.dasu.stats.Statistics;
import edu.northwestern.dasu.util.Pair;
import edu.northwestern.dasu.util.Util;

/**
 * @author Mario Sanchez &lt;msanchez@u.northwestern.edu&gt;
 * 
 *         The DasuScrapeModule class manages the set of 
 *         Scrapes performed by the peer.
 */

public class DasuDnsModule extends DasuProbeModule<FactDnsProbeResult> {	
	
	// XXX: need to finish hooking up probesubtask!!! This will require work in AMC, and how it calls the probe* functions.		
	private static boolean DEBUG = false;
	
	ArrayList<ArrayList<String>> myDns =  null;
	public Hashtable<Long, FactDnsProbeResult> dnsResultTable;
	private ArrayList<ProbeSubTask> toDoLookupList = new ArrayList<ProbeSubTask>();
	private static DasuDnsModule self;
	private Hashtable<String, Long> timeoutIsRunning;	
	
	public DasuDnsModule() {
		
		super("DasuDnsModule", 1.0f);
		this.pi = Main.getPluginInterface();
		dnsResultTable = new Hashtable<Long, FactDnsProbeResult>();
		timeoutIsRunning = new Hashtable<String, Long>();
		active = true;
		
		new FourLetterWordMain();
	}
	
	public synchronized static DasuDnsModule getInstance() {
		
		if (self == null) {
			self = new DasuDnsModule();
			self.setDaemon(true);
			return self;
		} else {
			return self;
		}		
	}

	@Override
	public void run() {
		
		while (!Main.isShuttingDown() && isActive()) {
		 
		  	ProbeSubTask pst = null;
			synchronized (toDoLookupList) {
				while ((toDoLookupList.size()==0) && !Main.isShuttingDown()) {
					try {
						toDoLookupList.wait(sleepInterval*1000);
					} catch (InterruptedException e) {
						LOGGER.info("STOP: Adios...");
						return;
					}
				}
				
				if (toDoLookupList.size() > 0)
					pst = toDoLookupList.remove(0);
				
				if (DEBUG)
					LOGGER.info("DNS: toLookup " + pst);
			}
	
			if ( pst != null ) {

				probeIp(pst);
				
				if ( yieldInterval != -1) {
					try {
						sleep(yieldInterval);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
			}
			
		} // end while active
		
		LOGGER.info("STOP: Adios...");
		
	}
	
	/*default probe uses default DNS server*/
	
	@Override
	public void probeAll(ProbeTask pt) {
		
		if ( pt != null) {
			synchronized (toDoLookupList) {
				for (ProbeSubTask pst : pt.getSubTasks())
					toDoLookupList.add(pst);
				toDoLookupList.notifyAll();
			}
		}
	}

	@Override
	public void probeWithPriority(ProbeSubTask pst) {
		
		if ( pst != null) {
			synchronized (toDoLookupList) {
				toDoLookupList.add(pst);
				toDoLookupList.notifyAll();
			}
		}
	}

	@Override
	public void probe(ProbeSubTask pst) {
		
		if (DEBUG)
			LOGGER.info("DNS: probe() " + pst);
		
		if ( pst != null ) {
			synchronized (toDoLookupList) {
				toDoLookupList.add(pst);
				toDoLookupList.notifyAll();
			}
		}
	}

	
	public String getType(String options) {
		
		String type = "A";
		
		if (options != null) {
			int arg = 0;
			String[] argv = options.split(" ");
			while (argv.length > arg && argv[arg].startsWith("-") && (argv[arg].length() > 1)) {
	
				switch (argv[arg].charAt(1)) {
	
				case 't':
					
					String typeString;
					
					if (argv[arg].length() > 2) {
						typeString = argv[arg].substring(2);
					} else {
						typeString = argv[++arg];
					}
	
					type = typeString;
					
					break;
				
				case 'x':
					
					type = "PTR";
					break;
				}
				
				arg++;
			}
		}
		
		return type;
		
	}
	
	/* (non-Javadoc)
	 * @see edu.northwestern.dasu.measurement.DasuProbeModule#probeIp(edu.northwestern.dasu.measurement.AMCoordinatorV2.ProbeSubTask)
	 */
	@Override
	protected Object probeIp(ProbeSubTask pst) {
		
		if ( !isActive() )
			return null;
		
		String server = null;
		if(pst.params != null && pst.params.containsKey("server")) server = (String) pst.params.get("server");

		String options = null;
		if(pst.params != null && pst.params.containsKey("options")) options = (String) pst.params.get("options");
		
		Long requestTime = Util.currentGMTTime();
		Message msg = doLookup(pst.ipAddress, server, options, pst.taskId);
		FactDnsProbeResult ret = null;
		
		if (DEBUG)
			LOGGER.info("DNS: msg: " + msg);

		try {
			if (( msg != null ) && ( msg.getHeader().getID() != -1)) {				
				
				ret = new FactDnsProbeResult(server, pst.ipAddress, options,
						requestTime, msg, Util.currentGMTTime(), getType(options), pst.taskId);
				ret.setTarget(pst.target);
				if ( ret.getServer() == null ) {
					
					org.xbill.DNS.ResolverConfig DNSConfig = new org.xbill.DNS.ResolverConfig();
					server = DNSConfig.server();
					FactDnsProbeResult fact =  new FactDnsProbeResult(DNSConfig.server(), pst.ipAddress, options,
							requestTime, msg, Util.currentGMTTime(), getType(options), pst.taskId);
					
					
					Statistics.getInstance().addDnsForLookup(fact.getServer());
					reportProbeResult(fact);

				}
				else { 
					Statistics.getInstance().addDnsForLookup(ret.getServer());
					reportProbeResult(ret);
				}
				
			}
			else {
				
				ret = new FactDnsProbeResult(server, pst.ipAddress, options,
						requestTime, new Message(), Util.currentGMTTime(), getType(options), pst.taskId);		
				ret.setFailed(true);		
				ret.setTarget(pst.target);
								
				if ( msg != null &&  msg.getHeader().getID() == -1)
					ret.setTimeout(true);
				
			}
		}

		catch (Exception e) {
			e.printStackTrace();

			ret = new FactDnsProbeResult(server, pst.ipAddress, options,
					requestTime, new Message(), Util.currentGMTTime(), getType(options), pst.taskId);		
			ret.setFailed(true);
			ret.setTarget(pst.target);
			
			if ( msg.getHeader().getID() == -1)
				ret.setTimeout(true);			
		}
		
		ret.setPst(pst);
		
		updateLookupResultQueue(ret);
		return null;

	}
	
	// TODO: merge with Util.doLookup.
	// JSO: not sure how this code differs from Util.doLookup, but a diff shows that it does.
	public Message doLookup(String nameString, String server, String args, Integer taskId) {

		String[] argv = new String[]{};
		if (args != null)
			argv = args.split(" ");
		
		org.xbill.DNS.Name name = null;
		int type = org.xbill.DNS.Type.A;
		int dclass = DClass.IN;
		int arg;
		Message query;
		Message response = null;
		Record rec;
		SimpleResolver res = null;
		boolean printQuery = false;
		
		if ( server == null ) {
			org.xbill.DNS.ResolverConfig DNSConfig = new org.xbill.DNS.ResolverConfig();
			server = DNSConfig.server();
		}
		if ( server == null) {
			LOGGER.info("DNS: server is null");
			return new Message();
		}

		if (timeoutIsRunning.containsKey(server)) {
			return new Message();
		}

		//server = "4.4.4.2";
		
		Statistics.getInstance().addDnsForLookup(server);
		
		try {
			
			res = new SimpleResolver(server);
			
			arg = 0;
			name = Name.fromString(nameString, Name.root);

			if ( argv.length > 0) {
				
				while (argv.length > arg && argv[arg].startsWith("-") && (argv[arg].length() > 1)) {
					
					switch (argv[arg].charAt(1)) {

					case 'x':
						name = ReverseMap.fromAddress(nameString);
						type = Type.PTR;
						dclass = DClass.IN;
						break;
						
					case 'p':
	
						String portStr;
						int port;
	
						if (argv[arg].length() > 2) {
							portStr = argv[arg].substring(2);
						} else {
							portStr = argv[++arg];
						}
	
						port = Integer.parseInt(portStr);
	
						if ((port < 0) || (port > 65536)) {
							LOGGER.info("DNS: Invalid port");
	
							return null;
						}
	
						res.setPort(port);
	
						break;
	
					case 'b':
	
						String addrStr;
	
						if (argv[arg].length() > 2) {
							addrStr = argv[arg].substring(2);
						} else {
							addrStr = argv[++arg];
						}
	
						InetAddress addr;
	
						try {
							addr = InetAddress.getByName(addrStr);
						} catch (Exception e) {
							LOGGER.info("DNS: Invalid address");
	
							return null;
						}
	
						res.setLocalAddress(addr);
	
						break;
	
					case 'k':
	
						String key;
	
						if (argv[arg].length() > 2) {
							key = argv[arg].substring(2);
						} else {
							key = argv[++arg];
						}
	
						res.setTSIGKey(TSIG.fromString(key));
	
						break;
	
					case 'T':
	
						res.setTCP(true);
						break;
						
					case 't':
						
						String typeString;
						
						if (argv[arg].length() > 2) {
							typeString = argv[arg].substring(2);
						} else {
							typeString = argv[++arg];
						}
	
						type = Type.value(typeString);
						
						break;
	
					case 'i':
						res.setIgnoreTruncation(true);
	
						break;
	
					case 'e':
	
						String ednsStr;
						int edns;
	
						if (argv[arg].length() > 2) {
							ednsStr = argv[arg].substring(2);
						} else {
							ednsStr = argv[++arg];
						}
	
						edns = Integer.parseInt(ednsStr);
	
						if ((edns < 0) || (edns > 1)) {
							LOGGER.info("DNS: Unsupported " + "EDNS level: "
									+ edns);
	
							return null;
						}
	
						res.setEDNS(edns);
	
						break;
	
					case 'd':
						res.setEDNS(0, 0, ExtendedFlags.DO, null);
	
						break;
	
					case 'q':
						printQuery = true;
	
						break;
	
					default:
						LOGGER.info("DNS: Invalid option: ");
						LOGGER.info("DNS: " + argv[arg]);
					}
	
					arg++;
				}
			}
			
			if (name == null) {
				return null;
			}
			
			rec = Record.newRecord(name, type, dclass);
			query = Message.newQuery(rec);

			if (printQuery) {
				System.out.println(query);
			}

			response = res.send(query);
			
		} catch (ArrayIndexOutOfBoundsException e) {
			LOGGER.info("DNS: " + e.toString());
			return null;
		} catch (TextParseException e) {
			LOGGER.info("DNS: " + e.toString());
			return null;
		} catch (IOException e) {
			
			if (e instanceof SocketTimeoutException) {
				
				if (DasuConfiguration.getInstance().getDnsTimeout() != 0) {
					
					 launchDnsTimeout(server);
				}
			}
			
			LOGGER.info("DNS: " + e.toString());
			return null;
		}

		return response;		
	}
	
	public ArrayList<ArrayList<String>> getMyDnsServers() {		
		return getMyDns(Main.getPublicIpAddress());
	}
	
	public ArrayList<ArrayList<String>> getMyDns(String myIp) {
		
		if (myDns != null)
			return myDns;
		
		String publicDnsIp, publicDnsIpAna;
		myDns = new ArrayList<ArrayList<String>>();	
		org.xbill.DNS.ResolverConfig DNSConfig = new org.xbill.DNS.ResolverConfig();
		int order = 0;
		String type = "";
	
		for (String n : DNSConfig.servers()) {

			order++;
			publicDnsIp = null;
			publicDnsIpAna = null;
			
			//dnsId = Main.getReporter().makeId("dnsips", "ip", n);
			//dnsId = Statistics.getInstance().getDnsId(n);
						
			if (DasuConfiguration.getInstance().isDoAnaLookup()) {
				 publicDnsIpAna = getPublicDnsIpAna(n, myIp);
				 type = "A";
			}
			else {
				publicDnsIp = getMyPublicDnsIp( n, myIp );
				type = "P";
			}

			if (DEBUG)
				LOGGER.info("DNS: " + n + " " + publicDnsIpAna + " " + order);
			
			if ( (!publicDnsIpAna.equals(n) || (publicDnsIpAna.equals(n) && (!isPrivateIp(n)))) ) {
				
				ReportableFact rf = new ReportableFact("dnsMapping");
				rf.getContentsMap().put("myIp", myIp);
				rf.getContentsMap().put("publicDnsIp", publicDnsIpAna);
				rf.getContentsMap().put("privateDnsIp", n);
				rf.getContentsMap().put("dnsType", "A");
				rf.getContentsMap().put("order", order);
				rf.getContentsMap().put("timeRecorded", Util.currentGMTTime());
				Statistics.getInstance().addReportableFact(rf);
				
				/*
				publicDnsIdAna = Statistics.getInstance().getDnsId(publicDnsIpAna);
				Statistics.addDnsMapping(myIp, publicDnsIdAna, dnsId, "A", order, Util.currentGMTTime());
				//Main.getReporter().insertDnsMapping(Statistics.getPeerId(myIp), publicDnsIdAna, dnsId, "A", order, Util.currentGMTTime());
				 */

				/*
				publicDnsIdAna = Main.getReporter().makeId("dnsips", "ip", publicDnsIpAna);
				if (!Statistics.dnsMap.containsKey(publicDnsIpAna)) {
					publicDnsIdAna = Main.getReporter().makeId("dnsips", "ip", publicDnsIpAna);						
					if (publicDnsIdAna == -1)
						Main.getReporter().insertDnsMapping(Statistics.getInstance().getPeerId(myIp), publicDnsIdAna, dnsId, "A", order);
					
					//pingDnsServers(publicDnsIpAna);
					//traceRouteDnsServers(publicDnsIpAna);				
					Statistics.dnsMap.put(publicDnsIpAna, Statistics.dnsMap.size());
				}
				 */
			}

			else {
				
				if ( isPrivateIp(n)) {
				  
					ReportableFact rf = new ReportableFact("dnsMapping");
					rf.getContentsMap().put("myIp", myIp);
					rf.getContentsMap().put("publicDnsIp", publicDnsIp);
					rf.getContentsMap().put("privateDnsIp", n);
					rf.getContentsMap().put("dnsType", "P");
					rf.getContentsMap().put("order", order);
					rf.getContentsMap().put("timeRecorded", Util.currentGMTTime());
					Statistics.getInstance().addReportableFact(rf);
				  
					/*
					publicDnsId = Statistics.getInstance().getDnsId(publicDnsIp);
					Statistics.addDnsMapping(myIp, publicDnsId, dnsId, "P", order, Util.currentGMTTime());
					//Main.getReporter().insertDnsMapping(Statistics.getInstance().getPeerId(myIp), publicDnsId, dnsId, "P", order, Util.currentGMTTime());
					*/

					/*
					publicDnsId = Main.getReporter().makeId("dnsips", "ip", publicDnsIp);
					if (!Statistics.dnsMap.containsKey(publicDnsIp)) {
						Main.getReporter().insertDnsMapping(Statistics.getInstance().getPeerId(myIp), publicDnsId, dnsId, "P", order);
						if (publicDnsId == -1)
							publicDnsId = Main.getReporter().makeId("dnsips", "ip", publicDnsIp);

						//pingDnsServers(publicDnsIp);
						//traceRouteDnsServers(publicDnsIp);
						Statistics.dnsMap.put(publicDnsIp, Statistics.dnsMap.size());
					}
					*/
			  }
			  		
			}
			
			ArrayList<String> in = new ArrayList<String>();
			in.add(n);
			in.add(type.equals("A") ? publicDnsIpAna : publicDnsIp);
			in.add(type);
			myDns.add(in);			
			
			/*
			if (!Statistics.dnsMap.containsKey(n))
				Statistics.dnsMap.put(n, Statistics.dnsMap.size());
			*/
			
		}
		
		return myDns;
	}
	
	public String getPublicDnsIpAna (String dnsIp, String myIp) {
	
		String returnDnsIp = dnsIp;
		Message msg = doLookup(myIp+".ana-aqualab.cs.northwestern.edu", dnsIp, null, -1);
		
		if (DEBUG)
			LOGGER.info("DNS: " + msg);
		
		if ( msg != null ) {
			Record[] records = msg.getSectionArray(Section.ANSWER);
			if (records.length != 0) {				
				returnDnsIp = records[0].rdataToString();	
			}
		}
		
		return returnDnsIp; 
	}
	
	public String getMyPublicDnsIp (String dnsIp, String myIp) {

		String returnDnsIp = dnsIp;
		
		Message msg = doLookup(myIp, null, "-x", -1);
		if ( msg != null ) {
			
			Record[] records = msg.getSectionArray(Section.ANSWER);
			if (records!=null && records.length != 0) {
				String ipAddressResponse = records[0].rdataToString();
				ipAddressResponse = 
					ipAddressResponse.substring(ipAddressResponse.indexOf("."));
				msg = doLookup(ipAddressResponse.substring(1), null, "-t NS", -1);
				records = msg.getSectionArray(Section.ANSWER);

				if (records.length != 0) {
					returnDnsIp = records[0].rdataToString();
					
					msg = doLookup(records[0].rdataToString(), null, null, -1);
					records = msg.getSectionArray(Section.ANSWER);
					
					if (records.length != 0) {
						returnDnsIp = records[0].rdataToString();
					}
				}
			}
		}
		
		if (!isValidIp(returnDnsIp))
			return dnsIp;
		
		return returnDnsIp;
	}
	
	private boolean isPrivateIp(String ip) {

		 if (ip.startsWith("127.") || ip.startsWith("192.168") || ip.startsWith("10.") || 
				 ip.matches("172.1[6-9].[\\d]{1,3}\\.[\\d]{1,3}") || ip.matches("172.2[0-9].[\\d]{1,3}\\.[\\d]{1,3}") ||
				 ip.matches("172.3[0-1].[\\d]{1,3}\\.[\\d]{1,3}"))
 			 return true;
		 
		 return false;  

	 }   
	
	public void launchDnsTimeout(final String dnsIp)
	{
	  
		if (DEBUG)
			LOGGER.info("DEBUG: DasuDnsModule: Timeout!!!" + timeoutIsRunning);
		
	  if (!timeoutIsRunning.containsKey(dnsIp)) {

		timeoutIsRunning.put(dnsIp, Util.currentGMTTime());
		
	  	Main.createThread("TimeoutDigger-" + dnsIp, new Runnable(){

			public void run() {
				org.xbill.DNS.Name name = null;
				int type = org.xbill.DNS.Type.A;
				int dclass = DClass.IN;
				boolean again = true;
				boolean giveUp = false;

				Message query;
				Record rec;
				SimpleResolver res = null;
				int timeout = DasuConfiguration.getInstance().getDnsTimeout();
				
				long b4 = Util.currentGMTTime();
				int i = 1;
				while (again) {
				  try {
					name = Name.fromString("google.com", Name.root);
					res = new SimpleResolver(dnsIp);
					rec = Record.newRecord(name, type, dclass);
					query = Message.newQuery(rec);
					res.send(query);
					again = false;
				  
				  } catch (IOException e1) {
					  
					  if (DEBUG)
						  LOGGER.info("DEBUG: DasuDnsModule: " + dnsIp + " dns TIMEOUT: try again in " + timeout + " seconds...");
					  
					  if (Util.currentGMTTime() - b4 >= i*5*60*1000) {
						  timeout *= 2;
						  i++;
						  
						  if (DEBUG)
							  LOGGER.info("DEBUG: DasuDnsModule: " + dnsIp + " timeout: " + timeout);
					  }
						  
					  //try again until timeout >= 10 mins, then give up...
					  if ( timeout >= 10*60 ) {
						  
						  if (DEBUG)
							  LOGGER.info("DEBUG: DasuDnsModule: " + dnsIp + " dns TIMEOUT: giving up...");
						  
						  again = false;
						  giveUp = true;
					  }
					  else {
				  
						  try {
								Thread.sleep(timeout*1000);
							  } catch (InterruptedException e) {
								//e.printStackTrace();
							  }
							  again = true;	
					  }
				  }
				  
				} //while
				
				long after = Util.currentGMTTime();
				if (DEBUG) {
					LOGGER.info("DEBUG: DasuDnsModule: " + dnsIp + " dns TIMEOUT: back online...");
					LOGGER.info("DEBUG: DasuDnsModule: " + dnsIp + ": " + Long.toString(after - b4));
				}
				
				timeoutIsRunning.remove(dnsIp);		
				
				FactDnsTimeout ftimeout =  new FactDnsTimeout(dnsIp, b4, after, giveUp, dnsIp);
				reportProbeResult(ftimeout);
				AMCoordinatorV2.assertToEngine(SourceType.RE, ftimeout);
				
			} //run
		     	
	  	 });


	  }	// if

	}
	
	/*
	private void updateLookupResultQueue(FactDnsProbeResult result) {
		
		Long id = System.nanoTime();
		dnsResultTable.put(id, result);
		
		synchronized(AMCoordinatorV2.events) {
			AMCoordinatorV2.events.add(
				new Pair<Long, EventType>(id, EventType.DNS_RESULT));
			AMCoordinatorV2.events.notifyAll();
		}
	}
	*/
	
	private void updateLookupResultQueue(FactDnsProbeResult result) {
		
		Long id = generateId();
		dnsResultTable.put(id, result);
		updateResultQueue(result, result.getProbeType(), id);
		
	}

	
	@Override
	public void stopModule() {
		
		LOGGER.info("STOP: stopping DasuDnsModule...");
		setActive(false);
		synchronized(toDoLookupList) {
			toDoLookupList.notify();
		}
	
	}

	public Pair<String, FactTemplate> proccessResult(long eventId) {
		
		FactDnsProbeResult dnsProbeResult = dnsResultTable.remove(eventId);		
		return new Pair<String, FactTemplate>(dnsProbeResult.getToLookup(), dnsProbeResult);
		
	}


}
