/**
 * Dasu Project
 *
 * File:         AMEngine.java
 * RCS:          $Id: AMEngine.java,v 1.0 Oct 2, 2012 12:02:11 AM mas939 Exp $
 * Description:  AMEngine class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Oct 2, 2012 at 12:02:11 AM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2012, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.measurement.ame;

import java.util.Date;
import java.util.logging.Logger;

import edu.northwestern.dasu.coordination.DistributedCoordinator;
import edu.northwestern.dasu.measurement.tasks.ProbeTask.ProbeSubTask;
import edu.northwestern.dasu.measurement.types.SourceType;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The AMEngine class ...
 */
public class AMEngine {
	
	protected final static Logger LOGGER = Logger.getLogger("edu.northwestern.dasu");
	DistributedCoordinator coordinator;
	String filename;
	String signature;
	Integer taskId;
	SourceType engineId;
	
	public AMEngine(String filename, String signature, Integer taskId, SourceType engine, DistributedCoordinator coordinator) throws Exception {
		
		if ( coordinator == null ) {
			throw new Exception();
		}

		this.filename = filename;
		this.signature = signature;
		this.taskId = taskId;
		this.engineId = engine;
		this.coordinator = coordinator;
		
		this.coordinator = coordinator;		
		this.coordinator.start();
		
		LOGGER.info("AME: " + engineId + " " + taskId + " " + coordinator );
	}
	
	public void stop() {
		
		LOGGER.info("AME: STOP: stopping " + engineId + " " + taskId + " DistributedCoordinator");
		coordinator.stopCoordinator();
	}
	
	public boolean schedulable(ProbeSubTask pst) {
		return coordinator.schedulable(pst);
	}
	
	public String toString() {
		return "AMEngine: " + engineId + "\n" + filename + "\n" + signature + "\n " + taskId + "\n" + coordinator; 
	}
	
	public static void main(String[] args) {
		
		long time = System.currentTimeMillis();
		long time2 = System.currentTimeMillis();
		System.out.println(time + " " + new Date(time));
		System.out.println(time+10 + " " + new Date(time+10));
		System.out.println(time2 + " " + new Date(time2));
	}

	
}
