/**
 * Dasu Project
 *
 * File:         Queue.java
 * RCS:          $Id: Queue.java,v 1.0 Apr 17, 2012 12:22:52 PM mas939 Exp $
 * Description:  Queue class (see below)
 * Author:       John Otto
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Apr 17, 2012 at 12:22:52 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement.scheduler
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2012, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.measurement.scheduler;

import java.util.PriorityQueue;
import java.util.concurrent.PriorityBlockingQueue;
import java.util.logging.Logger;

import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.measurement.tasks.ProbeTask;
import edu.northwestern.dasu.measurement.tasks.ProbeTask.ProbeSubTask;
import edu.northwestern.dasu.util.Pair;

/**
 * @author John Otto &lt;jotto@eecs.northwestern.edu&gt;
 *
 * The Queue class ...
 */
public abstract class Queue {
	// A basic, unlimited queue

	protected PriorityBlockingQueue<Pair<Long, ProbeSubTask>> q = new PriorityBlockingQueue<Pair<Long, ProbeSubTask>>();
	protected final static Logger LOGGER = Logger.getLogger("edu.northwestern.dasu");
	
	public boolean push(ProbeTask pt) {
		
		if ( pt == null)
			return false;
		
		if (!admittable())
			return false;
		
		for (ProbeSubTask pst : pt.getSubTasks()) {

			long t = System.nanoTime();
			if (q.contains(new Pair<Long, ProbeSubTask>(t, pst)))
				if ( Main.DEBUG)
					LOGGER.info("SCHEDULER: ###################\n\n\n\nALREADY IN THE QUEUE OVERRIDEINGG\n\n\n\n##############!!! "
								+ pt + " " + pst);

			q.add(new Pair<Long, ProbeSubTask>(t, pst));
		}
		return true;
	}

	public abstract boolean schedulable();

	public abstract boolean admittable();

	protected abstract void schedule();
	

	public Pair<Long, ProbeSubTask> peek() {
		// if the queue has something schedulable, return it here.
		// (follow the semantics of the priorityqueue -- return the
		// first task, but don't remove it from the queue.)
		if (!schedulable())
			return null;
		return q.peek();
	}

	public Pair<Long, ProbeSubTask> poll() {
		// follow the semantics of the priorityqueue -- remove and
		// return the first task from the queue.
		if (!schedulable())
			return null;
		schedule();
		
		if ( q.size() > 0) {
			return q.poll();			
		}
		
		return null;
	}
	
	
	public int size() {		
		return q.size();
	}

	/**
	 *  
	 * API Extended to allow popped elements to be pushed back
	 * when externally determined to be non-schedulable (external
	 * coordination for instance) 
	 * 
	 **/
		
	public void pollAndPushToSecond() {	
		//pop and push to second position in queue
		Pair<Long, ProbeSubTask> pair = poll();
		if ( pair != null) {	
			
			Pair<Long, ProbeSubTask> next = peek();
			if ( next != null ) {
				long t = next.getKey() + 1;				
				q.add(new Pair<Long, ProbeSubTask>(t, pair.getValue()));
			}
			else {
				long t = System.nanoTime();
				q.add(new Pair<Long, ProbeSubTask>(t, pair.getValue()));
			}
		}
	}
	
	public void pollAndPushToBack() {
		//pop and push to the end of the queue	
		Pair<Long, ProbeSubTask> pair = poll();
		if ( pair != null ) {
			long t = System.nanoTime();
			q.add(new Pair<Long, ProbeSubTask>(t, pair.getValue()));
		}
	}

}