/**
 * Dasu Project
 *
 * File:         QuotaQueue.java
 * RCS:          $Id: QuotaQueue.java,v 1.0 Apr 17, 2012 12:22:52 PM mas939 Exp $
 * Description:  QuotaQueue class (see below)
 * Author:       John Otto
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Apr 17, 2012 at 12:22:52 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement.scheduler
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2012, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.measurement.scheduler;

import java.util.ArrayList;

/**
 * @author John Otto &lt;jotto@eecs.northwestern.edu&gt;
 *
 * The QuotaQueue class ...
 */
public class QuotaQueue extends UnlimitedQueue {
	// Unlimited admission to the queue, but scheduling tasks is
	// rate-limited

	private int maxPerMin;
	private int maxPerHour;

	private ArrayList<Long> hourly = new ArrayList<Long>();
	private ArrayList<Long> minutely = new ArrayList<Long>();

	public QuotaQueue(int maxPerMin, int maxPerHour) {
		this.maxPerMin = maxPerMin;
		this.maxPerHour = maxPerHour;
	}

	@Override
	public boolean schedulable() {
		Long now = System.currentTimeMillis();
		while (hourly.size() > 0
				&& now - hourly.get(0) > 1000 * 60 * 60)
			hourly.remove(0);
		while (minutely.size() > 0 && now - minutely.get(0) > 1000 * 60)
			minutely.remove(0);
		return minutely.size() < maxPerMin
				&& hourly.size() < maxPerHour;
	}

	@Override
	protected void schedule() {
		Long now = System.currentTimeMillis();
		hourly.add(now);
		minutely.add(now);
	}

	@Override
	public boolean admittable() {
		return true;
	}

}