/**
 * Dasu Project
 *
 * File:         ProbeTask.java
 * RCS:          $Id$
 * Description:  ProbeTask class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      May 31, 2010 at 5:23:15 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package edu.northwestern.dasu.measurement.tasks;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Random;

import edu.northwestern.dasu.measurement.types.ProbeType;
import edu.northwestern.dasu.measurement.types.SourceType;
import edu.northwestern.dasu.measurement.types.TaskType;


/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The ProbeTask class ...
 */
public class ProbeTask implements Serializable {

	public static class ProbeSubTask implements Serializable {
				
		public ProbeTask parent;
		public long id = System.nanoTime() + new Random().nextLong() + new Random(1000).nextLong();
		//public long id = new Random().nextLong();
		
		public String ipAddress;
		public SourceType sourceType;
		public Integer taskId;
		public Map params;
		public String target = null;
		
		public ProbeSubTask(String ipAddress, SourceType sourceType, Integer taskId, Map params, String target) {
			this.ipAddress = ipAddress;
			this.sourceType = sourceType;
			this.taskId = taskId;
			this.params = params;
			this.target = target;
		}
		
		public Integer getTaskId() {
			return taskId;
		}
		
		public SourceType getSourceType() {
			return sourceType;
		}
		
		public void setParent(ProbeTask pt) {
			parent = pt;
		}
		
		@Override
		public String toString() {
			
			String s = new String();
			try {
				s = "parentId: " + parent.id + " id: " + id + " parentTaskId: " +  parent.taskId + " taskId: " + taskId + " SourceType: " + sourceType + " Target: " + target + " Dst: " + ipAddress;
			}
			catch(Exception e) {
				s = " id: " + id + " taskId: " + taskId + " SourceType: " + sourceType + " Target: " + target + " Dst: " + ipAddress;
			}
			
			return s;
			
		}
		
		@Override
		public boolean equals(Object obj) {
			
			if (!(obj instanceof ProbeSubTask))
				return false;
			
			ProbeSubTask other = ((ProbeSubTask) obj);
			if ( this.taskId.equals(other.taskId) &&
					this.ipAddress.equals(other.ipAddress) &&
					this.sourceType.equals(other.sourceType) )
				return true;
			
			return false;			
			
		}
		
		@Override
		public int hashCode() {
			
			if ( taskId != null && ipAddress != null && sourceType != null)
				return taskId.hashCode() + ipAddress.hashCode() + sourceType.hashCode();
			else
				return -1;
		
		}

	}
	
	public long id = System.nanoTime();

	public Integer taskId;
	public Integer priority = 0;	/*highest priority*/
	public Long startTime = -1L; 	/*right away*/
	public Long endTime = -1L;		/*right away*/
	public List<String> ipsToProbe = null;
	public ProbeType probeType = null;
	public Map<String, Object> params = null;
	
	public TaskType taskType = TaskType.BULK;
	public SourceType sourceType = 
		SourceType.AME;			/* to redirect output for task by the AMC*/	
	
	public String target = null;
		 
	public ProbeTask(Integer tid, List<String> ipsToProbe, ProbeType probeType) {
		this.taskId = tid;
		this.ipsToProbe = new ArrayList<String>(ipsToProbe);	
		this.probeType = probeType;
	}
	
	public ProbeTask(Integer tid, Integer prior, Long st, Long et, List<String> ipsToProbe, ProbeType probeType) {
		this.taskId = tid;
		this.priority = prior;
		this.startTime = st;
		this.endTime = et;
		this.ipsToProbe = new ArrayList<String>(ipsToProbe);
		this.probeType = probeType;
	}
		
	public ProbeTask(Integer tid, Integer prior, Long st, Long et, List<String> ipsToProbe, ProbeType probeType, TaskType taskType, SourceType sourceType) {
		this.taskId = tid;
		this.priority = prior;
		this.startTime = st;
		this.endTime = et;
		this.ipsToProbe = new ArrayList<String>(ipsToProbe);
		this.probeType = probeType;
		this.taskType = taskType;
		this.sourceType = sourceType;
	}
	
	public ProbeTask(Integer tid, Integer prior, Long st, Long et, List<String> ipsToProbe, ProbeType probeType, TaskType taskType, SourceType sourceType, Map<String, Object> params) {
		this.taskId = tid;
		this.priority = prior;
		this.startTime = st;
		this.endTime = et;
		this.ipsToProbe = new ArrayList<String>(ipsToProbe);
		this.probeType = probeType;
		this.taskType = taskType;
		this.sourceType = sourceType;
		this.params = params;
	}


	@Override
	public String toString() {
		
		return new String("taskId: " + taskId + "\npriority: " + priority + 
				"\nstartTime: " + new Date(startTime) + "\nendTime: " + new Date(endTime) +
				"\nipsToProbe: " + ipsToProbe + "\nProbeType: " + probeType + 
				"\nTaskType: " + taskType + "\nSourceType: " + sourceType + "\nTarget: " + target);
		
	}
	
	public Integer size() {
		if (ipsToProbe != null)
			return ipsToProbe.size();
		else
			return 0;
	}

	@Override
	public boolean equals(Object obj) {

		if (obj == null) {
	        return false;
	    }

		if (!(obj instanceof ProbeTask)) {
	        return false;
	    }
	    
	    return taskId.equals(((ProbeTask) obj).taskId);
	}

	@Override
	public int hashCode()
	{
		return taskId.hashCode();
	}
	
	public void setTarget(String target) {
		this.target = target;
	}
	
	public String getTarget() {
		return target;
	}
	
	public Collection<ProbeSubTask> getSubTasks() {
		
		HashSet<ProbeSubTask> psts = new HashSet<ProbeSubTask>();
		for(String ip: ipsToProbe) {
			ProbeSubTask pst = new ProbeSubTask(ip, sourceType, taskId, params, target);			
			pst.setParent(this);
			psts.add(pst);
		}
		return psts;
	}
}
