/**
 * Dasu Project
 *
 * File:         SandboxedResourceProfiler.java
 * RCS:          $Id: ResourceProfiler.java,v 1.0 Nov 16, 2011 10:59:51 AM mas939 Exp $
 * Description:  SandboxedResourceProfiler class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Nov 16, 2011 at 10:59:51 AM
 * Language:     Java
 * Package:      edu.northwestern.dasu.profilers
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2011, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package edu.northwestern.dasu.sandbox;

import java.lang.management.ManagementFactory;
import java.lang.management.ThreadMXBean;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.LinkedList;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The SandboxedResourceProfiler class ... for now, just to compute cpu usage on per-thread basis.
 */

public class SandboxedResourceProfiler implements Runnable {
	
	int MAX_NUM_TIME_ENTIES_PER_THEAD = 60*2;
	int WINDOW_SIZE = 45;
	private boolean active = false;
	private final static java.util.logging.Logger LOGGER = java.util.logging.Logger.getLogger("edu.northwestern.dasu");
	
	
	private Hashtable<Long, ThreadCpuInfo> threadIdsMap = new Hashtable<Long, ThreadCpuInfo>();
	
	private LinkedList<Float> vuzeCpuUsage = new LinkedList<Float>();
	private LinkedList<Long> timeAxis = new LinkedList<Long>();
	
	private ThreadMXBean threadBean = ManagementFactory.getThreadMXBean();
	private final long profileInterval = 1000;	
	private long prevUpdate = System.currentTimeMillis();
	
	private long startTime = System.nanoTime();	
	private long prevUpTime = startTime;
	private long upTime = 0;
	
	private static boolean DEBUG = true;	

	public SandboxedResourceProfiler() {
		
	}
    
	public void run() {
		
		if (!threadBean.isThreadCpuTimeSupported()) {
			
			if ( DEBUG ) {
				LOGGER.severe("PROFILER: ThreadCpuTime is not supportedd!!");
			}
			return;
		}
			
		active = true;	
		
		try {
	    
			while (isActive()) {
			  								
				Long time = System.currentTimeMillis();
				try {
					updateCurrentCpuUsage(time);
				}
				catch (Exception e) {
					e.printStackTrace();
				}
				
				if ( System.currentTimeMillis() - prevUpdate > 5*1000) {
					
					if (DEBUG)
						debug(time, 5f);
					
					cleanList();
					prevUpdate = System.currentTimeMillis();
				}

				
				//watchdog timer
				
				float averageLoad = 0;
				int windowSize = WINDOW_SIZE;
				for ( int i = vuzeCpuUsage.size()-1; i > 0 && windowSize > 0; i--) {
					averageLoad += vuzeCpuUsage.get(i);
					windowSize--;
				}
				averageLoad = averageLoad/(WINDOW_SIZE-windowSize);

				if ( averageLoad > 50 && windowSize == 0) {
					System.out.println("\n\n\n\n\n" + vuzeCpuUsage + "\n\n\n");
					System.out.println("\n\n\n\n" + averageLoad +"\n\n\n");
					System.out.println("\n\n\n\n\nSUTDOWN JVM\n\n\n");
					System.exit(1);					
				}
				
				Thread.sleep(profileInterval);
				
			}
		  
		} catch (InterruptedException iex) {
			Thread.currentThread().interrupted();
			throw new RuntimeException("Profiler interrupted");
		}
		
	}
	
	private void cleanList() {
		
		HashSet<Long> activeThreads = new HashSet<Long>();		
		for ( long t : threadBean.getAllThreadIds())
			activeThreads.add(t);
		
		threadIdsMap.keySet().retainAll(activeThreads);

		
	}
	
	
	//update the per-thread cpu usage (for all threads)
	private void updateCurrentCpuUsage(long time) {
			
		upTime = getUptime();		
		updateCpuTime(time);

    	long elapsedCpuTimeTotal = 0;
        long elapsedTime = upTime - prevUpTime;

        if (prevUpTime > 0L && upTime > prevUpTime) {
        	
        	for ( Long threadId : threadIdsMap.keySet() ) {
        		
        		elapsedCpuTimeTotal += threadIdsMap.get(threadId).getElapsedCpuTime(time);
        		
        	} 	
        }
        
        if ( vuzeCpuUsage.size() > MAX_NUM_TIME_ENTIES_PER_THEAD) {
        	vuzeCpuUsage.pop();
        	timeAxis.pop();
        }
        
        float totalCpuUsage = ((float)elapsedCpuTimeTotal / ((float)elapsedTime))*100;
        vuzeCpuUsage.add(totalCpuUsage);
        
        prevUpTime = upTime;
	}

//	private HashMap<Long, Thread> getAllThreads() {
//		
//		final ThreadMXBean bean = ManagementFactory.getThreadMXBean( );
//		for ( long id : bean.getAllThreadIds( ) ) {
//			System.out.println(id);
//		}		
//		 
//		HashMap<Long, Thread> ret = new HashMap<Long, Thread>();
//		ThreadGroup rootGroup = Thread.currentThread( ).getThreadGroup( );
//		ThreadGroup parentGroup;
//		while ( ( parentGroup = rootGroup.getParent() ) != null ) {
//			
//			System.out.println("->" + rootGroup + " " + rootGroup.activeCount());
//			Thread[] threads = new Thread[ rootGroup.activeCount() ];
//			while ( rootGroup.enumerate( threads, true ) == threads.length ) {
//			    threads = new Thread[ threads.length * 2 ];
//				for ( Thread t : threads ) {
//					if ( t != null)
//						System.out.println("-->" + t);
//				}
//			}
//		    rootGroup = parentGroup;
//		}
//		System.out.println("->" + rootGroup + " " + rootGroup.activeCount());
//		
//		Thread[] threads = new Thread[ rootGroup.activeCount() ];
//		while ( rootGroup.enumerate( threads, true ) == threads.length ) {
//		    threads = new Thread[ threads.length * 2 ];
//		}
//		
//		for ( Thread t : threads) {
//		
//			if ( t != null) {
//				ret.put(t.getId(), t);
//				System.out.println("-->" + t);
//			}
//		}
//		System.out.println("*******\n");
//		return ret;		
//	}

	private HashMap<Long, Thread> getAllThreads() {
		
		HashMap<Long, Thread> ret = new HashMap<Long, Thread>();
		ThreadGroup rootGroup = Thread.currentThread( ).getThreadGroup( );
		ThreadGroup parentGroup;
		while ( ( parentGroup = rootGroup.getParent() ) != null ) {
		    rootGroup = parentGroup;
		}
		
		Thread[] threads = new Thread[ rootGroup.activeCount() ];
		while ( rootGroup.enumerate( threads, true ) == threads.length ) {
		    threads = new Thread[ threads.length * 2 ];
		}
		
		for ( Thread t : threads) {
			
			if (( t != null) && ( t.isAlive()))
				ret.put(t.getId(), t);
		}
		
		return ret;		
	}
	
	//update the per-thread cpu timing (for all threads)
	private void updateCpuTime(Long time) {
				
		timeAxis.add(time);		
		HashMap<Long, Thread> threads = getAllThreads();
		
		for ( long threadId : threads.keySet() ) {
            
			if (threadId != -1) {
            	    								
            	if (!threadIdsMap.containsKey(threadId)) 
            		threadIdsMap.put(threadId, new ThreadCpuInfo(threadId, threads.get(threadId)));
                
            	threadIdsMap.get(threadId).addCpuTime(threadBean.getThreadCpuTime(threadId), time);
            	//threadIdsMap.get(threadId).addCpuTime(threadBean.getThreadCpuTime(threadId) +
            	//		threadBean.getThreadUserTime(threadId), time);
            	
            }
        }
        
	}
	
	public long getUptime() {
		return System.nanoTime() - startTime;		
	}	
	
	public void stop() {
		active = false;
	}
	
	public boolean isActive() {
		return active;
	}

	
    public void debug(Long time, float pct) {
    	
    	StringBuilder sb = new StringBuilder();    	
    	sb.append("\nPROFILER: jvm " + + vuzeCpuUsage.getLast() + "%\n");
		
		float reaper_cpu = 0;
		int reaper_count = 0;
		
		for ( Long threadId : threadIdsMap.keySet()) {		

			if (threadIdsMap.get(threadId).getCpuUsageList().size() > 0) {
			
				sb.append("PROFILER: *" + threadIdsMap.get(threadId).getThread().getName() + ": " +
						threadIdsMap.get(threadId).getCpuUsageList().getLast() + "\n");

			}
			
		}
		
		sb.append("PROFILER: reaper: " + reaper_cpu + "% count: " + reaper_count + "\n");		
		sb.append("PROFILER: *******\n");
		sb.append(this.getClass().getClassLoader() + "\n");
		LOGGER.info("PROFILER: " + sb);


    }
    
	private class ThreadCpuInfo {
		
		Long threadId;
		LinkedList<Long> cpuTime = new LinkedList<Long>();
		LinkedList<Long> cpuTimeTimeStamp = new LinkedList<Long>();		
		
		LinkedList<Float> cpuUsage = new LinkedList<Float>();
		LinkedList<Long> cpuUsageTimeStamp = new LinkedList<Long>();
		
		//ThreadInfo threadInfo;
		Thread thread;
		
		/*
		public ThreadCpuInfo(long id, ThreadInfo info) {
			this.threadId = id;			
			this.threadInfo = info;			
		}
		*/
		
		public ThreadCpuInfo(long id, Thread t) {
			this.threadId = id;
			this.thread = t;
		}
		
		public void addCpuTime(Long ctime, Long time) {
			
        	if ( cpuTime.size() == MAX_NUM_TIME_ENTIES_PER_THEAD) {
        		cpuTime.pop();
        		Long t = cpuTimeTimeStamp.pop();
        	}

			cpuTime.add(ctime);
			cpuTimeTimeStamp.add(time);
        	
        	updateCpuUsage();

		}
		
		/*
		public ThreadInfo getThreadInfo() {
			return threadInfo;
		}
		*/
		public Thread getThread() {
			return thread;
		}
		
		
		public long getElapsedCpuTime(long timeStamp) {

			if ( cpuTimeTimeStamp.size() > 1 ) {
				
				int index = -1;
				if ( cpuTimeTimeStamp.get(cpuTimeTimeStamp.size()-1).equals(timeStamp))
					index = cpuTimeTimeStamp.size()-1;
				else 
					index = cpuTimeTimeStamp.indexOf(timeStamp);
				
				if ( index != -1)
					return cpuTime.get(index) - cpuTime.get(index-1);
			}
			
			return 0l;		
			
		}
			
		public long getCpuTime(Long timeStamp) {
			
			if ( cpuTimeTimeStamp.size() > 0 ) {
				
				int index = -1;
				if ( cpuTimeTimeStamp.get(cpuTimeTimeStamp.size()-1).equals(timeStamp) )
					index = cpuTimeTimeStamp.size()-1;
				else
					index = cpuTimeTimeStamp.indexOf(timeStamp);
				
				if ( index != -1)
					return cpuTime.get(index);
			}
			
			return 0l;
			
		}
		
		public float getCpuUsage(Long timeStamp) {

			if ( cpuUsageTimeStamp.size() > 0) {
				
				int index = -1;
				if ( cpuUsageTimeStamp.get(cpuUsageTimeStamp.size()-1).equals(timeStamp) )
					index = cpuUsageTimeStamp.size()-1;
				else
					index = cpuUsageTimeStamp.indexOf(timeStamp);
				
				if ( index != -1)
					return cpuUsage.get(index);
			}
			
			return 0l;
			
		}
		
		public LinkedList<Float> getCpuUsageList() {
			return cpuUsage;
		}
		
		public LinkedList<Long> getTimeStamp() {
			return cpuUsageTimeStamp;
		}
		
		private boolean updateCpuUsage() {
						
			long elapsedTime = upTime - prevUpTime;
			
	        if (prevUpTime > 0L && upTime > prevUpTime) {
				
	        	if ( cpuTime.size() > 1) {
					// elapsedCpu is in ns
		    		int lastIndex = cpuTime.size()-1;
		            long elapsedCpuTime = cpuTime.get(lastIndex) - cpuTime.get(lastIndex-1);
		            	
		        	if ( cpuUsage.size() == MAX_NUM_TIME_ENTIES_PER_THEAD) {		        		
		        		cpuUsage.pop();
		        		cpuUsageTimeStamp.pop();
		        		//cpuUsageTimeStampSet.remove(t);
		        	}
	
		            float usage = ((float)elapsedCpuTime / ((float)elapsedTime))*100;	        		
		    		cpuUsage.add(usage);
		    		cpuUsageTimeStamp.add(cpuTimeTimeStamp.get(lastIndex));					
		        
		        	return true;
				}
	        }
	        

			return false;
    		
		}
				
	}
	
	public static void main(String[] args) throws InterruptedException {
		
		SandboxedResourceProfiler sp = new SandboxedResourceProfiler();
		Thread t = new Thread(sp);
		t.start();
		
		while ( t.isAlive() )
			Thread.sleep(1*1000);
	}
}
