package edu.northwestern.dasu.sandbox.applet;

import java.lang.reflect.Method;
import java.rmi.*;
import java.rmi.server.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.logging.Logger;

import edu.northwestern.dasu.drools.FactTemplate;
import edu.northwestern.dasu.drools.RuleBasedEngineInterface;
import edu.northwestern.dasu.measurement.AMCoordinatorV2;
import edu.northwestern.dasu.measurement.tasks.ProbeTask;
import edu.northwestern.dasu.measurement.types.SourceType;

public class RmiAMCoordinatorImpl extends UnicastRemoteObject implements RmiAMCoordinatorInterface, RuleBasedEngineInterface {
    
	private static final long serialVersionUID = 1L;	
	private static Logger LOGGER = Logger.getLogger("edu.northwestern.dasu");
	private HashMap<SourceType, ArrayList<NotifyInterface>> notifyMapSet = null;
	private HashMap<SourceType, ArrayList<FactTemplate>> results = null;

	
	public RmiAMCoordinatorImpl() throws RemoteException { 
    	
    	  	notifyMapSet = new HashMap<SourceType, ArrayList<NotifyInterface>>();
    	  	results = new HashMap<SourceType, ArrayList<FactTemplate>>();
    }
        
	public Object syncMethodCall(String methodName, Object[] args) throws RemoteException {
		
		for ( Method method : AMCoordinatorV2.class.getDeclaredMethods() ) {
			
			//System.out.println(methodName);
			if (method.getName().equals(methodName)) {
				try {
					return method.invoke(AMCoordinatorV2.getInstance(), args);	
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}

		return null;
	}    
    
    public boolean nextProbeTask(NotifyInterface notify) throws RemoteException {
    	    	
    	try {
    		
    		synchronized(notifyMapSet) {
    			if (!notifyMapSet.containsKey(notify.getEngineId())) {
    				notifyMapSet.put(notify.getEngineId(), new ArrayList<NotifyInterface>());    				
    			}
    			notifyMapSet.get(notify.getEngineId()).add(notify);
    			
    		}
    		
    		//System.out.println((ProbeTask)notify.getParameter());
	    	return AMCoordinatorV2.addProbeTask(((ProbeTask)notify.getParameter()));	    		    	
	    	
    	}
    	catch (Exception e) {
    		e.printStackTrace();
    		return false;
    	}
    }
    
    public FactTemplate getResult(NotifyInterface notify) throws RemoteException {
    
    	synchronized(results) {
    		
    		if ( results.containsKey(notify.getEngineId()))    			
    			return results.get(notify.getEngineId()).remove(0);
    	}
    	
    	return null;
    }
    
	public void addFact(FactTemplate factObject, SourceType engineId) {
		
		try {
			synchronized(results) {
				
				if ( !results.containsKey(engineId) )
					results.put(engineId, new ArrayList<FactTemplate>());
				
				results.get(engineId).add(factObject);
			}
			
			synchronized(notifyMapSet) {	
				notifyMapSet.get(engineId).remove(0).doneIt();
			}
		} catch (RemoteException e) {
			//e.printStackTrace();
			LOGGER.info("AME: ERROR: " + e.getLocalizedMessage());
		}
	}

}