/**
 * Dasu Project
 *
 * File:         StartActiveEngine.java
 * RCS:          $Id: StartActiveEngine.java,v 1.0 Jul 16, 2012 12:51:20 PM mas939 Exp $
 * Description:  StartActiveEngine class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Jul 16, 2012 at 12:51:20 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.sandbox
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2012, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.sandbox.applet;

import java.util.logging.Logger;
import edu.northwestern.dasu.drools.ActiveMeasurementEngine;
import edu.northwestern.dasu.drools.RuleBasedEngine;
import edu.northwestern.dasu.measurement.AMCoordinatorV2Interface;
import edu.northwestern.dasu.measurement.types.SourceType;
import edu.northwestern.dasu.sandbox.SandboxedActiveEngineClass;
import edu.northwestern.dasu.sandbox.mainSandboxedProcess;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The SandboxedActiveEngine class ...
 */
public class SandboxedActiveEngine extends SandboxedActiveEngineClass {

	//must be set before starting thread
	public RmiClientAMCoordinatorV2 AMCoordinatorV2 = null;
	public ActiveMeasurementEngine AMEngine = null;
	private static Logger LOGGER = Logger.getLogger("edu.northwestern.dasu");
	
	
	private String filename;
	private String sigFilename;
	private Integer taskId;
	private String engineId;	

	//public SandboxedActiveEngineV2(String filename, String sigFilename, Integer taskId, String engineId) throws Exception {
	public SandboxedActiveEngine(String args[]) throws Exception {
		this.filename = args[0];
		this.sigFilename = args[1];
		this.taskId = Integer.parseInt(args[2]);
		this.engineId = args[3];
		
		AMCoordinatorV2 = new RmiClientAMCoordinatorV2( SourceType.valueOf(engineId) );
		AMEngine = new ActiveMeasurementEngine(filename, sigFilename, taskId, AMCoordinatorV2, engineId);
		AMEngine.setDaemon(true);
		AMEngine.initialize();
		
	}

	
	public void run() {
		
		LOGGER.info("AME: SandboxedActiveEngine.class: File Found... Starting ActiveMeasurementEngine...");
		LOGGER.info("AME: SandboxedActiveEngine.class: " + filename + "\n" + sigFilename + "\ntaskId: " + taskId + "\n");
		
		try {
			//AMEngine.setDaemon(true);
			
			AMEngine.start();
			System.out.println("started: " + AMCoordinatorV2.getClass().getClassLoader());
			System.out.println("started");
			System.out.println("started: " + AMCoordinatorV2.getClass().getClassLoader());
			System.out.println("started");
			while (AMEngine.isAlive()) {
				Thread.sleep(5000);					
				AMEngine.displayFacts();
			}
			
			synchronized(mainSandboxedProcess.notify) {
				mainSandboxedProcess.notify.notify();
			}

			//System.exit(0);
		}
		catch (Exception e) {				
			e.printStackTrace();
			LOGGER.severe(e.getMessage());
			//System.exit(1);
			
		}
	}

	public AMCoordinatorV2Interface getCoordinator() {
		return AMCoordinatorV2;
	}

	public RuleBasedEngine getAMEngine() {
		return AMEngine;
	}
	
	public boolean isAlive() {
		return AMEngine.isAlive();
	}
	
}
