/**
 * Dasu Project
 *
 * File:         EventList.java
 * RCS:          $Id: EventList.java,v 1.0 Jul 29, 2011 5:18:33 PM mas939 Exp $
 * Description:  EventList class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Jul 29, 2011 at 5:18:33 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.simulator
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2011, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.simulator;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The SimulatedEventList class ...
 */

public class SimulatedEventList implements Iterable<Collection<SimulatedEvent>> {

	LinkedList<SimulatedEvent> events;
	
	public SimulatedEventList(Collection<SimulatedEvent> events) {
		this.events  = new LinkedList<SimulatedEvent>();
		this.events.addAll(events);		
	}
	
	public Iterator<Collection<SimulatedEvent>> iterator() {
		return new EventListIterator();
	}
	
	class EventListIterator implements Iterator<Collection<SimulatedEvent>> {

		Iterator<SimulatedEvent> iter = events.iterator();
		SimulatedEvent nextEvent = null;
		
		public EventListIterator() {
			
		}
		
		public boolean hasNext() {
			return nextEvent != null || iter.hasNext();
		}

		public Collection<SimulatedEvent> next() {

			//System.out.println("\n\n\nNEXT\n\n\n\n");
			LinkedList<SimulatedEvent> ret = new LinkedList<SimulatedEvent>();			
			while ( iter.hasNext() || nextEvent != null ) {
				
				SimulatedEvent event;
				
				if (nextEvent != null) {
					event = nextEvent;
					nextEvent = null;
					//System.out.println("**** diff than null! event: " + event);
				}
				else {
					event = iter.next();
				}
				
				//System.out.println("*****" + event);
				
				//group events between consecutive DownloadStatsDynamic of
				//DownloadStatStatic for the same downloadid or whenever the list if finished
				for ( SimulatedEvent e : ret) {
										
					if ( e.type.equals(event.type) && 
							(e.type.equals("DownloadStatsDynamic") || e.type.equals("DownloadStatsSummary")) ) {
						
						if ( e.fields.get("downloadId").equals( event.fields.get("downloadId") ) && 
								e.timeStamp.compareTo(event.timeStamp) != 0 ) {
						
							nextEvent = event;
							//System.out.println("break!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
							break;
						}
					}
				}
				
				if ( nextEvent != null) {
					//System.out.println("break!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
					break;
				}
				
				ret.add(event);
			}
			
			return ret;
		}

		
		public void remove() {
			// TODO Auto-generated method stub
			
		}

	}
	
//    @Override
//    public String toString() {
//    	
//    	return "EventList: < " + events + " >";
//    }
	
}
