/**
 * Dasu Project
 *
 * File:         EventList.java
 * RCS:          $Id: EventList.java,v 1.0 Jul 29, 2011 5:18:33 PM mas939 Exp $
 * Description:  EventList class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Jul 29, 2011 at 5:18:33 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.simulator
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2011, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.simulator;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;

import org.gudy.azureus2.plugins.download.Download;

import edu.northwestern.dasu.stats.DownloadStatsSummary;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The EventList class ...
 */

public class SimulatedStatsList implements Iterable<Collection<Object>> {

	SimulatedEventList eventList;
	
	public SimulatedStatsList(SimulatedEventList eventList) {		
		this.eventList = eventList;
	}
	
	public Iterator<Collection<Object>> iterator() {
		return new StatsListIterator();
	}
	
	class StatsListIterator implements Iterator<Collection<Object>> {

		Iterator<Collection<SimulatedEvent>> iter = eventList.iterator();
		
		public StatsListIterator() {
		}
		
		public boolean hasNext() {
			return iter.hasNext();
		}

		public Collection<Object> next() {

			//System.out.println("\n\n\nNEXT\n\n\n\n");
			LinkedList<Object> ret = new LinkedList<Object>();			
			while ( iter.hasNext() ) {
				
				Object obj;
				Collection<SimulatedEvent> events = iter.next();
				Download[] downloads = InterferenceSimulatorV3.getDownloads(events);
				for ( Download dld : downloads )
					ret.add(dld);
				
				for ( SimulatedEvent event : events) {
					
					System.out.println(event);
					
					if ( event.type.equals("DownloadStatsDynamic") || event.type.equals("DownloadStatsSummary"))
						continue;
					
					obj = InterferenceSimulatorV3.createObject(event);
					if ( obj != null)
						ret.add(obj);
				}
				
				break;
			}
			
			return ret;
		}
		
		public void remove() {
			// TODO Auto-generated method stub
			
		}

	}
	
    @Override
    public String toString() {
    	
    	return "StatsList: < " + eventList + " >";
    }
	
}
