/**
 * Dasu Project
 *
 * File:         StatManager.java
 * RCS:          $Id$
 * Description:  StatManager class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Apr 11, 2007 at 9:42:24 AM
 * Language:     Java
 * Package:      edu.northwestern.news.stats
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2007, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.stats;

import java.util.TimeZone;

import edu.northwestern.dasu.DasuConfiguration;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.drools.ReportableFact;
import edu.northwestern.dasu.timer.ITimer;
import edu.northwestern.dasu.util.Util;

/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The StatManager class runs and controls statistics-gathering code.
 */

public class StatManager {
	
	private final static java.util.logging.Logger LOGGER = java.util.logging.Logger.getLogger("edu.northwestern.dasu");
	private static StatManager self;
	private static ITimer peerStats;
	private static ITimer downloadStats;

	public static StatManager getInstance() {

		if (self == null) {
			self = new StatManager();		
		}
		return self;
	}
	
	public static void registerUsage() {
		
		String myIp = Main.getPublicIpAddress();
		while (myIp==null){
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			myIp = Main.getPublicIpAddress();
		} 
		
		// register use w/ DB, get update interval
		//int myId = Statistics.getInstance().getSelfId();
		
		ReportableFact rf = new ReportableFact("registerUsage");
		rf.getContentsMap().put("GMTTime", Util.currentGMTTime());
		rf.getContentsMap().put("uuid", DasuConfiguration.getInstance().getUUID());
		rf.getContentsMap().put("currentTime", Util.currentTimeMillis());
		rf.getContentsMap().put("timeZone", TimeZone.getDefault().getDisplayName(false, TimeZone.SHORT));
		rf.getContentsMap().put("failed", Util.isFailed());
		Statistics.getInstance().addReportableFact(rf);
		
	}
	
	public static void start() {
		
		String myIp = Main.getPublicIpAddress();
		while (myIp==null){
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			myIp = Main.getPublicIpAddress();
		} // end invalid IP address
		
		// register use w/ DB, get update interval
		int interval = DasuConfiguration.getInstance().getPeerUpdateIntervalSec()*1000;
		
		if (peerStats == null) {
			peerStats = Main.createTimer("PeerStatsTimer");
		}
		PeerStatistics.setRecordingInterval(interval); 
		peerStats.addPeriodicEvent(interval, PeerStatistics.getInstance());
		
		if (downloadStats == null){
			downloadStats = Main.createTimer("DownloadStatsTimer");			
		}
		DownloadStats.setRecordingInterval(interval);
		downloadStats.addPeriodicEvent(interval, DownloadStats.getInstance());
	}
	
	public static void stop(){
		
		LOGGER.info("STOP: Stopping stat manager");
		//flush();
		
		if (peerStats!=null) {
			peerStats.destroy();
			peerStats = null;
		}
		
		if (downloadStats!=null) {
			downloadStats.destroy();
			downloadStats = null;
		}		

		LOGGER.info("STOP: Stopping stat manager: Stopping DownloadStats...");
		if (DownloadStats.getInstance()!=null){
			DownloadStats.getInstance().stop();
		}
		
		LOGGER.info("STOP: Stopping stat manager: Stopping PeerStatistics...");
		if (PeerStatistics.getInstance()!=null){
			PeerStatistics.getInstance().stop();
		}
		
	}
	
	public static void flush(){
		PeerStatistics.getInstance().flush();
		DownloadStats.getInstance().flush();
	}
	
}
