/**
 * Dasu Project
 *
 * File:         Statistics.java
 * RCS:          $Id$
 * Description:  Statistics class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Jun 14, 2006 at 7:51:32 AM
 * Language:     Java
 * Package:      edu.northwestern.news.stats
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2006, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.stats;

import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;

import edu.northwestern.dasu.DasuConfiguration;
import edu.northwestern.dasu.DasuManager;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.database.DBReporterManager;
import edu.northwestern.dasu.drools.FactSpeedTestResult;
import edu.northwestern.dasu.drools.FactTemplate;
import edu.northwestern.dasu.drools.FactUpnpTestResult;
import edu.northwestern.dasu.drools.ReportableFact;
import edu.northwestern.dasu.drools.FactDnsTimeout;
import edu.northwestern.dasu.dynamic.drools.FactHttpProbeResult;
import edu.northwestern.dasu.dynamic.drools.FactNdtProbeResult;
import edu.northwestern.dasu.dynamic.drools.FactPingProbeResult;
import edu.northwestern.dasu.dynamic.drools.FactScrapeProbeResult;
import edu.northwestern.dasu.dynamic.drools.FactTraceResult;
import edu.northwestern.dasu.measurement.TraceEntry;
import edu.northwestern.dasu.measurement.types.TestType;
import edu.northwestern.dasu.timer.ITimer;
import edu.northwestern.dasu.timer.ITimerEvent;
import edu.northwestern.dasu.timer.ITimerEventPerformer;
import edu.northwestern.dasu.util.Util;

/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 * 
 *         The Statistics class manages statistics collected by this plugin.
 */
public class Statistics implements ITimerEventPerformer {
	
	private static Statistics self;
	public final static java.util.logging.Logger LOGGER = java.util.logging.Logger.getLogger("edu.northwestern.dasu");
	
	public Hashtable<String, Integer> peerMap = null;
	public Hashtable<Integer, Integer> peerIds = new Hashtable<Integer, Integer>();;
	public Hashtable<String, Integer> peerClientMap = null;
	public Hashtable<Integer, Integer> peerClientIds = new Hashtable<Integer, Integer>();;
	public Hashtable<String, Integer> routerMap = null;
	public Hashtable<Integer, Integer> routerIds = new Hashtable<Integer, Integer>();;
	public Hashtable<String, Integer> urlMap = null;
	public Hashtable<Integer, Integer> urlIds = new Hashtable<Integer, Integer>();;
	public Hashtable<String, Integer> dnsMap = null;
	public Hashtable<Integer, Integer> dnsIds = new Hashtable<Integer, Integer>();;
	private final long UPDATE_INTERVAL;
	
	public String[] baos = new String[]{"dsdBaos", "dssBaos", "eventBaos", "lpsdBaos", "lpssBaos",
		"pingDasuBaos", "httpDasuBaos", "dnsDasuBaos", "speedTestBaos", "upnpTestBaos", "rstStatBaos",
		"rpcsBaos", "ndtDasuBaos", "ispInfoBaos", "staticUPnPBaos", "interferenceBaos", "eventDasuBaos",
		"interferenceEventBaos", "taskBaos", "userCommentsBaos", "reBaos", "rpsdBaos", "rpssBaos", 
		"teBaos", "tedBaos", "seBaos", "sedBaos", "timeoutDnsBaos"};
	
	public ByteArrayOutputStream dsdBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream dssBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream eventBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream lpsdBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream lpssBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream pingDasuBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream httpDasuBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream dnsDasuBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream speedTestBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream upnpTestBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream rstStatBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream rpcsBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream ndtDasuBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream ispInfoBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream staticUPnPBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream interferenceBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream eventDasuBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream interferenceEventBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream taskBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream userCommentsBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream reBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream rpsdBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream rpssBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream teBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream tedBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream seBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream sedBaos = new ByteArrayOutputStream();
	public ByteArrayOutputStream timeoutDnsBaos = new ByteArrayOutputStream();
	
	private boolean DEBUG = false;
	
	/* this should be in bytes? */
	private int MAX_REPORT_RESULTS = 5; //number of reportablefacts before results are flushed to permanent
	
	//public static final Object sessionRestartMonitor = new Object();	
	//public static Long currentSessionStartTime = System.currentTimeMillis();
	//public static Long loadedSessionStartTime = 0l;

	//access to this object must be synchronized
	public SessionRestartMonitor sessionRestartMonitor = null; 
	
	/** list of all peers */
	private static HashSet<String> peers;		
	/** list of all facts to report **/
	public static HashSet<FactTemplate> reportableFacts;

	private ITimer timer;
	
	private Statistics() {
				
		peers = new HashSet<String>();		
		//reportableFacts = new HashSet<FactTemplate>();		
		
		UPDATE_INTERVAL = DasuConfiguration.getInstance()
				.getDbUpdateFreqSec() * 1000;

		timer = Main.createTimer("DasuStatistics");
		timer.addPeriodicEvent(UPDATE_INTERVAL, this);
		 
		if ( DEBUG )
			MAX_REPORT_RESULTS = 5;

		reportableFacts = DasuManager.getPersistentDataManager().getReportableFacts();
		sessionRestartMonitor = new SessionRestartMonitor(System.currentTimeMillis());
		loadBaos();
	}

	public static synchronized Statistics getInstance() {
		
		if (self == null) {
			self = new Statistics();
		}

		return self;
	}
	
	public void loadBaos() {
		
		for ( String bao : baos ) {
			ByteArrayOutputStream b = new ByteArrayOutputStream();
			try {
				
				byte[] bytes = DasuManager.getPersistentDataManager().getBaos().get(bao);
				if ( bytes != null) {
					b.write( bytes );
					this.getClass().getField(bao).set(this, b);
				}
				
			} catch (Exception e) {
				e.printStackTrace();
			}
			 
		}
	}

	public void addReportableFact( FactTemplate fpr ) {
		
		synchronized(reportableFacts) {
			
			if ( DasuConfiguration.getInstance().isSendToDatabase() ) {
				
				reportableFacts.add(fpr);
				if (DEBUG)
					LOGGER.info("REPORTER: " + fpr.getClass() + " " + reportableFacts.size());
			}
			else
				if (DEBUG)
					LOGGER.info("REPORTER: drop report, data reporting has been disabled by user");
		}
		
		if ( !Main.isShuttingDown() )
			perform(null);
		
	}
	
	/**
	 * Called right before plugin unloads.
	 */
	private void commitReportableFacts() {
		
		HashSet<FactTemplate> temp = new HashSet<FactTemplate>();
		synchronized(reportableFacts) {
			Iterator<FactTemplate> iter = reportableFacts.iterator();
			while ( iter.hasNext()) {
				temp.add(iter.next());
				iter.remove();
			}
		}
		
		if (DEBUG)
			LOGGER.info("REPORTER: commitReportableFacts " + temp.size() );

		if (DEBUG) {
			LOGGER.info("REPORTER: before commitReportableFacts()");
			DBReporterManager.getInstance().printDebug();
		}		
		
		Iterator<FactTemplate> iter = temp.iterator();
		int i = 0;
		while ( iter.hasNext()) {
			
			FactTemplate fpr = iter.next();
			iter.remove();
			try {
				if (DEBUG)
					LOGGER.info("********************REPORTER: " + i++ + " pushGenericReport: " + fpr.getClass());
				Main.getReporter().pushGenericReport(fpr);				
				
				if ( DasuConfiguration.getInstance().isEnablePhPReporter() )
					commitToPhpReporter(fpr);
			}
			catch (Exception e) {
				if (DEBUG)
					LOGGER.info("REPORTER: ERROR: commitReportableFacts(): " + e.getCause());
			}
		}	
		
		if (DEBUG) {
			LOGGER.info("REPORTER: after commitReportableFacta()");
			DBReporterManager.getInstance().printDebug();
		}
			
	} 
	
	private void commitToPhpReporter(FactTemplate fpr) {
		
		if (DEBUG)
			LOGGER.info("REPORTER: commitToPhpReporter: " + fpr.getClass() + " " + fpr.getClass().getSimpleName() + " " +
				fpr.getClass().getClassLoader());
		
			int myId = updateIdMap();
			//Transloader transloader = Transloader.DEFAULT;
			//ObjectWrapper fprObjectWrapped = transloader.wrap(fpr);
			
			if ( fpr.getClass().getSimpleName().equals("ReportableFact") ) {
				
				ReportableFact rf = (ReportableFact) fpr;
				
				if ( rf.getFactType().equals("dnsMapping") ) {						

						int publicDnsId = getDnsId(	(String) rf.getContentsObject("publicDnsIp"));
						int privateDnsId = getDnsId( (String) rf.getContentsObject("privateDnsIp"));
						
						String type = (String) rf.getContentsObject("type");
						int order = (Integer) rf.getContentsObject("order");
						long timeRecorded = (Long) rf.getContentsObject("timeRecorded");
						
						Main.getReporter().insertDnsMapping(myId, publicDnsId, privateDnsId, type, order, timeRecorded);

				}
				
				else if ( rf.getFactType().equals("configEvent") ) { 
					
					String event = (String) rf.getContentsObject("event");
					long timestamp = (Long) rf.getContentsObject("timestamp");
					Main.getReporter().insertConfigEventDasu(myId, event, timestamp);							
					
				}
				
				else if ( rf.getFactType().equals("interferenceEvent") ) {
				
					String type = (String) rf.getContentsObject("type");
					long timestamp = (Long) rf.getContentsObject("timestamp");
					long downloadId = (Long) rf.getContentsObject("downloadId");
					long droppedTo = (Long) rf.getContentsObject("droppedTo");
					float mean = (Float) rf.getContentsObject("mean");					
					Main.getReporter().insertInterferenceEvent(myId, type, timestamp, downloadId, droppedTo, mean);
				
				}
				
				else if ( rf.getFactType().equals("interference") ) {
										
					String type = (String) rf.getContentsObject("type");
					long timestamp = (Long) rf.getContentsObject("timestamp");
					Main.getReporter().insertInterference(myId, type, timestamp);

				}
				
				else if ( rf.getFactType().equals("feedback") ) {
					
					String type = (String) rf.getContentsObject("type");
					String name = (String) rf.getContentsObject("name");
					String email = (String) rf.getContentsObject("email");
					String message = (String) rf.getContentsObject("message");
					String dump = (String) rf.getContentsObject("dump");
					Long gmttime = (Long) rf.getCreationGMTTime();
					
					Main.getReporter().insertFeedback(type, name, email, message, dump, 
							gmttime-Util.dasuTimeOffset, gmttime);
					
				}
				
				else if ( rf.getFactType().equals("userReports") ) {
					
					String dids = (String) rf.getContentsObject("dids");					
					String comment = (String) rf.getContentsObject("comment");
					
					Main.getReporter().insertUserReports(myId, dids, comment);
					
				}
				
				else if ( rf.getFactType().equals("ispInfo") ) {

					String prefix = (String) rf.getContentsObject("prefix");
					String ispname = (String) rf.getContentsObject("ispname");
					Long timeRecorded = (Long) rf.getContentsObject("timeRecorded");
					Main.getReporter().insertIspInfo(myId, prefix, ispname, timeRecorded);
					
				}
				
				else if ( rf.getFactType().equals("ipChange") ) { 
															
					String oldIp = (String) rf.getContentsObject("oldIp");
					String newIp = (String) rf.getContentsObject("newIp");					
					String localIp = (String) rf.getContentsObject("localIp");
					Long timestamp = (Long) rf.getContentsObject("timestamp");
					
					int pub = getPeerId(oldIp);
					int nouvel = getPeerId(newIp);
					int local = getPeerId(localIp);

					Main.getReporter().reportIpChange(pub, nouvel, local, timestamp);

				}
				
				else if ( rf.getFactType().equals("registerUsage") ) {
							
					Long GMTTime = (Long) rf.getContentsObject("GMTTime");
					String uuid = (String) rf.getContentsObject("uuid");
					Long currentTime = (Long) rf.getContentsObject("currentTime");
					String timeZone = (String) rf.getContentsObject("timeZone");
					Boolean failed = (Boolean) rf.getContentsObject("failed");
					
					Main.getReporter().registerUsage(myId, GMTTime, uuid, currentTime,
								timeZone, failed);
					
				}
				
				else if ( rf.getFactType().equals("completedTask") ) {
					
					Long GMTTime = (Long) rf.getContentsObject("GMTTime");
					Integer taskId = (Integer) rf.getContentsObject("taskId");
					
					Main.getReporter().insertTask(myId, GMTTime, taskId);
					
				}
				
				else if ( rf.getFactType().equals("staticUPnPInfo") ) {
					
					HashMap<String, Object> vals = rf.getContentsMap();					
					String UpstreamMaxBitRate = (vals.containsKey("UpstreamMaxBitRate") && vals.get("UpstreamMaxBitRate") != null ? (String) vals.get("UpstreamMaxBitRate") : "");
					String DownstreamMaxBitRate = vals.containsKey("DownstreamMaxBitRate") && vals.get("DownstreamMaxBitRate") != null ? (String) vals.get("DownstreamMaxBitRate") : "";
					String NATEnabled = vals.containsKey("NATEnabled") && vals.get("NATEnabled") != null ? (String) vals.get("NATEnabled") : "";
					String Layer1UpstreamMaxBitRate = vals.containsKey("Layer1UpstreamMaxBitRate") && vals.get("Layer1UpstreamMaxBitRate") != null ? (String) vals.get("Layer1UpstreamMaxBitRate") : "";
					String Layer1DownstreamMaxBitRate = vals.containsKey("Layer1DownstreamMaxBitRate") && vals.get("Layer1DownstreamMaxBitRate") != null ? (String) vals.get("Layer1DownstreamMaxBitRate") : "";
					String PortMappingEnabled = vals.containsKey("PortMappingEnabled") && vals.get("PortMappingEnabled") != null ? (String) vals.get("PortMappingEnabled") : "";
					String WANAccessType = vals.containsKey("WANAccessType") && vals.get("WANAccessType") != null ? (String) vals.get("WANAccessType") : "";
					String MaximumActiveConnections = vals.containsKey("MaximumActiveConnections") && vals.get("MaximumActiveConnections") != null ? (String) vals.get("MaximumActiveConnections") : "";
					String ExternalIPAddress = vals.containsKey("ExternalIPAddress") && vals.get("ExternalIPAddress") != null  ? (String) vals.get("ExternalIPAddress") : "";

					Integer sourceId = -1;
					if ((ExternalIPAddress.length() > 0) && 
							ExternalIPAddress.matches("^[\\d]{1,3}\\.[\\d]{1,3}\\.[\\d]{1,3}\\.[\\d]{1,3}$")) {			
						
						addPeerForLookup(ExternalIPAddress);
						sourceId = peerMap.get(ExternalIPAddress);
						
					}
					
					if(NATEnabled != null){
						NATEnabled = NATEnabled.toLowerCase().equals("true") ? "1" : "0";
					} else {
						NATEnabled = "-1";
					}
					
					if(PortMappingEnabled != null){
						PortMappingEnabled = PortMappingEnabled.toLowerCase().equals("true") ? "1" : "0";
					} else {
						PortMappingEnabled = "-1";
					}
					
					Main.getReporter().insertStaticUPnPInfo(myId, UpstreamMaxBitRate, DownstreamMaxBitRate, NATEnabled,
							Layer1UpstreamMaxBitRate, Layer1DownstreamMaxBitRate, PortMappingEnabled, WANAccessType, MaximumActiveConnections,
							sourceId.toString());

				}		
				
				else {
					
					if (DEBUG) 
						LOGGER.info("REPORTER: Unsupported ReportableFact for PhpReproter: " + rf.getFactType());
				}
			
			}
			
			else if ( fpr.getClass().getSimpleName().equals("RSTStat") ) {
				
				try {
					//RSTStat r = (RSTStat) fprObjectWrapped.cloneWith(getClass().getClassLoader());
					RSTStat r = (RSTStat) fpr;
					addPeerForLookup(r.peerIp);				
					
					int peerId = peerMap.get(r.peerIp);
	
					Main.getReporter().insertRSTStat(peerId, myId, r.downloadId, r.numRst, r.timeRecorded,
						r.type, r.port, r.bitTorrentClient, r.source, boolToInt(r.isSeed), boolToInt(r.amSeed), 
						r.numSeeds, r.numPeers, r.encryption);
				}
				catch (Exception e) {
					
					if (DEBUG)
						LOGGER.info("REPORTER: ERROR: " + e.getMessage());
				}
										
			}
			
			else if ( fpr.getClass().getSimpleName().equals("DownloadStatsSummary") ) {
				
				try {

					DownloadStats.DownloadStatsSummary dss = (DownloadStats.DownloadStatsSummary) fpr;
					
					Main.getReporter().insertDownloadStatSummary(
						myId, dss.downloadId, dss.sumarizeTimeStamp, 
						dss.downloadingTime, dss.seedingTime, dss.maxDownloadRateLimit, dss.maxUploadLimit, 
						dss.totalUploaded, dss.totalDownloaded, dss.size, dss.pieceSize);


				}
				catch (Exception e) {
					if (DEBUG)
						LOGGER.info("REPORTER: ERROR: " + e.getMessage());
				}

			}
			
			else if ( fpr.getClass().getSimpleName().equals("DynamicLocalPeerStat") ) {
				
				try {
 
					DynamicLocalPeerStat dlps = (DynamicLocalPeerStat) fpr;
					
					Main.getReporter().insertLocalPeerStatDynamic(
							myId, 
							dlps.currentTime, dlps.uploadRate, dlps.downloadRate, 
							dlps.maxDownloadRate, dlps.maxUploadRate, dlps.autoSpeed, dlps.rst, 
							dlps.numConnections, dlps.passiveOpens, dlps.activeOpens, dlps.failedConns,
							dlps.numTorrents, dlps.numSeedingTorrents, dlps.numLeechingTorrents, dlps.numConnectedSeeds, dlps.numConnectedLeechers,
							dlps.bytesSentLpsd, dlps.bytesReceivedLpsd, dlps.bytesSentNetstat, dlps.bytesReceivedNetstat, dlps.swarmPeerSpeed,
							dlps.UPnPNumberOfActiveConnections, dlps.UPnPTotalBytesSent, dlps.UPnPTotalBytesReceived, dlps.UPnPTotalPacketsSent,
							dlps.UPnPTotalPacketsReceived, dlps.UPnPUptime);
				}
				catch (Exception e) {
					if (DEBUG)
						LOGGER.info("REPORTER: ERROR: " + e.getMessage());
				}

				
			}
			
			else if ( fpr.getClass().getSimpleName().equals("StaticLocalPeerStat") ) {
								
				try {
					StaticLocalPeerStat lpss = (StaticLocalPeerStat) fpr;
					
					Main.getReporter().insertLocalPeerStatStatic(myId, 			
						lpss.sessionUptimeInSeconds, 
						lpss.maxUploadRate, lpss.maxDownloadRate, lpss.maxPeerConnections, lpss.maxPeerConnectionsPerTorrent, 
						lpss.maxPeerConnectionsPerTorrentSeeding, lpss.maxUploadRateSeeding, lpss.maxSeedsPerTorrent, lpss.lazyBitfield?1:0, 
						lpss.requireEncryption?1:0, lpss.encryptionMinLevel, lpss.allowNonEncryptedOut?1:0, lpss.allowNonEncryptedIn?1:0,
						lpss.useCryptoPortExtension?1:0, lpss.tcpPortIn, lpss.udpPortIn);
				}
				catch (Exception e) {
					if (DEBUG)
						LOGGER.info("REPORTER: ERROR: " + e.getCause());
				}

			}
			
			else if ( fpr.getClass().getSimpleName().equals("RemotePeerStatDynamic") ) {
				
				try {
					
					//RemotePeerStatDynamic rpsd = (RemotePeerStatDynamic) fprObjectWrapped.cloneWith(getClass().getClassLoader());
					RemotePeerStatDynamic rpsd = (RemotePeerStatDynamic) fpr;
					String peerIp = rpsd.peerIp;
					int peerId = this.getPeerId(peerIp);
					
					Main.getReporter().insertRemotePeerStatDynamic(
							myId, 
							peerId, rpsd.currentTime, rpsd.bytesReceivedFromPeer, 
							rpsd.bytesSentToPeer, rpsd.currentReceiveRate, 
							rpsd.currentSendRate, rpsd.isSeed, rpsd.ping, rpsd.isInterested, rpsd.isInteresting);
				
				}
				catch (Exception e) {
					if (DEBUG)
						LOGGER.info("REPORTER: ERROR: " + e.getMessage());
				}

			}
			
			else if ( fpr.getClass().getSimpleName().equals("RemotePeerStatStatic") ) {
				
				try {
					RemotePeerStatStatic sps = (RemotePeerStatStatic) fpr;
					int peerId = this.getPeerId(sps.getPeerIp());
					int peerClientId = this.getPeerClientId(sps.getPeerClientName());
					
					Main.getReporter().insertRemotePeerStatStatic(myId, peerId, 				
							peerClientId, sps.currentLifetime, sps.tcpPort, 
							sps.udpPort, sps.sameCluster, sps.type, sps.encryption, sps.pieceSize);
				}
				catch (Exception e) {
					if (DEBUG)
						LOGGER.info("REPORTER: ERROR: " + e.getMessage() + " " + e.getCause());
				}

				
			}
			
			else if ( fpr.getClass().getSimpleName().equals("DownloadStatsDynamic") ) {
				
				try {
					
					DownloadStats.DownloadStatsDynamic dss = (DownloadStats.DownloadStatsDynamic) fpr;  
					Main.getReporter().insertDownloadStatDynamic(
							myId, dss.downloadId, dss.timeRecorded, dss.numPeers, dss.availability,
							dss.completed, dss.discarded, dss.downloadRate, dss.uploadRate, dss.shareRatio, 
							dss.numKnownNonSeeds, dss.numKnownSeeds, dss.state, dss.priority,
							dss.downloaded, dss.uploaded, dss.position, dss.totalNumKnownNonSeeds, 
							dss.totalNumKnownSeeds, dss.hashFails, dss.connectedSeeds, dss.connectedLeechers, dss.swarmPeerSpeed);

				}
				catch (Exception e) {
					if (DEBUG)
						LOGGER.info("REPORTER: ERROR: " + e.getMessage() + " " + e.getCause());
				}
			}
			
			else if ( fpr.getClass().getSimpleName().equals("FactNdtProbeResult") ) {
				
				try {
					
					//FactNdtProbeResult ndt = (FactNdtProbeResult) fprObjectWrapped.cloneWith(getClass().getClassLoader());
					FactNdtProbeResult ndt = (FactNdtProbeResult) fpr;
					
					Main.getReporter().insertNdtResultDasu(myId,
							ndt.osName, ndt.osArch, ndt.osVer, ndt.interfaceName, ndt.javaVer, 
							ndt.server, ndt.c2sRate, ndt.s2cRate, ndt.mylink, ndt.mismatch, ndt.avgrtt, 
							ndt.packetSize, ndt.connectionStalledTimes, ndt.connectionIdlePercentage, 
							boolToInt(ndt.c2sPacketQueuing), boolToInt(ndt.s2cPacketQueuing), 
							ndt.s2cThroughputPacketQueueingDetectedPercentage, 
							ndt.connectionReceiverLimitedPercentage, ndt.connectionSenderLimitedPercentage,
							ndt.connectionNetworkLimitedPercentage, boolToInt(ndt.RFC2018), boolToInt(ndt.RFC896), 
							boolToInt(ndt.RFC3168), boolToInt(ndt.RFC1323TimeStamping), boolToInt(ndt.RFC1323WindowScaling), 
							ndt.serverScalingFactor, ndt.clientScalingFactor, boolToInt(ndt.serverBehindFirewall), 
							boolToInt(ndt.clientBehindFirewall), ndt.theoreticalNetworkLimit, ndt.serverBuffer, 
							ndt.serverThroughputLimit, ndt.clientBuffer, ndt.clientThroughputLimit, 
							ndt.time, boolToInt(ndt.failed), ndt.errorMsg, ndt.c2sRateNetstat, ndt.s2cRateNetstat, 
							ndt.networkFlowControlThroughputLimit, boolToInt(ndt.congestion), ndt.congestionSignals, ndt.fastRetransmit, ndt.timeouts,
							ndt.outOfOrderPercentage, ndt.pktsRetrans, ndt.dupAcksIn, ndt.sACKsRcvd, ndt.packetLoss);
				}
				catch (Exception e) {
					if (DEBUG)
						LOGGER.info("REPORTER: ERROR: " + e.getMessage());
				}

				
			}
			
			else if ( fpr.getClass().getSimpleName().equals("FactUpnpTestResult") ) {
				
				try {
					//FactUpnpTestResult upnp = (FactUpnpTestResult) fprObjectWrapped.cloneWith(getClass().getClassLoader());
					FactUpnpTestResult upnp = (FactUpnpTestResult) fpr;
					
					Main.getReporter().insertUpnpTestResult(myId,
							upnp.testTime, upnp.port, upnp.protocol, boolToInt(upnp.success));
				
				}
				catch (Exception e) {
					if (DEBUG)
						LOGGER.info("REPORTER: ERROR: " + e.getMessage());
				}

			}
			
			else if ( fpr.getClass().getSimpleName().equals("FactSpeedTestResult") ) {

				try {
					
					//FactSpeedTestResult test = (FactSpeedTestResult) fprObjectWrapped.cloneWith(getClass().getClassLoader());
					FactSpeedTestResult test = (FactSpeedTestResult) fpr;

					Main.getReporter().insertSpeedTestResult(myId,
									test.testTime, test.downloadSpeed, test.uploadSpeed,
									boolToInt(test.encryption), testTypeToString(test.testType), 
									test.downloadSpeedNetstat, test.uploadSpeedNetstat);
				}
				catch (Exception e) {
					if (DEBUG)
						LOGGER.info("REPORTER: ERROR: " + e.getMessage());
				}
	
					
			}
			
			else if ( fpr.getClass().getSimpleName().contains("FactPingProbeResult") ) {

				try {

					//FactPingProbeResult ping = (FactPingProbeResult) fprObjectWrapped.cloneWith(getClass().getClassLoader());					
					FactPingProbeResult ping = (FactPingProbeResult) fpr;
					PingResultDasu p = new PingResultDasu( Main.getPublicIpAddress(), ping.ip,
							ping.rtt, ping.getCreationGMTTime(), ping.destType, ping.pst.taskId);
					
					p.sourceId = myId;
					
					if (p.destType == 1) {
						addPeerForLookup(p.destIp);
						p.destId = peerMap.get(p.destIp);
					}

					Main.getReporter().insertPingResultDasu(
							p.sourceId, p.destId, p.rtt, p.time, p.destType,
							p.taskId);
				}
				catch (Exception e) {
					if (DEBUG)
						LOGGER.info("REPORTER: ERROR: " + e.getCause());
				}

			}
			
			else if ( fpr.getClass().getSimpleName().equals("FactHttpProbeResult") ) {
				
				try {

					//FactHttpProbeResult http = (FactHttpProbeResult) fprObjectWrapped.cloneWith(getClass().getClassLoader());
					FactHttpProbeResult http = (FactHttpProbeResult) fpr;
					String url = http.getUrl();
					addUrl(url);				
					int urlId = urlMap.get(url);
					
					Main.getReporter().insertHttpResultDasu(myId,
						urlId, http.downloadDuration, http.downloadSpeed, http.time,
						http.noBytes, http.taskId);
				}
				catch (Exception e) {
					if (DEBUG)
						LOGGER.info("REPORTER: ERROR: " + e.getMessage());
				}
				
			}
			
			else if ( fpr.getClass().getSimpleName().equals("RemotePeerConnectionStat") ) {
				
				try {
					
					RemotePeerConnectionStat rpcs = (RemotePeerConnectionStat) fpr;
					
					int	peerId = peerMap.get(rpcs.peerIp);
					
					Main.getReporter().insertRemotePeerConnectionStat(peerId, myId, rpcs.timeRecorded,
						rpcs.type, rpcs.port, rpcs.bitTorrentClient, boolToInt(rpcs.isSeed), boolToInt(rpcs.amSeed), 
						rpcs.numSeeds, rpcs.numPeers, rpcs.encryption);
				}
				catch (Exception e) {
					if (DEBUG)
						LOGGER.info("REPORTER: ERROR: " + e.getMessage());
				}
				
			}
			
			else if (fpr.getClass().getSimpleName().equals("FactDnsProbeResult")) {
				
				try {
					
					String server = (String) fpr.getClass().getField("server").get(fpr);
					Long requestTime = (Long) fpr.getClass().getField("requestTime").get(fpr);
					Long responseTime = (Long) fpr.getClass().getField("responseTime").get(fpr);
					String toLookup = (String) fpr.getClass().getField("toLookup").get(fpr);
					String type = (String) fpr.getClass().getField("type").get(fpr);
					String options = (String) fpr.getClass().getField("options").get(fpr);
					Boolean failed = (Boolean) fpr.getClass().getField("failed").get(fpr);					
					Integer taskId = (Integer) fpr.getClass().getField("taskId").get(fpr);
					String simpleResponse = (String) fpr.getClass().getDeclaredMethod("getSimpleResponse", null).invoke(fpr, null);

					if (server == null)
						server = "";

					int dnsId = dnsMap.get(server);

					Main.getReporter().insertDnsResult(myId,
							dnsId, requestTime, responseTime, toLookup,
							type, options, simpleResponse, failed,
							taskId);
					
				}
				catch (Exception e) {
					if (DEBUG)
						LOGGER.info("REPORTER: ERROR: " + e.getMessage());
				}
				
			}
			
			else if ( fpr.getClass().getSimpleName().equals("FactScrapeProbeResult") ) {
				
				try {
					
					//FactScrapeProbeResult scrape = (FactScrapeProbeResult) fprObjectWrapped.cloneWith(getClass().getClassLoader());
					FactScrapeProbeResult scrape = (FactScrapeProbeResult) fpr;
												
					long entryId = Main.getReporter().insertScrapeEntry(
							scrape.type, myId,  scrape.tracker, scrape.hash, scrape.getIps().size(), 
							scrape.numSeeds, scrape.numLeechers, scrape.taskId, scrape.time);
					
					if (entryId > -1) {
						
						for (int i=0; i < scrape.getIps().size(); i++) {
				
							String ip = scrape.getIps().get(i);
							addPeerForLookup(ip);
							int peerid = peerMap.get(ip);
						
							Main.getReporter().insertScrapeEntryData(entryId,
									peerid, scrape.getPorts().get(i), scrape.getUdpPorts().get(i));
								
						}
					} 
				}
				catch (Exception e) {
					if (DEBUG)
						LOGGER.info("REPORTER: ERROR: " + e.getMessage());
				}

				
			}
			
			else if ( fpr.getClass().getSimpleName().equals("FactTraceResult") ) {

				try {
					
					//FactTraceResult tr = (FactTraceResult) fprObjectWrapped.cloneWith(getClass().getClassLoader());				
					FactTraceResult tr = (FactTraceResult) fpr;
					addPeerForLookup(tr.dest);
					addPeerForLookup(tr.source);
					
					for (int i=0; i< tr.entries.size(); i++) {
						TraceEntry te = tr.entries.get(i);
						
						for (int j = 0; j < te.router.length; j++) {
							String router;
	
							if (te.numRouters - 1 < j)
								router = te.router[0];
							else
								router = te.router[j];
	
							addRouterForLookup(router);
						}
					}
	
					int sourceId = myId;
					int destId = peerMap.get(tr.dest);
	
					int lastIndex = tr.entries.size() - 1;
					Float rtts[];
					for (int i = tr.entries.size() - 1; i >= 0; i--) {
						rtts = tr.entries.get(i).rtt;
						if (rtts[0] + rtts[1] + rtts[2] == -3) {
							lastIndex = i;
						} else {
							break;
						}
					}
	
					int taskId = tr.taskId;
					
					// make top-level entry
					long entryId = Main.getReporter().insertTraceEntry(
							sourceId, destId, tr.timestamp, lastIndex, taskId);
	
					// if valid, insert trace route entries
					if (entryId > -1) {
	
						TraceEntry te;
						for (int j = 0; j <= lastIndex; j++) {
							te = tr.entries.get(j);
							int myRouterIds[] = new int[3];
							for (int i = 0; i < te.router.length; i++) {
								String router;
	
								if (te.numRouters - 1 < i) {
									router = te.router[0];
								} else {
									router = te.router[i];
								}
	
								if (!routerMap.containsKey(router)) {
									routerMap.put(router, routerMap.size());
								}
								myRouterIds[i] = routerMap.get(router);
							}
							
							Main.getReporter().insertTraceEntryData(entryId,
									myRouterIds, te);
						} 
					}
					
				}
				catch (Exception e) {
					if (DEBUG)
						LOGGER.info("REPORTER: ERROR: " + e.getMessage());
				}

			}
			
			else if (fpr.getClass().getSimpleName().equals("FactDnsTimeout") ) {

				try {
					
					FactDnsTimeout dns = (FactDnsTimeout) fpr;					
					int myDnsId = getDnsId(dns.dnsIp);
			  
			 		Main.getReporter().insertDnsTimeout(myDnsId, myId, dns.startTime, dns.endTime, boolToInt(dns.gaveUp));
			 		
				}
				catch (Exception e) {
					if (DEBUG)
						LOGGER.info("REPORTER: ERROR: " + e.getMessage());
				}


			}
			else {
				LOGGER.severe("REPORTER: ERROR: unkownClass! " + fpr.getClass() + " " + fpr.getClass().getSimpleName() + " " +
					fpr.getClass().getClassLoader());
			}

		//}
		//
		//else {
		//	if (DEBUG)
		//		LOGGER.info("REPORTER: INFO: cant connect to db, wait for some time");
		//}
 
	}
	
	private boolean addPeerClient(String peerClient) {
		
		synchronized (peerClientMap) {
			if (!peerClientMap.containsKey(peerClient)) {
				peerClientMap.put(peerClient, peerClientMap.size());
			}
		}

		return true;
	}
	
	public boolean addPeerForLookup(String peer) {
				
		synchronized (peerMap) {
			if (!peerMap.containsKey(peer)) {
				peerMap.put(peer, peerMap.size());
			}
		}

		return peers.add(peer);
	}
	
	public boolean addDnsForLookup(String server) {
		
		if (server == null )
			server = "";

		synchronized (dnsMap) {
			if (!dnsMap.containsKey(server)) {
				dnsMap.put(server, dnsMap.size());
			}
		}
		
		return true;
		
	}
	
	private boolean addUrl(String url) {
		
		if ( !DBReporterManager.getInstance().canConnect() )
			return false;

		synchronized (urlMap) {
			if (!urlMap.containsKey(url)) {
				urlMap.put(url, urlMap.size());
			}			
		}
		
		return true;
		
	}

	/**
	 * Called right before plugin unloads.
	 */
	public void flush() {
		
		LOGGER.info("REPORTER: Flushing statistics");
		commitReportableFacts();
		DBReporterManager.getInstance().flush();

	}
	
	public int makeUUIDid(String uuid) {
		
		return DBReporterManager.getInstance().makeUUIDid(uuid);
		
	}

	private char testTypeToString(TestType testType) {
		char ret = 'u';
		if ((testType.compareTo(TestType.AZUREUS_DL) == 0)
				|| (testType.compareTo(TestType.AZUREUS_DL_ENCRYPTION)) == 0)
			ret = 'd';

		return ret;
	}


	public int updateIdMap() {

		String myIp = Main.getPublicIpAddress();

		if (!peerMap.containsKey(myIp))			
			peerMap.put(myIp, peerMap.size());

		return peerMap.get(myIp);
		
	}


	private static int boolToInt(boolean b) {
		if (b)
			return 1;

		return 0;
	}	

	
	public void perform(ITimerEvent event) {
				
		try {
			
			boolean commit = false;
			synchronized(reportableFacts) {					
				if ( reportableFacts.size() > MAX_REPORT_RESULTS ) {			
					commit = true;
				}
			}
			
			if ( commit ) 
				commitReportableFacts();
			
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	public int getPeerId(String ip) {
		
		if (peerMap.get(ip) == null) {
			if (!addPeerForLookup(ip))
				return -1;
		}

		return (peerMap.get(ip));
	}
	
	public int getDnsId(String ip) {
		
		if ( ip == null )
			return -1;
		
		if ( !dnsMap.containsKey(ip) ) {
			if (!addDnsForLookup(ip))
				return -1;
		}

		return dnsMap.get(ip);
	}


	public int getPeerClientId(String client) {
		
		if (client.matches("(.*)\\[(.*)\\]") || client.contains("Unknown")
				|| client.contains("Inconnu") || client.contains("Ukjent")
				|| client.contains("Unbekannt")
				|| client.contains("Sconosciuto")) {
			
			if (client.toLowerCase().contains("exeem"))
				client = "Exeem";
			else
				client = "Unknown";
		}

		if (peerClientMap.get(client) == null) {
			if (!addPeerClient(client))
				return -1;
		}

		return peerClientMap.get(client);
	}

	public void stop() {

		//flush();
		
		if (timer != null) {
			timer.destroy();
		}
		
		
		for ( String bao : baos ) {
			
			try {
				
				DasuManager.getPersistentDataManager().getBaos().put(bao, 
						((ByteArrayOutputStream) this.getClass().getField(bao).get(this)).toByteArray());
				
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
			
	}
		
	/**
	 * Register peer client name for DB lookup
	 * 
	 * @param peerClient
	 */
	protected void addPeerClientForLookup(String peerClient) {
		synchronized (peerClientMap) {
			if (peerClientMap.get(peerClient) == null) {
				peerClientMap.put(peerClient, peerClientMap.size());
			}
		}

	}

	/**
	 * Register peer client name for DB lookup
	 * 
	 * @param peerClient
	 */
	public void addRouterForLookup(String routerIp) {
		synchronized (routerMap) {
			if (routerMap.get(routerIp) == null) {
				routerMap.put(routerIp, routerMap.size());
			}
		}
	}
	
	/** This session monitor is not necessary anymore, no need to swap
	 *  databases for the PhpReporter, simply always use the same dbIdMaps
	 *  between sessions and clear the Maps whenever possible to avoid overflow 
	 *
	 */
	public class SessionRestartMonitor {
		
		private Long loadedSessionStartTime = 0l;
		private Long currentSessionStartTime = 0l;

		public SessionRestartMonitor(long currentTime) {
			
			LOGGER.info("REPORTER: starting session: " + new Date(currentTime));
			currentSessionStartTime = currentTime;
			
			if ( DasuManager.getPersistentDataManager().getDbIdMaps().size() == 0 ) {
				DasuManager.getPersistentDataManager().getDbIdMaps().put(currentTime, new Hashtable<String, Hashtable<String, Integer>>());
				DasuManager.getPersistentDataManager().getDbIdMaps().get(currentTime).put("peerMap", new Hashtable<String, Integer>());
				DasuManager.getPersistentDataManager().getDbIdMaps().get(currentTime).put("routerMap", new Hashtable<String, Integer>());
				DasuManager.getPersistentDataManager().getDbIdMaps().get(currentTime).put("dnsMap", new Hashtable<String, Integer>());
				DasuManager.getPersistentDataManager().getDbIdMaps().get(currentTime).put("peerClientMap", new Hashtable<String, Integer>());
				DasuManager.getPersistentDataManager().getDbIdMaps().get(currentTime).put("urlMap", new Hashtable<String, Integer>());
			
				
			}
			else {
				
				long previousSessionTime = DasuManager.getPersistentDataManager().getDbIdMaps().lastKey();
				DasuManager.getPersistentDataManager().getDbIdMaps().put(currentTime, 
						DasuManager.getPersistentDataManager().getDbIdMaps().get(previousSessionTime));
				DasuManager.getPersistentDataManager().getDbIdMaps().remove(previousSessionTime);
				loadedSessionStartTime = currentTime;
				
				LOGGER.info("REPORTER: swapping session: " + new Date(previousSessionTime));
			}
			
			swapDbIdsToSession(getSessionTimes().get(0));
		}
		
		public Long getLoadedSessionStartTime() {
			return loadedSessionStartTime;
		}
		
		public Long getCurrentSessionStartTime() {
			return currentSessionStartTime;
		}
		
		public List<Long> getSessionTimes() {
			
			List<Long> ret = new ArrayList<Long>();
			ret.addAll(DasuManager.getPersistentDataManager().getDbIdMaps().keySet());
			Collections.sort(ret);
			return ret;
			
		}
		
		private void removeSession(long sessionTime) {
			
			if (DasuManager.getPersistentDataManager().getDbIdMaps().containsKey(sessionTime) &&
					sessionTime != loadedSessionStartTime) {
				DasuManager.getPersistentDataManager().getDbIdMaps().remove(sessionTime);
			}
			else {
				LOGGER.warning("REPORTER: failed! sessionTime does not exist or is the current active session: " + new Date(sessionTime));
			}
			
		}
		
		public synchronized Long swapDbIdsToSessionStartTimeBefore(long thisTime) {
			
			Iterator<Long> iter = getSessionTimes().iterator();
			while ( iter.hasNext() ) {
				
				long session = iter.next();
				if ( session <= thisTime ) {
					swapDbIdsToSession(session);
					return session;
				}
			}
			
			LOGGER.warning("REPORTER: could not find session with start time previous to: " + new Date(thisTime));
			return -1l;
			
		}
		
		public synchronized Long swapToNextSessionAndRemoveCurrent() {
			
			List<Long> sessionTimes = getSessionTimes();
			if ( loadedSessionStartTime < sessionTimes.get(0) ) {
				LOGGER.warning("REPORTER: current session must be the firs session to allow this");
				return -1l;
			}
			
			if ( loadedSessionStartTime.equals(currentSessionStartTime) ) {
				LOGGER.warning("REPORTER: current session is the currentSession");
				return -1l;
			}
			
			//this should never happen
			if ( sessionTimes.size() == 1) {
				LOGGER.warning("REPORTER: current session is the only one!");
				return -1l;
			}

			Long loadedSession = this.loadedSessionStartTime;
			Long newSession = sessionTimes.get(1);
			swapDbIdsToSession(newSession);
			removeSession(loadedSession);
			
			return newSession;
			
		}
		
		private void swapDbIdsToSession(long sessionTime) {
			
			LOGGER.info("REPORTER: swapping session: " + new Date(sessionTime));
			
			if (DasuManager.getPersistentDataManager().getDbIdMaps().containsKey(sessionTime) ) {
			
				peerMap = DasuManager.getPersistentDataManager().getDbIdMaps().get(sessionTime).get("peerMap");
				routerMap = DasuManager.getPersistentDataManager().getDbIdMaps().get(sessionTime).get("routerMap");
				dnsMap = DasuManager.getPersistentDataManager().getDbIdMaps().get(sessionTime).get("dnsMap");
				peerClientMap = DasuManager.getPersistentDataManager().getDbIdMaps().get(sessionTime).get("peerClientMap");
				urlMap = DasuManager.getPersistentDataManager().getDbIdMaps().get(sessionTime).get("urlMap");

				peerIds.clear();
				routerIds.clear();
				urlIds.clear();
				peerClientIds.clear();
				dnsIds.clear();

				loadedSessionStartTime = sessionTime;
								
				StringBuilder sb = new StringBuilder();
				for ( Long s : getSessionTimes() ) {
					sb.append( "[" + new Date(s) + "]");				
				}
				LOGGER.info("\n=================================\n" +
						"REPORTER: existing sessions: " + sb.toString() +
						"\n=================================\n");
			}
			else
				LOGGER.info("REPORTER: could not swap session, sessionTime does not exist: " + new Date(sessionTime));
			
		}
	}
}

