/**
 * Dasu Project
 *
 * File:         DasuView.java
 * RCS:          $Id$
 * Description:  DasuView class (see below)
 * Author:       Zachary Bischof
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Feb 26, 2010 at 12:21:13 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.ui
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package edu.northwestern.dasu.ui;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.BindException;
import java.net.InetSocketAddress;
import java.util.ArrayList; 
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.logging.Logger;

import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.BrowserFunction;
import org.eclipse.swt.browser.CloseWindowListener;
import org.eclipse.swt.browser.OpenWindowListener;
import org.eclipse.swt.browser.TitleEvent;
import org.eclipse.swt.browser.TitleListener;
import org.eclipse.swt.browser.VisibilityWindowListener;
import org.eclipse.swt.browser.WindowEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.gudy.azureus2.ui.swt.Utils;
import org.gudy.azureus2.ui.swt.plugins.UISWTViewEvent;
import org.gudy.azureus2.ui.swt.plugins.UISWTViewEventListener;

import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;
import com.sun.net.httpserver.HttpServer;

import edu.northwestern.dasu.DasuData;
import edu.northwestern.dasu.DasuManager;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.util.json.parser.JSONParser;
import edu.northwestern.dasu.util.json.parser.ParseException;

import edu.northwestern.dasu.drools.FactFireAction;
import edu.northwestern.dasu.measurement.AMCoordinatorV2;
import edu.northwestern.dasu.measurement.types.SourceType;


/*

// How to update the ISP
String prefix = NetworkAdminASNLookupImpl.getInstance().getASNInfo().getBGPPrefix();
DasuManager.getPersistentDataManager().setIspInfo(new Pair(ispName, Util.currentGMTTime()));
Statistics.getInstance().commitIspInfo(prefix,ispName);

// Report undetected interference
Statistics.getInstance().commitUserComments(interferenceReportTorrentIds, interferenceReportComments);

*/

/**
 * @author Zachary Bischof &lt;Zachary.Bischof@eecs.northwestern.edu&gt;
 *
 * The DasuView class ...
 */

public class DasuView implements UISWTViewEventListener {

	public final static Logger LOGGER = Logger.getLogger("edu.northwestern.dasu");
	
	// Constants
	public static final String VIEWID = "Dasu";
	private static final String jsname = "dasu_";
	private static final int basePort = 30000;
	private static final int backlog = 1000;
	// number of connections to queue. since httpserver is
	//single threaded (I think), this should be reasonably high.
	
	// Objects
	public DasuData dd;
	private static DasuView self;
	private Display display;
	private Browser browserUi = null;
	private static ArrayList<Browser> browsers = new ArrayList<Browser>();
	
	// State
	protected boolean starting = false;
	protected boolean failed = false;
	
	private boolean isCreated = false;
	private boolean isLoaded = false;
	private int serverPort = -1;

	private HttpHandler handler;
	HttpServer s = null;
	private HashMap<String, ByteArrayOutputStream> uiFiles = null;
	
	/**
	 * @return Returns value of isLoaded.
	 */
	public boolean isLoaded() {
		return isLoaded;
	}

	public DasuView() {
		self = this;
		
		// Register the UI with DasuData
		dd = DasuData.getInstance();
		dd.setUi(this);
	}
	
	public class PageHandler implements HttpHandler {
		
		public void handle(HttpExchange e) throws IOException {
			// Try to load the file from the UI directory
			
			File f = new File(Main.getDir() + java.io.File.separator + "ui" + e.getRequestURI().getPath());
			
			//System.err.println(e.getRequestURI().getPath());
			
			if(!f.exists())
			{
				e.sendResponseHeaders(404, 0);
				e.getResponseBody().close();
				return;
			}
			
			e.sendResponseHeaders(200, 0);
			
			OutputStream os = e.getResponseBody();
			// Write the page
			FileInputStream fr = new FileInputStream(f);
			
			
			byte[] buf = new byte[1024];
		    int len;
		    while ((len = fr.read(buf)) > 0){
		        os.write(buf, 0, len);
		    }
		    e.close();
		    fr.close();  
		}
		
	}
	
	public class ZipPageHandler implements HttpHandler {
		
		public void handle(HttpExchange e) throws IOException {
			// Try to load the file from the files in the zip archive
			
			String str = e.getRequestURI().getPath().substring(1); // drop the initial slash
			
			if(!uiFiles.containsKey(str)) {
				e.sendResponseHeaders(404, 0);
				e.getResponseBody().close();
				return;	
			}
			
			e.sendResponseHeaders(200, 0);
			
			OutputStream os = e.getResponseBody();
			os.write(uiFiles.get(str).toByteArray());
		    e.close();
		}
		
	}
	
	public int startHttpServer(int port, HttpHandler handler) {
		
		while(true) {
			try {
				s = HttpServer.create(new InetSocketAddress("localhost", port), backlog);
			} catch (BindException e) {
				port += 1;
				continue;
			} catch (IOException e) {
				e.printStackTrace();
			}
			break;
		}
		
		s.createContext("/", handler);
		s.start();
		
		return port;
	}
	
	public static DasuView getInstance() {
		return self;
	}
	
	public String getFullTitle() {
		return "Dasu";
	}

	public String getData() {
		return "Dasu";
	}
	
	public boolean eventOccurred(UISWTViewEvent event) {
		
		switch (event.getType()) {
			case UISWTViewEvent.TYPE_CREATE:
				if (isCreated){
					return false;
				}
				isCreated = true;
				break;

			case UISWTViewEvent.TYPE_DESTROY:
				delete();
				break;

			case UISWTViewEvent.TYPE_INITIALIZE:
				initialize((Composite)event.getData());
				break;

			case UISWTViewEvent.TYPE_LANGUAGEUPDATE:
				if(isLoaded) asyncExecJs("ui_setLocale('" + Locale.getDefault().toString() + "')");
				break;
		}
		return true;
	}

	public void delete() {
		browserUi = null;
		display = null;
		isLoaded = false;
		isCreated = false;
	}
	
	
    static void initialize(final Display display, Browser browser) {
    	
		browser.addOpenWindowListener(new OpenWindowListener(){

            public void open(WindowEvent event) {

				// Certain platforms can provide a default full browser.
	            // simply return in that case if the application prefers
	            // the default full browser to the embedded one set below.
	            if (!event.required) return;

	            // Embed the new window
	            Shell shell = new Shell(display);
	            shell.setText("New Window2"); 
	            shell.setSize(400, 300); 
	            
	            shell.setLayout(new FillLayout());
	            Browser browser2 = new Browser(shell, SWT.NONE);
	    		browser2.setJavascriptEnabled(true);
	            initialize(display, browser2);

	            DasuView.getInstance().new OpenWebLinkFunction(browser2, jsname + "openWebLink");
	            DasuView.getInstance().new FireActionFunction(browser2, jsname + "fireAction");
	            DasuView.getInstance().new UpdateFunction(browser2, jsname + "update");
	            DasuView.getInstance().new ErrorFunction(browser2, jsname + "error");
	            DasuView.getInstance().new LoadedFunction(browser2, jsname + "loaded");

	            event.browser = browser2;
	            synchronized(browsers) {
	            	browsers.add(browser2);
	            }
	            
	            System.out.println("browser: " + browsers.size() + "  " + browsers);
	        	
		}});	
        
        browser.addVisibilityWindowListener(new VisibilityWindowListener() {
                public void hide(WindowEvent event) {
                        Browser browser = (Browser)event.widget;
                        Shell shell = browser.getShell();
                        shell.setVisible(false);
                }
                public void show(WindowEvent event) {
                	
                        Browser browser = (Browser)event.widget;
                        Shell shell = browser.getShell();
                        if (event.location != null) shell.setLocation(event.location);                                                
                        if (event.size != null) {
                                Point size = event.size;
                                shell.setSize(shell.computeSize(size.x, size.y));
                        }
                        
                        if (event.addressBar || event.menuBar || event.statusBar || event.toolBar) {
                                // Create widgets for the address bar, menu bar, status bar and/or tool bar
                                // leave enough space in the Shell to accommodate a Browser of the size
                                // given by event.size                        	
                        	
                        }
                        shell.open();
                }
        });
        
        browser.addCloseWindowListener(new CloseWindowListener() {
                public void close(WindowEvent event) {
                        Browser browser = (Browser)event.widget;
                        Shell shell = browser.getShell();
                        shell.close();
                }
        });
        
		browser.addTitleListener(new TitleListener() {

			public void changed(TitleEvent event) {
				
                Browser browser = (Browser)event.widget;
                Shell shell = browser.getShell();
                shell.setText(event.title);
				
			}
			
		});
				

    }
    	    
    public void addOpenWindowListener(Browser browser, final Display display) {
    	
    	browser.addOpenWindowListener(new OpenWindowListener(){

            public void open(WindowEvent event) {

				// Certain platforms can provide a default full browser.
	            // simply return in that case if the application prefers
	            // the default full browser to the embedded one set below.
	            if (!event.required) return;

	            // Embed the new window
	            Shell shell = new Shell(display);
	            shell.setText("New Window"); 
	            shell.setSize(400, 300); 
	            
	            shell.setLayout(new FillLayout());
	            Browser browser = new Browser(shell, SWT.NONE);
	    		browser.setJavascriptEnabled(true);
	            initialize(display, browser);

	            new OpenWebLinkFunction(browser, jsname + "openWebLink");
	    		new FireActionFunction(browser, jsname + "fireAction");
	    		new UpdateFunction(browser, jsname + "update");
	    		new ErrorFunction(browser, jsname + "error");
	    		new LoadedFunction(browser, jsname + "loaded");

	    		//browser.setSize(200, 200);
	            event.browser = browser;
	            synchronized(browsers) {
	            	browsers.add(browser);
	            }
	        	
		}});	
		browser.addCloseWindowListener(new CloseWindowListener(){

			public void close(WindowEvent event) {
                Browser browser = (Browser)event.widget;
                Shell shell = browser.getShell();
                shell.close();
			}
			
		});
		
    }
    
	public void initialize(Composite composite) {
		
		display = composite.getDisplay();
		browserUi = Utils.createSafeBrowser(composite, SWT.NONE);
		browserUi.setJavascriptEnabled(true);
		
		if ( DasuManager.USE_SIMULATION_V2)
			addOpenWindowListener(browserUi, display);
		
		// Register browser functions
		// http://dev.eclipse.org/viewcvs/viewvc.cgi/org.eclipse.swt.snippets/src/org/eclipse/swt/snippets/Snippet307.java?view=co
		new OpenWebLinkFunction(browserUi, jsname + "openWebLink");
		new FireActionFunction(browserUi, jsname + "fireAction");
		new UpdateFunction(browserUi, jsname + "update");
		new ErrorFunction(browserUi, jsname + "error");
		new LoadedFunction(browserUi, jsname + "loaded");
		
		if( !(starting || failed) && serverPort == -1) {
			starting = true;
			// Only do this once!
			if(Main.SKIP_UPDATE_UI_RULES) {
				LOGGER.severe("OK if debugging, not for release! Not serving UI from ZIP, or doing anything to keep the UI updated!");
				handler = new PageHandler(); // simple version, don't serve from ZIP, update, or do anything...
				serverPort = startHttpServer(basePort, handler);
				starting = false;
				browserUi.setUrl("http://127.0.0.1:" + serverPort + "/index.html");
			} else {
				Thread t = new Thread() {
					public void run() {
						LOGGER.info("UI: Waiting for updater to finish before starting the UI");
						try {
							// Wait for the updater to complete
							while(DasuManager.updater == null || !DasuManager.updater.isReady()) {								
									Thread.sleep(500);
									
									if ( Main.isShuttingDown() )
										interrupt();
							}
						} catch (InterruptedException e) {
							e.printStackTrace();
							LOGGER.severe("UI: DasuView interrupted while waiting for DasuManager.updater to become ready...");
							failed = true;
							starting = false;
							return;
						}
						
						LOGGER.info("UI: Updater finished, UI can be started");
						
						// Load and serve from ZIP file
						uiFiles = DasuManager.updater.getUiData();
						handler = new ZipPageHandler();
						serverPort = startHttpServer(basePort, handler);
						starting = false;
						
						display.syncExec(new Runnable() {							
							public void run() {
								browserUi.setUrl("http://127.0.0.1:" + serverPort + "/index.html");
							}
						});
					}
				};
				t.setName("Dasu::ViewWaitThread");
				t.start();
			}
		} else {
			display.syncExec(new Runnable() {
				public void run() {
					browserUi.setUrl("http://127.0.0.1:" + serverPort + "/index.html");
				}
			});
		}
	}
	
	public boolean syncExecJs(String js) {
		if(!isLoaded || display == null) {
			LOGGER.info("not executing because not ready: " + js);
			return false;
		}
		final String str = js;
		if ( display.isDisposed())
			return false;
		
		display.syncExec(new Runnable() {
			public void run() {
				
				if (( browserUi != null) && (!browserUi.isDisposed()) )
					browserUi.execute(str);
				
				synchronized(browsers) {

					Iterator<Browser> iter = browsers.iterator();
					while (iter.hasNext()) {
					
						Browser b = iter.next();
						try{
							if ( !b.isDisposed() )
								b.execute(str);
						}
						catch (SWTException e) {
							e.printStackTrace();
							iter.remove();
						}
					}
				}				
				
			}
		});
		return true;
	}
	
	public boolean asyncExecJs(String js) {
		if(!isLoaded || display == null) {
			LOGGER.info("not executing because not ready: " + js);
			return false;
		}
		final String str = js;
		
		if ( display.isDisposed())
			return false;
		
		display.asyncExec(new Runnable() {
			public void run() {
				if (( browserUi != null) && (!browserUi.isDisposed()) )
					browserUi.execute(str);

				synchronized(browsers) {

					Iterator<Browser> iter = browsers.iterator();
					while (iter.hasNext()) {
					
						Browser b = iter.next();
						try{
							if ( !b.isDisposed())
								b.execute(str);
						}
						catch (SWTException e) {
							e.printStackTrace();
							iter.remove();
						}
					}
				}
				
			}
		});
		return true;
	}
	
	class LoadedFunction extends BrowserFunction {
		LoadedFunction (Browser browser, String name) {
			super (browser, name);
		}
		public Object function (Object[] arguments) {
			
			LOGGER.info("ui loaded!");
			
			isLoaded = true;
			Locale def = Locale.getDefault();
        	String defStr = "";
        	if(def != null) {
        		defStr = def.toString();
        	}
        	
        	asyncExecJs("ui_setLocale('" + defStr + "')");
        	
            // enable debugging...
            if(Main.DEBUG) {
            	LOGGER.info("enabling debug mode!");
            	asyncExecJs("enableDebug();");
            }

            // flush values stored locally
            DasuData.getInstance().flushToUi();
			
			return true;
		}
	}
	
	class OpenWebLinkFunction extends BrowserFunction {
		OpenWebLinkFunction (Browser browser, String name) {
			super (browser, name);
		}
		public Object function (Object[] arguments) {
			if(arguments.length != 1 || !(arguments[0] instanceof String)) return false;
			Utils.launch((String) arguments[0]);
			return true;
		}
	}	

	class FireActionFunction extends BrowserFunction {
		FireActionFunction (Browser browser, String name) {
			super (browser, name);
		}
		public Object function (Object[] arguments) {
			if(arguments.length != 2) return false;
			if(!(arguments[0] instanceof String)) return false;
			if(!(arguments[1] instanceof String)) return false;
			
			LOGGER.info("fire action type " + arguments[0] + " action " + arguments[1]);
	    	AMCoordinatorV2.assertToEngine(SourceType.RE, new FactFireAction((String) arguments[0], (String) arguments[1]));
			return true;
		}
	}

	class ErrorFunction extends BrowserFunction {
		ErrorFunction (Browser browser, String name) {
			super (browser, name);
		}
		public Object function (Object[] arguments) {
			if(arguments.length != 3) return false;
			
			if ( Main.DEBUG ) {
				LOGGER.severe("Javascript error in [" + arguments[1] + "] on line [" + arguments[2] + "]: " + arguments[0]);
			}
			return true;
		}
	}

	/**
	 * Accept updates to variables, pushed from the UI.
	 */
	class UpdateFunction extends BrowserFunction {
		UpdateFunction (Browser browser, String name) {
			super (browser, name);
		}
		public Object function (Object[] arguments) {
			if(arguments.length != 2) return null;
			if(!(arguments[0] instanceof String)) return null;
			
			String key = (String) arguments[0];
			Object newVal = null;
			try {
				newVal = new JSONParser().parse((String) arguments[1]);
			} catch (ParseException e) {
				e.printStackTrace();
			}
			DasuData.getInstance().updateFromUi(key, newVal);
			
			return true;
		}
	}
	
	public void stopDasuView() {
		
		if ( s != null)
			s.stop(0);
	}
}
